@@ -22,7 +22,7 @@
 
 #include "config.h"
 
-#include <gnome.h>
+#include <string.h>
 
 #include "abook-completion.h"
 #include "information.h"
@@ -34,20 +34,33 @@
  * Create a new CompletionData
  */
 CompletionData *
-completion_data_new(LibBalsaAddress * address, gboolean alias)
+completion_data_new(InternetAddress * ia, const gchar * nick_name)
 {
+    GString *string;
+    gchar *address_string;
+#ifdef CASE_INSENSITIVE_NAME
+    gchar *string_n;
+#endif
     CompletionData *ret;
 
     ret = g_new0(CompletionData, 1);
 
-    g_object_ref(address);
-    ret->address = address;
+    internet_address_ref(ia);
+    ret->ia = ia;
 
+    string = g_string_new(nick_name);
+    if (string->len > 0)
+	g_string_append_c(string, ' ');
+    address_string = internet_address_to_string(ia, FALSE);
+    g_string_append(string, address_string);
+    g_free(address_string);
 #ifdef CASE_INSENSITIVE_NAME
-    ret->string =
-        g_ascii_strup(alias ? address->nick_name : address->full_name, -1);
+    string_n = g_utf8_normalize(string->str, -1, G_NORMALIZE_ALL);
+    g_string_free(string, TRUE);
+    ret->string = g_utf8_casefold(string_n, -1);
+    g_free(string_n);
 #else
-    ret->string = g_strdup(alias ? address->nick_name : address->full_name);
+    ret->string = g_string_free(string, FALSE);
 #endif
 
     return ret;
@@ -59,7 +72,7 @@ completion_data_new(LibBalsaAddress * address, gboolean alias)
 void
 completion_data_free(CompletionData * data)
 {
-    g_object_unref(data->address);
+    internet_address_unref(data->ia);
     g_free(data->string);
     g_free(data);
 }
@@ -81,3 +94,30 @@ address_compare(LibBalsaAddress *a, LibBalsaAddress *b)
 
     return g_ascii_strcasecmp(a->full_name, b->full_name);
 }
+
+/*
+ * A GCompletionStrncmpFunc for matching words instead of the whole
+ * string.
+ *
+ * s1 is the user input, s2 is the target.
+ */
+
+gint
+strncmp_word(const gchar * s1, const gchar * s2, gsize n)
+{
+    const gchar *match;
+    gint retval;
+
+    g_return_val_if_fail(s1 != NULL, -1);
+    g_return_val_if_fail(s2 != NULL, 1);
+
+    match = s2;
+    do {
+	if (!(retval = strncmp(s1, match, n)))
+	    break;
+	if ((match = strchr(match, ' ')))
+	    ++match;
+    } while (match);
+
+    return retval;
+}
