@@ -45,7 +45,7 @@ static const int DEBUG_LDAP = 0;
 /* don't search when prefix has length shorter than LDAP_MIN_LEN */
 static const unsigned LDAP_MIN_LEN=2;
 /* Which parameters do we want back? */
-char* attrs[] = {
+static char* book_attrs[] = {
     "cn",        /* maps to displayed name */
     "mail",      /* maps to itself         */
     "sn",        /* maps to last name      */
@@ -54,6 +54,14 @@ char* attrs[] = {
     "uid",       /* maps to nick name      */
     NULL
 };
+
+static char* complete_attrs[] = {
+    "cn",        /* maps to displayed name */
+    "mail",      /* maps to itself         */
+    "sn",        /* maps to last name      */
+    "givenname", /* maps to first name     */
+    NULL
+};
 /* End of FIXME */
 
 static LibBalsaAddressBookClass *parent_class = NULL;
@@ -229,29 +237,37 @@ static int
 libbalsa_address_book_ldap_open_connection(LibBalsaAddressBookLdap * ab)
 {
     int result;
-    int version = LDAP_VERSION3;
+    static const int version = LDAP_VERSION3;
+    gboolean v3_enabled;
+    LibBalsaAddressBook *lbab = LIBBALSA_ADDRESS_BOOK(ab);
 
     g_return_val_if_fail(ab->host != NULL, FALSE);
 
-    ab->directory = ldap_init(ab->host, LDAP_PORT);
-    if (ab->directory == NULL)
+    ldap_initialize(&ab->directory, ab->host);
+    if (ab->directory == NULL) { /* very unlikely... */
+        libbalsa_address_book_set_status(lbab, g_strdup("Host not found"));
 	return LDAP_SERVER_DOWN;
-
+    }
     /* ignore error if the V3 LDAP cannot be set */
-    if(ldap_set_option(ab->directory, LDAP_OPT_PROTOCOL_VERSION, &version)
-       != LDAP_SUCCESS) ldap_perror(ab->directory, "ldap_set_option");
-
-    if(ab->enable_tls) {
+    v3_enabled = 
+        ldap_set_option(ab->directory, LDAP_OPT_PROTOCOL_VERSION, &version)
+       == LDAP_OPT_SUCCESS;
+    if(!v3_enabled) ldap_perror(ab->directory, "ldap_set_option");
+    if(v3_enabled && ab->enable_tls) {
 #ifdef HAVE_LDAP_TLS
-        /* turn TLS on */
+        /* turn TLS on  but what if we have SSL already on? */
         result = ldap_start_tls_s(ab->directory, NULL, NULL);
         if(result != LDAP_SUCCESS) {
             ldap_unbind(ab->directory);
             ab->directory = NULL;
+            libbalsa_address_book_set_status
+                (lbab, g_strdup(ldap_err2string(result)));
             return result;
         }
 #else /* HAVE_LDAP_TLS */
-	return LDAP_INAPPRIOPRIATE_AUTH;
+     libbalsa_address_book_set_status(lbab,
+                                      _("TLS requested but not compiled in"));
+     return LDAP_INAPPRIOPRIATE_AUTH;
 #endif /* HAVE_LDAP_TLS */
     }
 
@@ -261,6 +277,8 @@ libbalsa_address_book_ldap_open_connection(LibBalsaAddressBookLdap * ab)
                                 ab->passwd);
 
     if (result != LDAP_SUCCESS) {
+        libbalsa_address_book_set_status(lbab,
+                                         g_strdup(ldap_err2string(result)));
 	ldap_unbind_s(ab->directory);
 	ab->directory = NULL;
     }
@@ -296,11 +314,8 @@ libbalsa_address_book_ldap_load(LibBalsaAddressBook * ab,
     for(attempt=0; attempt<2; attempt++) {
         if (ldap_ab->directory == NULL) {
             if ((rc=libbalsa_address_book_ldap_open_connection(ldap_ab))
-                != LDAP_SUCCESS) {
-                libbalsa_address_book_set_status
-                    (ab, g_strdup(ldap_err2string(rc)));
+                != LDAP_SUCCESS)
                 return LBABERR_CANNOT_CONNECT;
-            }
         }
         
         /* 
@@ -317,7 +332,7 @@ libbalsa_address_book_ldap_load(LibBalsaAddressBook * ab,
             : g_strdup("(objectClass=inetOrgPerson)");
         msgid = ldap_search(ldap_ab->directory, ldap_ab->base_dn,
                             LDAP_SCOPE_SUBTREE, 
-                            ldap_filter, attrs, 0);
+                            ldap_filter, book_attrs, 0);
         if (msgid == -1) {
             libbalsa_address_book_ldap_close_connection(ldap_ab);
             continue; /* try again */
@@ -415,6 +430,48 @@ libbalsa_address_book_ldap_get_address(LibBalsaAddressBook * ab,
     return address;
 }
 
+static InternetAddress*
+lbabl_get_internet_address(LDAP *dir, LDAPMessage * e)
+{
+    InternetAddress *ia;
+    BerElement *ber = NULL;
+    char *attr, **vals;
+    int i;
+    gchar *email = NULL, *sn = NULL, *cn = NULL, *first = NULL;
+
+    for (attr = ldap_first_attribute(dir, e, &ber);
+	 attr != NULL; 
+         attr = ldap_next_attribute(dir, e, ber)) {
+	/*
+	 * For each attribute, get the attribute name and values.
+	 */
+	if ((vals = ldap_get_values(dir, e, attr)) != NULL) {
+	    for (i = 0; vals[i] != NULL; i++) {
+		if ((g_ascii_strcasecmp(attr, "sn") == 0) && (!sn))
+		    sn = g_strdup(vals[i]);
+		if ((g_ascii_strcasecmp(attr, "cn") == 0) && (!cn))
+		    cn = g_strdup(vals[i]);
+		if ((g_ascii_strcasecmp(attr, "givenName") == 0) && (!first))
+		    first = g_strdup(vals[i]);
+		if ((g_ascii_strcasecmp(attr, "mail") == 0) && (!email))
+		    email = g_strdup(vals[i]);
+	    }
+	    ldap_value_free(vals);
+	}
+    }
+    /*
+     * Record will have e-mail (searched)
+     */
+    if(email == NULL) email = g_strdup("none");
+    g_return_val_if_fail(email != NULL, NULL);
+
+    if(!cn)
+        cn = create_name(first, sn);
+    ia = internet_address_new_name(cn, email);
+    g_free(email); g_free(sn); g_free(cn); g_free(first);
+
+    return ia;
+}
 /*
  * create_name()
  *
@@ -451,7 +508,7 @@ libbalsa_address_book_ldap_add_address(LibBalsaAddressBook *ab,
     gchar *dn;
     LDAPMod *mods[7];
     LDAPMod modarr[6];
-    int cnt, rc;
+    int cnt;
     char *cn[]   = {NULL, NULL};
     char *gn[]   = {NULL, NULL};
     char *org[]  = {NULL, NULL};
@@ -463,12 +520,8 @@ libbalsa_address_book_ldap_add_address(LibBalsaAddressBook *ab,
     g_return_val_if_fail(address->address_list, LBABERR_CANNOT_WRITE);
 
     if (ldap_ab->directory == NULL) {
-        if( (rc=libbalsa_address_book_ldap_open_connection(ldap_ab))
-	    != LDAP_SUCCESS) {
-	    libbalsa_address_book_set_status(ab,
-					     g_strdup(ldap_err2string(rc)));
+        if(libbalsa_address_book_ldap_open_connection(ldap_ab) != LDAP_SUCCESS)
 	    return LBABERR_CANNOT_CONNECT;
-	}
     }
 
     dn = g_strdup_printf("mail=%s,%s",
@@ -518,8 +571,6 @@ libbalsa_address_book_ldap_add_address(LibBalsaAddressBook *ab,
         if( (rc=libbalsa_address_book_ldap_open_connection(ldap_ab))
 	    != LDAP_SUCCESS) {
 	    g_free(dn);
-	    libbalsa_address_book_set_status(ab,
-					     g_strdup(ldap_err2string(rc)));
 	    return LBABERR_CANNOT_CONNECT;
 	}
         /* fall through */
@@ -546,11 +597,8 @@ libbalsa_address_book_ldap_remove_address(LibBalsaAddressBook *ab,
 
     if (ldap_ab->directory == NULL) {
         if( (rc=libbalsa_address_book_ldap_open_connection(ldap_ab))
-	    != LDAP_SUCCESS) {
-	    libbalsa_address_book_set_status(ab,
-					     g_strdup(ldap_err2string(rc)));
+	    != LDAP_SUCCESS)
 	    return LBABERR_CANNOT_CONNECT;
-	}
     }
 
     dn = g_strdup_printf("mail=%s,%s",
@@ -566,8 +614,6 @@ libbalsa_address_book_ldap_remove_address(LibBalsaAddressBook *ab,
 	    if( (rc=libbalsa_address_book_ldap_open_connection(ldap_ab))
 		!= LDAP_SUCCESS) {
                 g_free(dn);
-		libbalsa_address_book_set_status(ab,
-						 g_strdup(ldap_err2string(rc)));
 		return LBABERR_CANNOT_CONNECT;
 	    }
             /* fall through */
@@ -618,11 +664,8 @@ libbalsa_address_book_ldap_modify_address(LibBalsaAddressBook *ab,
      * attributes. */
     if (ldap_ab->directory == NULL) {
         if( (rc=libbalsa_address_book_ldap_open_connection(ldap_ab))
-	    != LDAP_SUCCESS) {
-	    libbalsa_address_book_set_status(ab,
-					     g_strdup(ldap_err2string(rc)));
+	    != LDAP_SUCCESS)
 	    return LBABERR_CANNOT_CONNECT;
-	}
     }
 
     dn = g_strdup_printf("mail=%s,%s",
@@ -680,8 +723,6 @@ libbalsa_address_book_ldap_modify_address(LibBalsaAddressBook *ab,
 	    if( (rc=libbalsa_address_book_ldap_open_connection(ldap_ab))
 		!= LDAP_SUCCESS) {
 		g_free(dn);
-		libbalsa_address_book_set_status(ab,
-						 g_strdup(ldap_err2string(rc)));
 		return LBABERR_CANNOT_CONNECT;
 	    }
             /* fall through */
@@ -818,7 +859,7 @@ libbalsa_address_book_ldap_alias_complete(LibBalsaAddressBook * ab,
 {
     static struct timeval timeout = { 15, 0 }; /* 15 sec timeout */
     LibBalsaAddressBookLdap *ldap_ab;
-    LibBalsaAddress *addr;
+    InternetAddress *addr;
     GList *res = NULL;
     gchar* filter;
     gchar* ldap;
@@ -832,18 +873,15 @@ libbalsa_address_book_ldap_alias_complete(LibBalsaAddressBook * ab,
     if (!ab->expand_aliases || strlen(prefix)<LDAP_MIN_LEN) return NULL;
     if (ldap_ab->directory == NULL) {
         if( (rc=libbalsa_address_book_ldap_open_connection(ldap_ab))
-	    != LDAP_SUCCESS) {
-	    libbalsa_address_book_set_status(ab,
-					     g_strdup(ldap_err2string(rc)));
+	    != LDAP_SUCCESS)
 	    return NULL;
-	}
     }
 
     /*
      * Attempt to search for e-mail addresses.  It returns success
      * or failure, but not all the matches.
      */
-    *new_prefix = NULL;
+    if(new_prefix) *new_prefix = NULL;
     ldap = rfc_2254_escape(prefix);
 
     filter = g_strdup_printf("(&(mail=*)"
@@ -852,7 +890,7 @@ libbalsa_address_book_ldap_alias_complete(LibBalsaAddressBook * ab,
     g_free(ldap);
     result = NULL;
     rc = ldap_search_st(ldap_ab->directory, ldap_ab->base_dn,
-			LDAP_SCOPE_SUBTREE, filter, attrs, 0, 
+			LDAP_SCOPE_SUBTREE, filter, complete_attrs, 0, 
 			&timeout, &result);
     if(DEBUG_LDAP)
         g_print("Sent LDAP request: %s (basedn=%s) res=0x%x\n", 
@@ -862,9 +900,9 @@ libbalsa_address_book_ldap_alias_complete(LibBalsaAddressBook * ab,
     case LDAP_SUCCESS:
 	for(e = ldap_first_entry(ldap_ab->directory, result);
 	    e != NULL; e = ldap_next_entry(ldap_ab->directory, e)) {
-	    addr = libbalsa_address_book_ldap_get_address(ab, e);
-	    if(!*new_prefix) 
-		*new_prefix = libbalsa_address_to_gchar(addr, 0);
+	    addr = lbabl_get_internet_address(ldap_ab->directory, e);
+	    if(new_prefix && !*new_prefix) 
+		*new_prefix = internet_address_to_string(addr, FALSE);
 	    res = g_list_prepend(res, addr);
 	}
     case LDAP_SIZELIMIT_EXCEEDED:
@@ -890,6 +928,7 @@ libbalsa_address_book_ldap_alias_complete(LibBalsaAddressBook * ab,
     if(result) ldap_msgfree(result);
 
     if(res) res = g_list_reverse(res);
+
     return res;
 }
 #endif				/*LDAP_ENABLED */
