@@ -26,8 +26,6 @@
 
 #include "config.h"
 
-#include <gnome.h>
-
 #include <stdio.h>
 #include <sys/stat.h>
 #include <ctype.h>
@@ -37,6 +35,8 @@
 #include "abook-completion.h"
 #include "address-book-ldif.h"
 #include "information.h"
+#include "libbalsa-conf.h"
+#include "i18n.h"
 
 /* FIXME: Make an option */
 #define CASE_INSENSITIVE_NAME
@@ -150,10 +150,9 @@ libbalsa_address_book_ldif_init(LibBalsaAddressBookLdif * ab)
     ab->address_list = NULL;
     ab->mtime = 0;
 
-    ab->name_complete  = 
-	g_completion_new((GCompletionFunc)completion_data_extract);
-    ab->alias_complete = 
+    ab->name_complete = 
 	g_completion_new((GCompletionFunc)completion_data_extract);
+    g_completion_set_compare(ab->name_complete, strncmp_word);
 }
 
 static void
@@ -165,8 +164,6 @@ ab_ldif_clear(LibBalsaAddressBookLdif *addr_ldif)
     
     g_list_foreach(addr_ldif->name_complete->items, 
 		   (GFunc)completion_data_free, NULL);
-    g_list_foreach(addr_ldif->alias_complete->items, 
-		   (GFunc)completion_data_free, NULL);
 }
 
 static void
@@ -182,8 +179,6 @@ libbalsa_address_book_ldif_finalize(GObject * object)
 
     g_completion_free(addr_ldif->name_complete); 
     addr_ldif->name_complete = NULL;
-    g_completion_free(addr_ldif->alias_complete);
-    addr_ldif->alias_complete = NULL;
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -400,6 +395,20 @@ ldif_address_book_need_reload(LibBalsaAddressBookLdif *ab)
 	return FALSE;
 }
 
+static gboolean
+starts_from(const gchar *str, const gchar *filter_hi)
+{
+    if(!str) return FALSE;
+    while(*str && *filter_hi &&
+          g_unichar_toupper(g_utf8_get_char(str)) == 
+          g_utf8_get_char(filter_hi)) {
+        str       = g_utf8_next_char(str);
+        filter_hi = g_utf8_next_char(filter_hi);
+    }
+           
+    return *filter_hi == '\0';
+}
+
 static LibBalsaABErr
 libbalsa_address_book_ldif_load(LibBalsaAddressBook * ab, 
                                 const gchar *filter,
@@ -407,15 +416,24 @@ libbalsa_address_book_ldif_load(LibBalsaAddressBook * ab,
                                 gpointer closure)
 {
     GList *lst;
+    int len = filter ? strlen(filter) : 0;
+    gchar *filter_hi = NULL;
 
     if(!load_ldif_file(ab)) return LBABERR_CANNOT_READ;
+    if(len)
+        filter_hi = g_utf8_strup(filter, -1);
 
     for (lst = LIBBALSA_ADDRESS_BOOK_LDIF(ab)->address_list; 
 	 lst; lst = g_list_next(lst)) {
-	if (callback)
-	    callback(ab, LIBBALSA_ADDRESS(lst->data), closure);
+        LibBalsaAddress *adr = LIBBALSA_ADDRESS(lst->data);
+        if(callback &&(!len ||
+                       starts_from(adr->last_name, filter_hi) ||
+                       starts_from(adr->full_name, filter_hi) ) )
+	    callback(ab, adr, closure);
     }
     if(callback) callback(ab, NULL, closure);
+    if(len)
+        g_free(filter_hi);
     return LBABERR_OK;
 }
 
@@ -465,7 +483,6 @@ load_ldif_file(LibBalsaAddressBook *ab)
     ab_ldif_clear(addr_ldif);
     
     g_completion_clear_items(addr_ldif->name_complete);
-    g_completion_clear_items(addr_ldif->alias_complete);
     
     if( (gc = fopen(addr_ldif->path, "r")) == NULL)
         return FALSE;
@@ -572,22 +589,32 @@ load_ldif_file(LibBalsaAddressBook *ab)
     addr_ldif->address_list = list;
 
     completion_list = NULL;
-    for (;list; list = g_list_next(list)) {
-	cmp_data = completion_data_new(LIBBALSA_ADDRESS(list->data), FALSE);
+    for (;list; list = list->next) {
+	LibBalsaAddress *address = LIBBALSA_ADDRESS(list->data);
+	InternetAddress *ia;
+
+        if (address->address_list->next) {
+            GList *l;
+
+            ia = internet_address_new_group(address->full_name);
+            for (l = address->address_list; l; l = l->next) {
+                InternetAddress *member =
+                    internet_address_new_name(NULL, l->data);
+                internet_address_add_member(ia, member);
+                internet_address_unref(member);
+            }
+        } else {
+            ia = internet_address_new_name(address->full_name,
+                                           address->address_list->data);
+        }
+	cmp_data = completion_data_new(ia, address->nick_name);
+	internet_address_unref(ia);
 	completion_list = g_list_prepend(completion_list, cmp_data);
     }
     completion_list = g_list_reverse(completion_list);
     g_completion_add_items(addr_ldif->name_complete, completion_list);
     g_list_free(completion_list);
 
-    completion_list = NULL;
-    for(list = addr_ldif->address_list; list; list = g_list_next(list) ) {
-	cmp_data = completion_data_new(LIBBALSA_ADDRESS(list->data), TRUE);
-	completion_list = g_list_prepend(completion_list, cmp_data);
-    }
-    completion_list = g_list_reverse(completion_list);
-    g_completion_add_items(addr_ldif->alias_complete, completion_list);
-    g_list_free(completion_list);
     ab->dist_list_mode = TRUE;
     return TRUE;
 }
@@ -622,9 +649,10 @@ libbalsa_address_book_ldif_add_address(LibBalsaAddressBook * ab,
     LibBalsaAddress *address;
     FILE *fp;
     gchar *value, *value_spec;
+    LibBalsaABErr res = LBABERR_OK;
 
-    if(!load_ldif_file(ab)) 
-        return LBABERR_CANNOT_READ;
+    load_ldif_file(ab); /* Ignore error if any; we may be adding */
+                        /* the first entry in the book. */
     
     
     for(list = LIBBALSA_ADDRESS_BOOK_LDIF(ab)->address_list;
@@ -694,12 +722,13 @@ libbalsa_address_book_ldif_add_address(LibBalsaAddressBook * ab,
     for(list = new_address->address_list; list; list = g_list_next(list)) {
 	if (list->data && *(gchar*)(list->data)) {
 	    value_spec = string_to_value_spec((gchar *) list->data);
-	    fprintf(fp, "mail:%s\n", value_spec);
+	    if(fprintf(fp, "mail:%s\n", value_spec) < 0)
+                res = LBABERR_CANNOT_WRITE;
 	    g_free(value_spec);
 	}
     }
     fclose(fp);
-    return LBABERR_OK;
+    return res;
 }
 
 static LibBalsaABErr
@@ -729,7 +758,7 @@ libbalsa_address_book_ldif_save_config(LibBalsaAddressBook * ab,
 
     vc = LIBBALSA_ADDRESS_BOOK_LDIF(ab);
 
-    gnome_config_set_string("Path", vc->path);
+    libbalsa_conf_set_string("Path", vc->path);
 
     if (LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->save_config)
 	LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->save_config(ab, prefix);
@@ -746,7 +775,7 @@ libbalsa_address_book_ldif_load_config(LibBalsaAddressBook * ab,
     vc = LIBBALSA_ADDRESS_BOOK_LDIF(ab);
 
     g_free(vc->path);
-    vc->path = gnome_config_get_string("Path");
+    vc->path = libbalsa_conf_get_string("Path");
 
     if (LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->load_config)
 	LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->load_config(ab, prefix);
@@ -758,10 +787,8 @@ libbalsa_address_book_ldif_alias_complete(LibBalsaAddressBook * ab,
 					  gchar ** new_prefix)
 {
     LibBalsaAddressBookLdif *vc;
-    GList *resa = NULL, *resb = NULL;
+    GList *list;
     GList *res = NULL;
-    gchar *p1 = NULL, *p2 = NULL;
-    LibBalsaAddress *addr1, *addr2;
 
     g_return_val_if_fail(LIBBALSA_IS_ADDRESS_BOOK_LDIF(ab), NULL);
 
@@ -772,49 +799,13 @@ libbalsa_address_book_ldif_alias_complete(LibBalsaAddressBook * ab,
 
     load_ldif_file(ab);
 
-    resa = g_completion_complete(vc->name_complete, (gchar*)prefix, &p1);
-    resb = g_completion_complete(vc->alias_complete, (gchar*)prefix, &p2);
-
-    if ( p1 && p2 ) {
-	if ( strlen(p1) > strlen(p2) ) {
-	    *new_prefix = p1;
-	    g_free(p2);
-	} else {
-	    *new_prefix = p2;
-	    g_free(p1);
-	}
-    } else {
-	*new_prefix = p1?p1:p2;
+    for (list = g_completion_complete(vc->name_complete, (gchar *) prefix,
+                                      new_prefix);
+         list; list = list->next) {
+	InternetAddress *ia = ((CompletionData *) list->data)->ia;
+	internet_address_ref(ia);
+        res = g_list_prepend(res, ia);
     }
 
-    /*
-      Extract a list of addresses.
-      pick any of them if two addresses point to the same structure.
-      pick addr1 if it is available and there is no addr2
-                    or it is smaller than addr1.
-      in other case, pick addr2 (one of addr1 or addr2 must be not-null).
-    */
-    while ( resa || resb ) {
-	addr1 = resa ? ((CompletionData*)resa->data)->address : NULL;
-	addr2 = resb ? ((CompletionData*)resb->data)->address : NULL;
-
-	if (addr1 == addr2) {
-	    res = g_list_prepend(res, addr1);
-	    g_object_ref(addr1);
-	    resa = g_list_next(resa);
-	    resb = g_list_next(resb);
-	} else if (resa != NULL &&
-		   (resb == NULL || address_compare(addr1, addr2) > 0) ) {
-	    res = g_list_prepend(res, addr1);
-	    g_object_ref(addr1);
-	    resa = g_list_next(resa);
-	} else {
-	    res = g_list_prepend(res, addr2);
-	    g_object_ref(addr2);
-	    resb = g_list_next(resb);
-	}
-    }
-    res = g_list_reverse(res);
-
-    return res;
+    return g_list_reverse(res);
 }
