@@ -30,8 +30,6 @@
 
 #include "config.h"
 
-#include <gnome.h>
-
 #include <stdio.h>
 #include <sys/stat.h>
 #include <string.h>
@@ -39,6 +37,8 @@
 #include "address-book.h"
 #include "address-book-vcard.h"
 #include "abook-completion.h"
+#include "libbalsa-conf.h"
+#include "i18n.h"
 
 /* FIXME: Perhaps the whole thing could be rewritten to use a g_scanner ?? */
 
@@ -146,8 +146,9 @@ libbalsa_address_book_vcard_init(LibBalsaAddressBookVcard * ab)
     ab->address_list = NULL;
     ab->mtime = 0;
 
-    ab->name_complete  = g_completion_new((GCompletionFunc)completion_data_extract);
-    ab->alias_complete = g_completion_new((GCompletionFunc)completion_data_extract);
+    ab->name_complete =
+	g_completion_new((GCompletionFunc)completion_data_extract);
+    g_completion_set_compare(ab->name_complete, strncmp_word);
 }
 
 static void
@@ -164,10 +165,7 @@ libbalsa_address_book_vcard_finalize(GObject * object)
     addr_vcard->address_list = NULL;
 
     g_list_foreach(addr_vcard->name_complete->items, (GFunc)completion_data_free, NULL);
-    g_list_foreach(addr_vcard->alias_complete->items, (GFunc)completion_data_free, NULL);
-    
     g_completion_free(addr_vcard->name_complete);
-    g_completion_free(addr_vcard->alias_complete);
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -207,6 +205,21 @@ vcard_address_book_need_reload(LibBalsaAddressBookVcard *ab)
     }
 }
 
+static gboolean
+starts_from(const gchar *str, const gchar *filter_hi)
+{
+    if(!str) return FALSE;
+    while(*str && *filter_hi &&
+          g_unichar_toupper(g_utf8_get_char(str)) == 
+          g_utf8_get_char(filter_hi)) {
+        str       = g_utf8_next_char(str);
+        filter_hi = g_utf8_next_char(filter_hi);
+    }
+           
+    return *filter_hi == '\0';
+}
+ 
+
 static LibBalsaABErr
 libbalsa_address_book_vcard_load(LibBalsaAddressBook * ab,
                                  const gchar *filter,
@@ -214,19 +227,84 @@ libbalsa_address_book_vcard_load(LibBalsaAddressBook * ab,
                                  gpointer closure)
 {
     GList *lst;
+    int len = filter ? strlen(filter) : 0;
+    gchar *filter_hi = NULL;
 
     if(!load_vcard_file(ab)) return LBABERR_CANNOT_READ;
+    if(len)
+        filter_hi = g_utf8_strup(filter, -1);
 
-    lst = LIBBALSA_ADDRESS_BOOK_VCARD(ab)->address_list;
-    while (lst) {
-	if ( callback ) 
-	    callback(ab, LIBBALSA_ADDRESS(lst->data), closure);
-	lst = g_list_next(lst);
+    for(lst = LIBBALSA_ADDRESS_BOOK_VCARD(ab)->address_list;
+        lst; 
+        lst = g_list_next(lst) ) {
+        LibBalsaAddress *adr = LIBBALSA_ADDRESS(lst->data);
+        if(callback &&(!len ||
+                       starts_from(adr->last_name, filter_hi) ||
+                       starts_from(adr->full_name, filter_hi) ) )
+	    callback(ab, adr, closure);
     }
     if(callback) callback(ab, NULL, closure);
+    if(len)
+        g_free(filter_hi);
     return LBABERR_OK;
 }
 
+static gchar *
+validate_vcard_string(gchar * vcstr)
+{
+    gchar * utf8res;
+    gsize b_written;
+
+    /* check if it's a utf8 clean string and return it in this case */
+    if (g_utf8_validate(vcstr, -1, NULL))
+	return vcstr;
+
+    /* try to convert from the user's locale setting */
+    utf8res = g_locale_to_utf8(vcstr, -1, NULL, &b_written, NULL);
+    if (!utf8res)
+	return vcstr;
+
+    g_free(vcstr);
+    return utf8res;
+}
+
+/* To create distribution lists based on the ORG field:
+ * #define MAKE_GROUP_BY_ORGANIZATION TRUE
+ */
+#if MAKE_GROUP_BY_ORGANIZATION
+static void
+group_address(const gchar * group_name, GSList * group_addresses,
+              GList ** completion_list)
+{
+    GSList *l;
+    CompletionData *cmp_data;
+    InternetAddress *ia;
+
+    if (!group_addresses || !group_addresses->next)
+        return;
+
+    ia = internet_address_new_group(group_name);
+    for (l = group_addresses; l; l = l->next) {
+        GList *mailbox;
+        LibBalsaAddress *address = LIBBALSA_ADDRESS(l->data);
+
+        for (mailbox = address->address_list; mailbox;
+             mailbox = mailbox->next) {
+            InternetAddress *member =
+                internet_address_new_name(address->full_name,
+                                          mailbox->data);
+            internet_address_add_member(ia, member);
+            internet_address_unref(member);
+        }
+    }
+    g_slist_free(group_addresses);
+
+    cmp_data = completion_data_new(ia, NULL);
+    internet_address_unref(ia);
+    *completion_list = g_list_prepend(*completion_list, cmp_data);
+}
+#endif                          /* MAKE_GROUP_BY_ORGANIZATION */
+
 /* FIXME: Could stat the file to see if it has changed since last time
    we read it */
 static gboolean
@@ -234,12 +312,15 @@ load_vcard_file(LibBalsaAddressBook *ab)
 {
     FILE *gc;
     gchar string[LINE_LEN];
-    gchar *name = NULL, *id = NULL;
+    gchar *name = NULL, *id = NULL, *org = NULL;
     gint in_vcard = FALSE;
     GList *list = NULL;
     GList *completion_list = NULL;
     GList *address_list = NULL;
     CompletionData *cmp_data;
+#if MAKE_GROUP_BY_ORGANIZATION
+    GHashTable *group_table;
+#endif                          /* MAKE_GROUP_BY_ORGANIZATION */
 
     LibBalsaAddressBookVcard *addr_vcard;
 
@@ -254,11 +335,7 @@ load_vcard_file(LibBalsaAddressBook *ab)
 
     g_list_foreach(addr_vcard->name_complete->items,
                    (GFunc)completion_data_free, NULL);
-    g_list_foreach(addr_vcard->alias_complete->items,
-                   (GFunc)completion_data_free, NULL);
-
     g_completion_clear_items(addr_vcard->name_complete);
-    g_completion_clear_items(addr_vcard->alias_complete);
 
     if( (gc = fopen(addr_vcard->path, "r")) == NULL)
 	return FALSE;
@@ -284,33 +361,27 @@ load_vcard_file(LibBalsaAddressBook *ab)
 
 		address->address_list = g_list_reverse(address_list);
 
-		if (name) {
-                    gint bread, bwritten;
-                    GError *err = NULL;
-                    gchar *cved = g_locale_to_utf8(name, -1, 
-                                                   &bread, &bwritten, &err);
-                    if(err) g_error_free(err); /* ignore errors */
-		    if(cved) {
-                        address->full_name = cved; g_free(name);
-                    } else
-                        address->full_name = name; /* conv error */
-                        
-                } else if (id)
+		if (name)
+		    address->full_name = name;
+                else if (id)
 		    address->full_name = g_strdup(id);
 		else
 		    address->full_name = g_strdup(_("No-Name"));
 
-
 		/* FIXME: Split into Firstname, Middlename and Lastname... */
 
+		address->organization = org;
+
 		list = g_list_prepend(list, address);
 		address_list = NULL;
 	    } else {		/* record without e-mail address, ignore */
 		g_free(name);
 		g_free(id);
+		g_free(org);
 	    }
 	    name = NULL;
 	    id = NULL;
+	    org = NULL;
 	    in_vcard = FALSE;
 	    continue;
 	}
@@ -322,11 +393,19 @@ load_vcard_file(LibBalsaAddressBook *ab)
 
 	if (g_ascii_strncasecmp(string, "FN:", 3) == 0) {
 	    id = g_strdup(string + 3);
+	    id = validate_vcard_string(id);
 	    continue;
 	}
 
 	if (g_ascii_strncasecmp(string, "N:", 2) == 0) {
 	    name = extract_name(string + 2);
+	    name = validate_vcard_string(name);
+	    continue;
+	}
+
+	if (g_ascii_strncasecmp(string, "ORG:", 4) == 0) {
+	    org = g_strdup(string + 4);
+	    org = validate_vcard_string(org);
 	    continue;
 	}
 
@@ -350,25 +429,73 @@ load_vcard_file(LibBalsaAddressBook *ab)
     addr_vcard->address_list = list;
 
     completion_list = NULL;
-    while ( list ) {
-	cmp_data = completion_data_new(LIBBALSA_ADDRESS(list->data), FALSE);
-	completion_list = g_list_prepend(completion_list, cmp_data);
-	list = g_list_next(list);
+#if MAKE_GROUP_BY_ORGANIZATION
+    group_table =
+	g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+#endif                          /* MAKE_GROUP_BY_ORGANIZATION */
+    for (;list; list = list->next) {
+	LibBalsaAddress *address = list->data;
+#if MAKE_GROUP_BY_ORGANIZATION
+	gchar **groups, **group;
+#endif                          /* MAKE_GROUP_BY_ORGANIZATION */
+	GList *l;
+
+	if (address->address_list->next && ab->dist_list_mode) {
+	    /* Create a group address. */
+	    InternetAddress *ia =
+		internet_address_new_group(address->full_name);
+
+	    for (l = address->address_list; l; l = l->next) {
+		InternetAddress *member =
+		    internet_address_new_name(NULL, l->data);
+		internet_address_add_member(ia, member);
+		internet_address_unref(member);
+	    }
+	    cmp_data = completion_data_new(ia, address->nick_name);
+	    completion_list = g_list_prepend(completion_list, cmp_data);
+	    internet_address_unref(ia);
+	} else {
+	    /* Create name addresses. */
+	    GList *l;
+
+	    for (l = address->address_list; l; l = l->next) {
+		InternetAddress *ia =
+		    internet_address_new_name(address->full_name, l->data);
+		cmp_data = completion_data_new(ia, address->nick_name);
+		completion_list = g_list_prepend(completion_list, cmp_data);
+		internet_address_unref(ia);
+	    }
+	}
+
+#if MAKE_GROUP_BY_ORGANIZATION
+	if (!address->organization || !*address->organization)
+	    continue;
+	groups = g_strsplit(address->organization, ";", 0);
+	for (group = groups; *group; group++) {
+	    gchar *group_name;
+	    GSList *group_addresses;
+
+	    g_strstrip(*group);
+            group_name = group == groups ? g_strdup(*group) :
+                g_strconcat(*groups, " ", *group, NULL);
+	    group_addresses = g_hash_table_lookup(group_table, group_name);
+	    group_addresses = g_slist_prepend(group_addresses, address);
+	    g_hash_table_replace(group_table, group_name, group_addresses);
+	}
+	g_strfreev(groups);
+#endif                          /* MAKE_GROUP_BY_ORGANIZATION */
     }
+
+#if MAKE_GROUP_BY_ORGANIZATION
+    g_hash_table_foreach(group_table, (GHFunc) group_address,
+	                 &completion_list);
+    g_hash_table_destroy(group_table);
+#endif                          /* MAKE_GROUP_BY_ORGANIZATION */
+
     completion_list = g_list_reverse(completion_list);
     g_completion_add_items(addr_vcard->name_complete, completion_list);
     g_list_free(completion_list);
 
-    completion_list = NULL;
-    list = addr_vcard->address_list;
-    while( list ) {
-	cmp_data = completion_data_new(LIBBALSA_ADDRESS(list->data), TRUE);
-	completion_list = g_list_prepend(completion_list, cmp_data);
-	list = g_list_next(list);
-    }
-    completion_list = g_list_reverse(completion_list);
-    g_completion_add_items(addr_vcard->alias_complete, completion_list);
-    g_list_free(completion_list);
     return TRUE;
 }
 
@@ -431,8 +558,10 @@ libbalsa_address_book_vcard_add_address(LibBalsaAddressBook * ab,
     GList *list;
     LibBalsaAddress *address;
     FILE *fp;
+    LibBalsaABErr res = LBABERR_OK;
 
-    if(!load_vcard_file(ab)) return LBABERR_CANNOT_READ;
+    load_vcard_file(ab); /* Ignore reading error, we may be adding */
+                         /* the first address. */
 
     list = LIBBALSA_ADDRESS_BOOK_VCARD(ab)->address_list;
     while (list) {
@@ -452,13 +581,8 @@ libbalsa_address_book_vcard_add_address(LibBalsaAddressBook * ab,
 	fprintf(fp, "FN:%s\n", new_address->full_name);
     if (new_address->first_name && *new_address->first_name != '\0') {
 	if (new_address->last_name && *new_address->last_name != '\0') {
-	    if (new_address->middle_name
-		&& *new_address->middle_name != '\0')
-		fprintf(fp, "N:%s;%s;%s\n", new_address->last_name,
-			new_address->first_name, new_address->middle_name);
-	    else
-		fprintf(fp, "N:%s;%s\n", new_address->last_name,
-			new_address->first_name);
+            fprintf(fp, "N:%s;%s\n", new_address->last_name,
+                    new_address->first_name);
 	} else
 	    fprintf(fp, "N:;%s\n", new_address->first_name);
     }
@@ -468,9 +592,10 @@ libbalsa_address_book_vcard_add_address(LibBalsaAddressBook * ab,
     for (list = new_address->address_list; list; list = g_list_next(list))
 	fprintf(fp, "EMAIL;INTERNET:%s\n", (gchar *) list->data);
 
-    fprintf(fp, "END:VCARD\n\n");
+    res = fprintf(fp, "END:VCARD\n\n") > 0 
+        ? LBABERR_OK : LBABERR_CANNOT_WRITE;
     fclose(fp);
-    return LBABERR_OK;
+    return res;
 }
 
 static LibBalsaABErr
@@ -500,7 +625,7 @@ libbalsa_address_book_vcard_save_config(LibBalsaAddressBook * ab,
 
     vc = LIBBALSA_ADDRESS_BOOK_VCARD(ab);
 
-    gnome_config_set_string("Path", vc->path);
+    libbalsa_conf_set_string("Path", vc->path);
 
     if (LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->save_config)
 	LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->save_config(ab, prefix);
@@ -517,21 +642,20 @@ libbalsa_address_book_vcard_load_config(LibBalsaAddressBook * ab,
     vc = LIBBALSA_ADDRESS_BOOK_VCARD(ab);
 
     g_free(vc->path);
-    vc->path = gnome_config_get_string("Path");
+    vc->path = libbalsa_conf_get_string("Path");
 
     if (LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->load_config)
 	LIBBALSA_ADDRESS_BOOK_CLASS(parent_class)->load_config(ab, prefix);
 }
 
-static GList *libbalsa_address_book_vcard_alias_complete(LibBalsaAddressBook * ab,
-							 const gchar * prefix, 
-							 gchar ** new_prefix)
+static GList *
+libbalsa_address_book_vcard_alias_complete(LibBalsaAddressBook * ab,
+					   const gchar * prefix, 
+					   char ** new_prefix)
 {
     LibBalsaAddressBookVcard *vc;
-    GList *resa = NULL, *resb = NULL;
+    GList *list;
     GList *res = NULL;
-    gchar *p1 = NULL, *p2 = NULL;
-    LibBalsaAddress *addr1, *addr2;
 
     g_return_val_if_fail(LIBBALSA_IS_ADDRESS_BOOK_VCARD(ab), NULL);
 
@@ -542,49 +666,14 @@ static GList *libbalsa_address_book_vcard_alias_complete(LibBalsaAddressBook * a
 
     load_vcard_file(ab);
 
-    resa = g_completion_complete(vc->name_complete, (gchar*)prefix, &p1);
-    resb = g_completion_complete(vc->alias_complete, (gchar*)prefix, &p2);
-
-    if ( p1 && p2 ) {
-	if ( strlen(p1) > strlen(p2) ) {
-	    *new_prefix = p1;
-	    g_free(p2);
-	} else {
-	    *new_prefix = p2;
-	    g_free(p1);
-	}
-    } else {
-	*new_prefix = p1?p1:p2;
+    for (list = g_completion_complete(vc->name_complete, (gchar *) prefix,
+                                      new_prefix);
+         list; list = list->next) {
+	InternetAddress *ia = ((CompletionData *) list->data)->ia;
+	internet_address_ref(ia);
+        res = g_list_prepend(res, ia);
     }
 
-    /*
-      Extract a list of addresses.
-      pick any of them if two addresses point to the same structure.
-      pick addr1 if it is available and there is no addr2 
-                    or it is smaller than addr1.
-      in other case, pick addr2 (one of addr1 or addr2 must be not-null).
-    */
-    while ( resa || resb ) {
-	addr1 = resa ? ((CompletionData*)resa->data)->address : NULL;
-	addr2 = resb ? ((CompletionData*)resb->data)->address : NULL;
-	
-	if (addr1 == addr2) {
-	    res = g_list_prepend(res, addr1);
-	    g_object_ref(addr1);
-	    resa = g_list_next(resa);
-	    resb = g_list_next(resb);
-	} else if (resa != NULL && 
-		   (resb == NULL || address_compare(addr1, addr2) > 0) ) {
-	    res = g_list_prepend(res, addr1);
-	    g_object_ref(addr1);
-	    resa = g_list_next(resa);
-	} else {
-	    res = g_list_prepend(res, addr2);
-	    g_object_ref(addr2);
-	    resb = g_list_next(resb);
-	}
-    }
-    res = g_list_reverse(res);
-
-    return res;
+    return g_list_reverse(res);
 }
+
