@@ -26,11 +26,13 @@
 #include <stdio.h>
 
 #include <glib.h>
+#include <gmime/gmime.h>
+#include <gdk/gdk.h>
 
 #include "config.h"
 
 #ifdef HAVE_GPGME
-#include "rfc3156.h"
+#include "gmime-gpgme-signature.h"
 #endif
 
 typedef enum _LibBalsaMessageBodyType LibBalsaMessageBodyType;
@@ -47,21 +49,35 @@ enum _LibBalsaMessageBodyType {
     LIBBALSA_MESSAGE_BODY_TYPE_VIDEO
 };
 
+typedef enum _LibBalsaAttachMode LibBalsaAttachMode;
+
+enum _LibBalsaAttachMode {
+    LIBBALSA_ATTACH_AS_ATTACHMENT = 1,
+    LIBBALSA_ATTACH_AS_INLINE,
+    LIBBALSA_ATTACH_AS_EXTBODY
+};
+
 struct _LibBalsaMessageBody {
     LibBalsaMessage *message;	/* The message of which this is a part */
+    /* FIXME: remove buffer and buf_len to decrease memory usage. */
     gchar *buffer;		/* holds raw data of the MIME part, or NULL */
+    ssize_t buflen;             /* size of the block */
     LibBalsaMessageHeaders *embhdrs;  /* headers of a message/rfc822 part */
-    gchar *mime_type;           /* the mime type/subtype of buffer, or NULL, if plain */
-    MuttBody *mutt_body;	/* pointer to BODY struct of mutt message */
+    LibBalsaMessageBodyType body_type;
+    gchar *content_type;        /* value of the Content-Type header of
+                                 * the body including mime type with
+                                 * optional parameters. NULL, if text/plain. */
+    const gchar *content_dsp;	/* content-disposition */ 
+    const gchar *content_id;    /* content-id */
     gchar *filename;		/* holds filename for attachments and such (used mostly for sending) */
-    gboolean attach_as_extbody; /* if an attachment shall be appended as external-body (sending) */
+    LibBalsaAttachMode attach_mode; /* attachment mode for sending */
     gchar *temp_filename;	/* Holds the filename of a the temporary file where this part is saved */
     gchar *charset;		/* the charset, used for sending, replying. */
-    guint disposition;      /* content-disposition */
+    GMimeObject *mime_part;	/* mime body */
 
 #ifdef HAVE_GPGME
-    gchar *decrypt_file;        /* temp file for a decrypted body */
-    LibBalsaSignatureInfo* sig_info;  /* info about a pgp signature body */
+    gboolean was_encrypted;
+    GMimeGpgmeSigstat* sig_info;  /* info about a pgp signature body */
 #endif
 
     LibBalsaMessageBody *next;	/* Next part in the message */
@@ -74,18 +90,25 @@ void libbalsa_message_body_free(LibBalsaMessageBody * body);
 LibBalsaMessageBodyType libbalsa_message_body_type(LibBalsaMessageBody *
 						   body);
 
-void libbalsa_message_body_set_mutt_body(LibBalsaMessageBody * body,
-					 MuttBody * mutt_body);
+void libbalsa_message_body_set_mime_body(LibBalsaMessageBody * body,
+					 GMimeObject * mime_part);
 
+GMimeStream *libbalsa_message_body_get_stream(LibBalsaMessageBody * body);
+gssize libbalsa_message_body_get_content(LibBalsaMessageBody * body,
+                                         gchar ** buf);
+GdkPixbuf *libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body,
+                                            GError ** err);
+
+gboolean libbalsa_message_body_save_fd(LibBalsaMessageBody * body, int fd,
+                                       gboolean filter_crlf);
 gboolean libbalsa_message_body_save(LibBalsaMessageBody * body,
-				    const gchar * prefixm,
-                                    const gchar * filename);
-gboolean libbalsa_message_body_save_temporary(LibBalsaMessageBody * body,
-					      const gchar * prefix);
+                                    const gchar * filename,
+                                    gboolean filter_crlf);
+gboolean libbalsa_message_body_save_temporary(LibBalsaMessageBody * body);
 
 gchar *libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 					   const gchar * param);
-gchar *libbalsa_message_body_get_content_type(LibBalsaMessageBody * body);
+gchar *libbalsa_message_body_get_mime_type(LibBalsaMessageBody * body);
 
 gboolean libbalsa_message_body_is_multipart(LibBalsaMessageBody * body);
 gboolean libbalsa_message_body_is_inline(LibBalsaMessageBody* body);
