@@ -22,24 +22,33 @@
 #ifndef __FOLDER_SCANNERS_H__
 #define __FOLDER_SCANNERS_H__
 
-typedef GNode* (LocalHandler)(GNode*root, const char*d_name, const char* fn);
-typedef gboolean ImapCheck(LibBalsaServer * server, const char *fn,
+typedef gboolean LocalCheck(const gchar * fn, guint depth);
+typedef void LocalMark(gpointer node);
+typedef gpointer LocalHandler(gpointer root, const char *d_name,
+			      const char *fn, GType type);
+
+typedef gboolean ImapCheck(const char *fn, LibBalsaServer * server,
                            guint depth);
-typedef void ImapMark(const char * fn, gpointer data);
+typedef void ImapMark(const char *fn, gint noselect, gint noscan,
+		      gpointer data);
 typedef void ImapHandler(const char *fn, char delim, gint noselect,
-			 gint noinferiors, gpointer data);
+			 gint marked, gint noscan, gpointer data);
 
 /* read_dir used by mailbox-node append-subtree callback */
-void libbalsa_scanner_local_dir(GNode *rnode, const gchar * prefix, 
+void libbalsa_scanner_local_dir(gpointer rnode, const gchar * prefix, 
+				LocalCheck check_local_path,
+				LocalMark mark_local_path,
 				LocalHandler folder_handler, 
-				LocalHandler mailbox_handler);
+                                LocalHandler mailbox_handler,
+                                GType parent_type);
 
-void libbalsa_scanner_imap_dir(GNode *rnode, LibBalsaServer * server, 
-			       const gchar* path, gboolean subscribed, 
-			       gboolean list_inbox,
+void libbalsa_scanner_imap_dir(gpointer rnode, LibBalsaServer * server, 
+                               const gchar* path, gboolean subscribed, 
+                               gboolean list_inbox,
                                ImapCheck check_imap_path,
                                ImapMark mark_imap_path,
-			       ImapHandler mailbox_handler,
-			       gpointer cb_data);
+                               ImapHandler handle_imap_path,
+                               gpointer cb_data, 
+                               GError **error);
 
 #endif				/* __FOLDER_SCANNERS_H__ */
