@@ -25,21 +25,18 @@
 
 typedef struct _LibBalsaMailbox LibBalsaMailbox;
 typedef struct _LibBalsaMailboxRemote LibBalsaMailboxRemote;
+typedef struct _LibBalsaMailboxSearchIter LibBalsaMailboxSearchIter;
 typedef struct _LibBalsaMessage LibBalsaMessage;
 typedef struct _LibBalsaMessageHeaders LibBalsaMessageHeaders;
 typedef struct _LibBalsaMessageBody LibBalsaMessageBody;
 typedef struct _LibBalsaServer LibBalsaServer;
+typedef struct _LibBalsaCondition LibBalsaCondition;
 
-/* Opaque structures */
-typedef struct body MuttBody;
-typedef struct header MuttHeader;
 
-#include "address.h"
 #include "message.h"
 #include "body.h"
 #include "files.h"
 #include "mime.h"
-#include "notify.h"
 
 #include "information.h"
 
@@ -65,11 +62,36 @@ typedef struct header MuttHeader;
 
 #define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
 
+/*
+ * Error domains for GError: only one for now, more to come.
+ */
+enum {
+    LIBBALSA_SCANNER_ERROR,
+    LIBBALSA_MAILBOX_ERROR
+};
+
+/*
+ * Error codes for GError: only one for now, more to come.
+ */
+enum {
+    LIBBALSA_SCANNER_ERROR_IMAP,
+    LIBBALSA_MAILBOX_APPEND_ERROR,
+    LIBBALSA_MAILBOX_AUTH_ERROR,
+    LIBBALSA_MAILBOX_COPY_ERROR,
+    LIBBALSA_MAILBOX_RENAME_ERROR,
+    LIBBALSA_MAILBOX_CREATE_ERROR,
+    LIBBALSA_MAILBOX_DELETE_ERROR,
+    LIBBALSA_MAILBOX_NETWORK_ERROR,
+    LIBBALSA_MAILBOX_OPEN_ERROR,
+    LIBBALSA_MAILBOX_TOOMANYOPEN_ERROR
+};
+
+
 /*
  * Initialize the library
  */
 void libbalsa_init(LibBalsaInformationFunc information_callback);
-void libbalsa_set_spool(gchar * spool);
+void libbalsa_set_spool(const gchar * spool);
 
 void libbalsa_show_message_source(LibBalsaMessage * msg,
                                   const gchar * font,
@@ -94,10 +116,25 @@ gchar *libbalsa_guess_ldif_file(void);
 
 gboolean libbalsa_ldap_exists(const gchar *server);
 
-void libbalsa_lock_mutt(void);
-void libbalsa_unlock_mutt(void);
-
-void libbalsa_mktemp (char *);
 void libbalsa_assure_balsa_dir(void);
+gchar* libbalsa_date_to_gchar(const time_t *date, const gchar *date_string);
+LibBalsaMessageStatus libbalsa_get_icon_from_flags(LibBalsaMessageFlag flags);
 
-#endif				/* __LIBBALSA_H__ */
+#ifdef USE_TLS
+#include <openssl/ssl.h>
+gboolean libbalsa_is_cert_known(X509* cert, long vfy_result);
+void libbalsa_certs_destroy(void);
+#endif
+
+#ifdef BALSA_USE_THREADS
+#include <pthread.h>
+pthread_t libbalsa_get_main_thread(void);
+gboolean libbalsa_am_i_subthread(void);
+#else
+#define libbalsa_am_i_subthread() FALSE
+#endif /* BALSA_USE_THREADS */
+void libbalsa_message(const char *fmt, ...);
+gchar * libbalsa_rot(const gchar * pass);
+
+
+#endif                          /* __LIBBALSA_H__ */
