@@ -35,6 +35,9 @@
     (G_TYPE_CHECK_INSTANCE_TYPE ((obj), LIBBALSA_TYPE_MAILBOX_LOCAL))
 #define LIBBALSA_IS_MAILBOX_LOCAL_CLASS(klass) \
     (G_TYPE_CHECK_CLASS_TYPE ((klass), LIBBALSA_TYPE_MAILBOX_LOCAL))
+#define LIBBALSA_MAILBOX_LOCAL_GET_CLASS(mailbox) \
+    (G_TYPE_INSTANCE_GET_CLASS ((mailbox), LIBBALSA_TYPE_MAILBOX_LOCAL, \
+				LibBalsaMailboxLocalClass))
 
 GType libbalsa_mailbox_local_get_type(void);
 
@@ -44,11 +47,19 @@ typedef struct _LibBalsaMailboxLocalClass LibBalsaMailboxLocalClass;
 struct _LibBalsaMailboxLocal {
     LibBalsaMailbox mailbox;
 
+    guint sync_id;  /* id of the idle mailbox sync job  */
+    guint sync_time; /* estimated time of sync job execution (in seconds),
+                      * used to  throttle frequency of large mbox syncing. */
+    guint sync_cnt; /* we do not want to rely on the time of last sync since
+                     * some sync can be faster than others. Instead, we
+                     * average the syncing time for mailbox. */
+    GArray *threading_info;
 };
 
 struct _LibBalsaMailboxLocalClass {
     LibBalsaMailboxClass klass;
 
+    LibBalsaMessage *(*load_message)(LibBalsaMailbox *mb, guint msgno);
     void (*remove_files)(LibBalsaMailboxLocal *mb);
 };
 
@@ -56,8 +67,26 @@ GObject *libbalsa_mailbox_local_new(const gchar * path, gboolean create);
 gint libbalsa_mailbox_local_set_path(LibBalsaMailboxLocal * mailbox,
 				     const gchar * path);
 
-#define libbalsa_mailbox_local_get_path(mbox) ((LIBBALSA_MAILBOX(mbox))->url+7)
+#define libbalsa_mailbox_local_get_path(mbox) \
+	((const gchar *) (LIBBALSA_MAILBOX(mbox))->url+7)
 
+void libbalsa_mailbox_local_load_messages(LibBalsaMailbox * mailbox,
+					  guint last_msgno);
+void libbalsa_mailbox_local_msgno_removed(LibBalsaMailbox * mailbox,
+					  guint msgno);
 void libbalsa_mailbox_local_remove_files(LibBalsaMailboxLocal *mailbox);
 
+/* Helpers for maildir and mh. */
+GMimeMessage *libbalsa_mailbox_local_get_mime_message(LibBalsaMailbox *
+						      mailbox,
+						      const gchar * name1,
+						      const gchar * name2);
+GMimeStream *libbalsa_mailbox_local_get_message_stream(LibBalsaMailbox *
+						       mailbox,
+						       const gchar * name1,
+						       const gchar * name2);
+
+/* Queued sync. */
+void libbalsa_mailbox_local_queue_sync(LibBalsaMailboxLocal * local);
+
 #endif				/* __LIBBALSA_MAILBOX_LOCAL_H__ */
