@@ -27,6 +27,9 @@
 #include <stdio.h>
 #include <time.h>
 
+#include <gmime/gmime.h>
+
+#include "config.h"
 #ifdef HAVE_GPGME
 #include "rfc3156.h"
 #endif
@@ -35,7 +38,7 @@
 #include <auth-client.h>
 #endif
 
-/* #define MESSAGE_COPY_CONTENT 1 */
+#define MESSAGE_COPY_CONTENT 1
 #define LIBBALSA_TYPE_MESSAGE \
     (libbalsa_message_get_type())
 #define LIBBALSA_MESSAGE(obj) \
@@ -50,25 +53,33 @@
 
 typedef struct _LibBalsaMessageClass LibBalsaMessageClass;
 typedef enum _LibBalsaMessageFlag LibBalsaMessageFlag;
-typedef enum _LibBalsaMsgCreateResult LibBalsaMsgCreateResult;
 
 enum _LibBalsaMessageFlag {
-    LIBBALSA_MESSAGE_FLAG_NEW = 1 << 1,
-    LIBBALSA_MESSAGE_FLAG_DELETED = 1 << 2,
-    LIBBALSA_MESSAGE_FLAG_REPLIED = 1 << 3,
-    LIBBALSA_MESSAGE_FLAG_FLAGGED = 1 << 4
+    LIBBALSA_MESSAGE_FLAG_NEW     = 1 << 0,
+    LIBBALSA_MESSAGE_FLAG_DELETED = 1 << 1,
+    LIBBALSA_MESSAGE_FLAG_REPLIED = 1 << 2,
+    LIBBALSA_MESSAGE_FLAG_FLAGGED = 1 << 3,
+    LIBBALSA_MESSAGE_FLAG_RECENT  = 1 << 4,
+    LIBBALSA_MESSAGE_FLAG_SELECTED= 1 << 5	/* pseudo flag */
 };
 
-enum _LibBalsaMsgCreateResult {
-    LIBBALSA_MESSAGE_CREATE_OK,
-#ifdef HAVE_GPGME
-    LIBBALSA_MESSAGE_SIGN_ERROR,
-    LIBBALSA_MESSAGE_ENCRYPT_ERROR,
-#endif
-    LIBBALSA_MESSAGE_CREATE_ERROR,
-    LIBBALSA_MESSAGE_SEND_ERROR
+#define LIBBALSA_MESSAGE_FLAGS_REAL \
+   (LIBBALSA_MESSAGE_FLAG_NEW       | \
+    LIBBALSA_MESSAGE_FLAG_DELETED   | \
+    LIBBALSA_MESSAGE_FLAG_REPLIED   | \
+    LIBBALSA_MESSAGE_FLAG_FLAGGED   | \
+    LIBBALSA_MESSAGE_FLAG_RECENT)
+
+typedef enum _LibBalsaMessageStatus LibBalsaMessageStatus;
+enum _LibBalsaMessageStatus {
+    LIBBALSA_MESSAGE_STATUS_UNREAD,
+    LIBBALSA_MESSAGE_STATUS_DELETED,
+    LIBBALSA_MESSAGE_STATUS_FLAGGED,
+    LIBBALSA_MESSAGE_STATUS_REPLIED,
+    LIBBALSA_MESSAGE_STATUS_ICONS_NUM
 };
 
+
 #ifdef HAVE_GPGME
 typedef enum _LibBalsaMsgProtectState LibBalsaMsgProtectState;
 
@@ -82,6 +93,39 @@ enum _LibBalsaMsgProtectState {
 };
 #endif
 
+typedef enum _LibBalsaMessageAttach LibBalsaMessageAttach;
+enum _LibBalsaMessageAttach {
+    LIBBALSA_MESSAGE_ATTACH_ATTACH,
+#ifdef HAVE_GPGME
+    LIBBALSA_MESSAGE_ATTACH_GOOD,
+    LIBBALSA_MESSAGE_ATTACH_NOTRUST,
+    LIBBALSA_MESSAGE_ATTACH_BAD,
+    LIBBALSA_MESSAGE_ATTACH_SIGN,
+    LIBBALSA_MESSAGE_ATTACH_ENCR,
+#endif
+    LIBBALSA_MESSAGE_ATTACH_ICONS_NUM
+};
+
+
+/*
+ * Message info used by mailbox;
+ *
+ *   headers->from
+ *   headers->date
+ *   headers->to_list
+ *   headers->content_type
+ *   subj
+ *   length
+ *   lines_len (scrap?)
+ *
+ * loaded by driver:
+ *   flags
+ *
+ * needed for threading local mailboxes:
+ *   message_id
+ *   references
+ *   in_reply_to
+ */
 /*
  * LibBalsaMessageHeaders contains all headers which are used to display both
  * a "main" message's headers as well as those from an embedded message/rfc822
@@ -95,14 +139,17 @@ struct _LibBalsaMessageHeaders {
     gchar *subject;
 
     /* from, reply, and disposition notification addresses */
-    LibBalsaAddress *from;
-    LibBalsaAddress *reply_to;
-    LibBalsaAddress *dispnotify_to;
+    InternetAddressList *from;
+    InternetAddressList *reply_to;
+    InternetAddressList *dispnotify_to;
 
     /* primary, secondary, and blind recipent lists */
-    GList *to_list;
-    GList *cc_list;
-    GList *bcc_list;
+    InternetAddressList *to_list;
+    InternetAddressList *cc_list;
+    InternetAddressList *bcc_list;
+
+    /* Mime type */
+    GMimeContentType *content_type;
 
     /* File Carbon Copy Mailbox URL */
     gchar *fcc_url;
@@ -125,15 +172,13 @@ struct _LibBalsaMessage {
     LibBalsaMessageFlag flags;
 
     /* headers */
-    MuttHeader* header;
     LibBalsaMessageHeaders *headers;
     int updated; /** whether complete headers have been fetched */
 
-    /* remail header if any */
-    gchar *remail;
+    GMimeMessage *mime_msg;
 
     /* sender address */
-    LibBalsaAddress *sender;
+    InternetAddressList *sender;
 
     /* subject line; we still need it here for sending;
      * although _SET_SUBJECT might resolve it(?) 
@@ -147,14 +192,13 @@ struct _LibBalsaMessage {
 #define LIBBALSA_MESSAGE_GET_SUBJECT(m) libbalsa_message_get_subject(m)
 #endif
 #define LIBBALSA_MESSAGE_SET_SUBJECT(m,s) \
-        { g_free((m)->subj); (m)->subj = (s); }
+        do { g_free((m)->subj); (m)->subj = (s); } while (0)
 
     /* replied message ID's */
     GList *references;
-    GList *references_for_threading; /* oldest first */
 
     /* replied message ID; from address on date */
-    gchar *in_reply_to;
+    GList *in_reply_to;
 
     /* message ID */
     gchar *message_id;
@@ -167,10 +211,11 @@ struct _LibBalsaMessage {
     LibBalsaMsgProtectState prot_state;
 #endif
 
-    /* a forced multipart subtype or NULL for mixed */
+    /* a forced multipart subtype or NULL for mixed; used only for
+     * sending */
     gchar *subtype;
 
-    /* additional message content type parameters */
+    /* additional message content type parameters; used only for sending */
     GList *parameters;
 
     /* message body */
@@ -178,18 +223,18 @@ struct _LibBalsaMessage {
     LibBalsaMessageBody *body_list;
     /*  GList *body_list; */
 
-    gint msgno;     /* message no; always copy for faster sorting. */
+    glong msgno;     /* message no; always copy for faster sorting;
+		      * counting starts at 1. */
 #if MESSAGE_COPY_CONTENT
 #define LIBBALSA_MESSAGE_GET_NO(m)      ((m)->msgno)
     glong length;   /* byte len */
-    gint lines_len; /* line len */
 #define LIBBALSA_MESSAGE_GET_LENGTH(m)  ((m)->length)
-#define LIBBALSA_MESSAGE_GET_LINES(m) ((m)->lines_len)
 #else
 #define LIBBALSA_MESSAGE_GET_LENGTH(m) libbalsa_message_get_length(m)
-#define LIBBALSA_MESSAGE_GET_LINES(m)  libbalsa_message_get_lines(m)
 #define LIBBALSA_MESSAGE_GET_NO(m)  libbalsa_message_get_no(m)
 #endif
+
+    unsigned has_all_headers:1;
 };
 
 #define LIBBALSA_MESSAGE_HAS_FLAG(message, mask) \
@@ -202,11 +247,8 @@ struct _LibBalsaMessage {
     LIBBALSA_MESSAGE_HAS_FLAG(message, LIBBALSA_MESSAGE_FLAG_REPLIED)
 #define LIBBALSA_MESSAGE_IS_FLAGGED(message) \
     LIBBALSA_MESSAGE_HAS_FLAG(message, LIBBALSA_MESSAGE_FLAG_FLAGGED)
-
-#define LIBBALSA_MESSAGE_SET_FLAGS(message, mask) \
-    LIBBALSA_MESSAGE(message)->flags |= mask
-#define LIBBALSA_MESSAGE_UNSET_FLAGS(message, mask) \
-    LIBBALSA_MESSAGE(message)->flags &= ~mask
+#define LIBBALSA_MESSAGE_IS_RECENT(message) \
+    LIBBALSA_MESSAGE_HAS_FLAG(message, LIBBALSA_MESSAGE_FLAG_RECENT)
 
 struct _LibBalsaMessageClass {
     GObjectClass parent_class;
@@ -224,9 +266,11 @@ GType libbalsa_message_get_type(void);
  * message headers
  */
 void libbalsa_message_headers_destroy(LibBalsaMessageHeaders * headers);
-void libbalsa_message_headers_from_mutt(LibBalsaMessageHeaders *headers,
-					MuttHeader *mheader);
-GList *libbalsa_message_user_hdrs_from_mutt(MuttHeader* header);
+void libbalsa_message_headers_from_gmime(LibBalsaMessageHeaders *headers,
+					 GMimeMessage *msg);
+void libbalsa_message_init_from_gmime(LibBalsaMessage * message,
+				      GMimeMessage * msg);
+GList *libbalsa_message_user_hdrs_from_gmime(GMimeMessage *msg);
 
 
 /*
@@ -234,46 +278,25 @@ GList *libbalsa_message_user_hdrs_from_mutt(MuttHeader* header);
  */
 LibBalsaMessage *libbalsa_message_new(void);
 
-gboolean libbalsa_message_copy(LibBalsaMessage * message,
-			       LibBalsaMailbox * dest);
 gboolean libbalsa_message_save(LibBalsaMessage * message,
 			       const gchar *filename);
-gboolean libbalsa_messages_move(GList * messages,
-				LibBalsaMailbox * dest);
-gboolean libbalsa_messages_copy(GList * messages,
-                                LibBalsaMailbox * dest);
 
-void libbalsa_messages_delete(GList * messages, gboolean del);
-
-void libbalsa_messages_read(GList * messages, gboolean read);
-void libbalsa_messages_flag(GList * messages, gboolean flag);
+void libbalsa_messages_change_flag(GList * messages,
+                                   LibBalsaMessageFlag flag,
+                                   gboolean set);
 
 void libbalsa_message_reply(LibBalsaMessage * message);
+gboolean libbalsa_message_set_msg_flags(LibBalsaMessage * message,
+					LibBalsaMessageFlag set,
+					LibBalsaMessageFlag clear);
 
 void libbalsa_message_append_part(LibBalsaMessage * message,
 				  LibBalsaMessageBody * body);
 
-gboolean libbalsa_message_body_ref(LibBalsaMessage * message, gboolean read);
+gboolean libbalsa_message_body_ref(LibBalsaMessage * message, gboolean read,
+                                   gboolean fetch_all_headers);
 void libbalsa_message_body_unref(LibBalsaMessage * message);
 
-LibBalsaMsgCreateResult libbalsa_message_queue(LibBalsaMessage* message, 
-					       LibBalsaMailbox* outbox, LibBalsaMailbox* fccbox,
-					       gint encoding, gboolean flow);
-#if ENABLE_ESMTP
-LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage* message,
-					      LibBalsaMailbox* outbox,  
-					      LibBalsaMailbox* fccbox,
-					      gint encoding, gchar* smtp_server,
-					      auth_context_t smtp_authctx,
-					      gint tls_mode, gboolean flow,
-                                              gboolean debug);
-#else
-LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage* message,
-					      LibBalsaMailbox* outbox,  
-					      LibBalsaMailbox* fccbox,
-					      gint encoding, gboolean flow,
-                                              gboolean debug);
-#endif
 gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 				   LibBalsaMailbox * draftbox,
 				   LibBalsaMessage * reply_message,
@@ -283,20 +306,14 @@ gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 /*
  * misc message releated functions
  */
-gchar *libbalsa_message_headers_date_to_gchar(LibBalsaMessageHeaders * headers,
-					      const gchar * date_string);
-#define libbalsa_message_date_to_gchar(m,s) \
-        libbalsa_message_headers_date_to_gchar((m)->headers,s)
-gchar *libbalsa_message_size_to_gchar(LibBalsaMessage * message,
-                                      gboolean lines);
-gchar *libbalsa_message_title(LibBalsaMessage * message,
-                              const gchar * format);
 gchar **libbalsa_create_hdr_pair(const gchar * name, gchar * value);
 
 const gchar *libbalsa_message_pathname(LibBalsaMessage * message);
 gchar *libbalsa_message_charset(LibBalsaMessage * message);
 const gchar *libbalsa_message_body_charset(LibBalsaMessageBody * body);
 gboolean libbalsa_message_is_multipart(LibBalsaMessage * message);
+gboolean libbalsa_message_is_partial(LibBalsaMessage * message,
+				     gchar ** id);
 gboolean libbalsa_message_has_attachment(LibBalsaMessage * message);
 #ifdef HAVE_GPGME
 gboolean libbalsa_message_is_pgp_signed(LibBalsaMessage * message);
@@ -305,10 +322,11 @@ gboolean libbalsa_message_is_pgp_encrypted(LibBalsaMessage * message);
 
 GList *libbalsa_message_find_user_hdr(LibBalsaMessage * message, 
                                       const gchar * find);
-FILE* libbalsa_message_get_part_by_id(LibBalsaMessage* msg, const gchar* id);
+GMimeStream *libbalsa_message_get_part_by_id(LibBalsaMessage * message,
+                                             const gchar * id);
 
 void libbalsa_message_set_dispnotify(LibBalsaMessage *message, 
-				     LibBalsaAddress *address);
+				     InternetAddress *ia);
 
 /* use LIBBALSA_MESSAGE_GET_SUBJECT() macro, we may optimize this
    function out if we find a way.
@@ -319,5 +337,18 @@ guint libbalsa_message_get_lines(LibBalsaMessage* msg);
 glong libbalsa_message_get_length(LibBalsaMessage* msg);
 #endif
 glong libbalsa_message_get_no(LibBalsaMessage* msg);
-void libbalsa_message_headers_update(LibBalsaMessage * message);
+LibBalsaMessageAttach libbalsa_message_get_attach_icon(LibBalsaMessage *
+						       message);
+#define libbalsa_message_date_to_gchar(m, f) libbalsa_date_to_gchar(&(m)->headers->date, (f))
+#define libbalsa_message_headers_date_to_gchar(h, f) libbalsa_date_to_gchar(&(h)->date, (f))
+
+GList *libbalsa_message_refs_for_threading(LibBalsaMessage* msg);
+
+void libbalsa_message_load_envelope(LibBalsaMessage *message);
+gboolean libbalsa_message_set_headers_from_string(LibBalsaMessage *message,
+                                                  const gchar *str);
+void libbalsa_message_set_references_from_string(LibBalsaMessage * message,
+						 const gchar *str);
+void libbalsa_message_set_in_reply_to_from_string(LibBalsaMessage * message,
+						  const gchar *str);
 #endif				/* __LIBBALSA_MESSAGE_H__ */
