@@ -20,13 +20,14 @@
  * 02111-1307, USA.
  */
 
-#include <string.h>
 #include "config.h"
 
+#include <string.h>
+
 #include "libbalsa.h"
-#include "mailbackend.h"
 #include "misc.h"
 #include "html.h"
+#include "i18n.h"
 
 /* FIXME: The content of this file could go to message.c */
 
@@ -46,11 +47,11 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 		  gchar * reply_prefix_str, gint llen, gboolean ignore_html,
                   gboolean flow)
 {
-    FILE *part;
-    size_t alloced;
     gchar *res = NULL;
+    size_t allocated;
     GString *reply = NULL;
-    gboolean is_html;
+    gchar *mime_type;
+    LibBalsaHTMLType html_type;
 
     switch (libbalsa_message_body_type(body)) {
     case LIBBALSA_MESSAGE_BODY_TYPE_OTHER:
@@ -67,50 +68,24 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 	break;
     case LIBBALSA_MESSAGE_BODY_TYPE_TEXT:
 	/* don't return text/html stuff... */
-	is_html = (body->mutt_body->subtype
-		   && !strcmp("html", body->mutt_body->subtype));
-	if (ignore_html && is_html)
+	mime_type = libbalsa_message_body_get_mime_type(body);
+	html_type = libbalsa_html_type(mime_type);
+	g_free(mime_type);
+
+	if (ignore_html && html_type)
 	    break;
 
-	libbalsa_message_body_save_temporary(body, NULL);
-
-	part = fopen(body->temp_filename, "r");
-	if (!part)
-	    break;
-	alloced = libbalsa_readfile(part, &res);
-	fclose(part);
+	allocated = libbalsa_message_body_get_content(body, &res);
 	if (!res)
-	    break;
+	    return NULL;
 
 #ifdef HAVE_GTKHTML
-	if (is_html) {
-	    GtkWidget *html;
-	    GString *str;
-
-	    str = g_string_new(NULL);
-	    html = libbalsa_html_new(res, alloced, str, NULL, NULL);
-	    gtk_widget_destroy(html);
-	    g_free(res);
-	    if (str->len)
-		res = g_string_free(str, FALSE);
-	    else {
-		g_string_free(str, TRUE);
-		break;
-	    }
+	if (html_type) {
+	    allocated = libbalsa_html_filter(html_type, &res, allocated);
+	    libbalsa_html_to_string(&res, allocated);
 	}
 #endif /* HAVE_GTKHTML */
 
-#ifdef HAVE_GPGME
-	/* if this is a RFC 2440 signed part, strip the signature status */
-	if (body->sig_info) {
-	    gchar *p = 
-		g_strrstr(res, _("\n--\nThis is an OpenPGP signed message part:\n"));
-	    
-	    if (p)
-		*p = '\0';
-	}
-#endif
-
 	if (llen > 0) {
             if (flow && libbalsa_message_body_is_flowed(body)) {
                 /* we're making a `format=flowed' message, and the
@@ -180,7 +155,10 @@ process_mime_multipart(LibBalsaMessage * message,
 	if (!s)
 	    continue;
 	if (res) {
-	    res = g_string_append(res, s->str);
+	    if (res->str[res->len - 1] != '\n')
+		g_string_append_c(res, '\n');
+	    g_string_append_c(res, '\n');
+	    g_string_append(res, s->str);
 	    g_string_free(s, TRUE);
 	} else
 	    res = s;
@@ -195,6 +173,7 @@ content2reply(LibBalsaMessage * message, gchar * reply_prefix_str,
     LibBalsaMessageBody *body;
     GString *reply = NULL, *res;
 
+    libbalsa_message_body_ref(message, FALSE, FALSE);
     body = message->body_list;
     for (body = message->body_list; body; body = body->next) {
 	res = process_mime_part(message, body, reply_prefix_str, llen,
@@ -207,6 +186,7 @@ content2reply(LibBalsaMessage * message, gchar * reply_prefix_str,
 	} else
 	    reply = res;
     }
+    libbalsa_message_body_unref(message);
 
     return reply;
 }
