@@ -22,24 +22,33 @@
 
 #include "config.h"
 
-#include <sys/utsname.h>
+#define _SVID_SOURCE           1
+#define _XOPEN_SOURCE          500
 #include <ctype.h>
-#include <string.h>
-#include <errno.h>
 #include <dirent.h>
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/file.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/utsname.h>
+
+#ifdef HAVE_GNOME
+#include <libgnomevfs/gnome-vfs.h>
+#endif
 
-#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnome/gnome-i18n.h>
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 #include "misc.h"
 #include "html.h"
+#include "i18n.h"
 
 static const gchar *libbalsa_get_codeset_name(const gchar *txt, 
 					      LibBalsaCodeset Codeset);
-
+static int getdnsdomainname(char *s, size_t l);
 
 /* libbalsa_lookup_mime_type:
    find out mime type of a file. Must work for both relative and absolute
@@ -48,6 +57,7 @@ static const gchar *libbalsa_get_codeset_name(const gchar *txt,
 gchar*
 libbalsa_lookup_mime_type(const gchar * path)
 {
+#ifdef HAVE_GNOME
     GnomeVFSFileInfo* vi = gnome_vfs_file_info_new();
     gchar* uri, *mime_type;
 
@@ -64,7 +74,10 @@ libbalsa_lookup_mime_type(const gchar * path)
     g_free(uri);
     mime_type = g_strdup(gnome_vfs_file_info_get_mime_type(vi));
     gnome_vfs_file_info_unref(vi);
-    return mime_type;
+    return mime_type ? mime_type : g_strdup("application/octet-stream");
+#else
+    return g_strdup("application/octet-stream");
+#endif/* HAVE_GNOME */
 }
 
 gchar *
@@ -81,10 +94,58 @@ libbalsa_get_hostname(void)
     return g_strdup (utsname.nodename);
 }
 
+static int 
+getdnsdomainname (char *s, size_t l)
+{
+  FILE *f;
+  char tmp[1024];
+  char *p = NULL;
+  char *q;
+
+  if ((f = fopen ("/etc/resolv.conf", "r")) == NULL) return (-1);
+
+  tmp[sizeof (tmp) - 1] = 0;
+
+  l--; /* save room for the terminal \0 */
+
+  while (fgets (tmp, sizeof (tmp) - 1, f) != NULL)
+  {
+    p = tmp;
+    while ( g_ascii_isspace (*p)) p++;
+    if (strncmp ("domain", p, 6) == 0 || strncmp ("search", p, 6) == 0)
+    {
+      p += 6;
+      
+      for (q = strtok (p, " \t\n"); q; q = strtok (NULL, " \t\n"))
+	if (strcmp (q, "."))
+	  break;
+
+      if (q)
+      {
+	  char *a = q;
+	  
+	  for (; *q; q++)
+	      a = q;
+	  
+	  if (*a == '.')
+	      *a = '\0';
+	  
+	  g_stpcpy (s, q);
+	  fclose (f);
+	  return 0;
+      }
+      
+    }
+  }
+
+  fclose (f);
+  return (-1);
+}
+
 gchar *
 libbalsa_get_domainname(void)
 {
-    char domainname[SYS_NMLN];
+    char domainname[256]; /* arbitrary length */
     struct utsname utsname;
     gchar *d;
  
@@ -94,7 +155,7 @@ libbalsa_get_domainname(void)
         return g_strdup( d+1 );
     }
  
-    if ( getdnsdomainname(domainname, SYS_NMLN) == 0 ) {
+    if ( getdnsdomainname(domainname, sizeof(domainname)) == 0 ) {
 	return g_strdup(domainname);
     }
     return NULL;
@@ -130,36 +191,32 @@ libbalsa_urlencode(const gchar* str)
     return retval;
 }
 
-
-/* FIXME: Move to address.c and change name to
- *   libbalsa_address_list_to_string or something */
 gchar *
-libbalsa_make_string_from_list(const GList * the_list)
+libbalsa_urldecode(const gchar * str)
 {
-    return libbalsa_make_string_from_list_p(the_list);
-}
-/* private to libbalsa: */
-gchar *
-libbalsa_make_string_from_list_p(const GList * the_list)
-{
-    const GList *list;
-    GString *gs = g_string_new(NULL);
+    gchar *retval;
+    gchar *x;
 
-    for (list = the_list; list; list = list->next) {
-        LibBalsaAddress *addy = list->data;
-        gchar *str = libbalsa_address_to_gchar_p(addy, -1);
-        if (str) {
-            if (gs->len > 0)
-                g_string_append(gs, ", ");
-            g_string_append(gs, str);
-            g_free(str);
-        }
+    retval = g_new(char, strlen(str));
+
+    for (x = retval; *str != '\0'; str++, x++) {
+	*x = *str;
+	if (*x == '+')
+	    *x = ' ';
+	else if (*x == '%') {
+	    if (!*++str || !g_ascii_isxdigit(*str))
+		break;
+	    *x = g_ascii_xdigit_value(*str);
+	    if (!*++str || !g_ascii_isxdigit(*str))
+		break;
+	    *x = *x << 4 | g_ascii_xdigit_value(*str);
+	}
     }
 
-    return g_string_free(gs, FALSE);
+    *x = '\0';
+    return retval;
 }
 
-
 /* readfile allocates enough space for the ending '\0' characeter as well.
    returns the number of read characters.
 */
@@ -347,6 +404,11 @@ libbalsa_wrap_string(gchar * str, int width)
 /* Updated 2003 to implement DelSp=Yes as in
  * http://www.ietf.org/internet-drafts/draft-gellens-format-bis-01.txt
  */
+/* Now documented in RFC 3676:
+ * http://www.ietf.org/rfc/rfc3676.txt
+ * or
+ * http://www.faqs.org/rfcs/rfc3676.html
+ */
 
 #define MAX_WIDTH	997	/* enshrined somewhere */
 #define QUOTE_CHAR	'>'
@@ -407,6 +469,8 @@ unwrap_rfc2646(gchar * str, gboolean from_screen, gboolean delsp)
             if (*str)
                 str++;
 
+            if(len>0 && p[len-1] == '\r')
+                len--;  /* take care of '\r\n' line endings */
 	    flowed = (len > 0 && p[len - 1] == ' ' && !sig_sep);
 	    if (flowed && delsp)
 		/* Don't include the last space. */
@@ -528,8 +592,12 @@ dowrap_rfc2646(GList * list, gint width, gboolean to_screen,
              * */
             if (str > start && isspace((int)str[-1]) && str[-1] != ' ')
                 g_string_append_c(result, ' ');
-            if (*str)           /* line separator */
-                g_string_append_c(result, '\n');
+            if (*str) {         /* line separator */
+                if (to_screen || str == start)
+		    g_string_append_c(result, '\n');
+		else		/* DelSP = Yes */
+		    g_string_append(result, " \n");
+	    }
         } while (*str);         /* end of loop over output lines */
 
         g_free(text->str);
@@ -1011,31 +1079,65 @@ libbalsa_truncate_string(const gchar *str, gint length, gint dots)
 }
 
 /* libbalsa_expand_path:
-   do different kind of filename expansions, e.g. ~ -> $HOME, etc.
+   We handle only references to ~/.
 */
 gchar*
 libbalsa_expand_path(const gchar * path)
 {
-    char buf[_POSIX_PATH_MAX];
-    char* res;
-    strcpy(buf, path);
-    libbalsa_lock_mutt();
-    res = mutt_expand_path(buf, sizeof(buf));
-    libbalsa_unlock_mutt();
-    return g_strdup(res);
+    const gchar *home = g_get_home_dir();
+   
+    if(path[0] == '~') {
+        if(path[1] == '/')
+            return g_strconcat(home, path+1, NULL);
+        else if(path[1] == '\0')
+            return g_strdup(home);
+        /* else: unrecognized combination */
+    }
+    return g_strdup(path);
 }
 
-/* libbalsa_contract_path:
-   do a reverse transformation.
-*/
-void
-libbalsa_contract_path(gchar *path)
+
+
+/* create a uniq directory, resulting name should be freed with g_free */
+gboolean 
+libbalsa_mktempdir (char **s)
 {
-    libbalsa_lock_mutt();
-    mutt_pretty_mailbox(path);
-    libbalsa_unlock_mutt();
+    gchar *name;
+    int fd;
+
+    g_return_val_if_fail(s != NULL, FALSE);
+
+    do {
+	GError *error = NULL;
+	fd = g_file_open_tmp("balsa-tmpdir-XXXXXX", &name, &error);
+	close(fd);
+	unlink(name);
+	/* Here is a short time that the name could be reused */
+	fd = mkdir(name, 0700);
+	if (fd == -1) {
+	    g_free(name);
+	    if (!g_error_matches(error, G_FILE_ERROR, G_FILE_ERROR_EXIST))
+		return FALSE;
+	}
+	if (error)
+	    g_error_free(error);
+    } while (fd == -1);
+    *s = name;
+    /* FIXME: rmdir(name) at sometime */
+    return TRUE;
 }
 
+/* libbalsa_set_fallback_codeset: sets the codeset for incorrectly
+ * encoded characters. */
+static LibBalsaCodeset sanitize_fallback_codeset = WEST_EUROPE;
+LibBalsaCodeset
+libbalsa_set_fallback_codeset(LibBalsaCodeset codeset)
+{     
+    LibBalsaCodeset ret = sanitize_fallback_codeset;
+    sanitize_fallback_codeset = codeset;
+    return ret;
+}
+    
 /* libbalsa_utf8_sanitize
  *
  * Validate utf-8 text, and if validation fails, replace each offending
@@ -1054,56 +1156,133 @@ libbalsa_contract_path(gchar *path)
  * NOTE:    The text is either modified in place or replaced and freed.
  */
 gboolean
-libbalsa_utf8_sanitize(gchar **text, gboolean fallback, LibBalsaCodeset codeset,
+libbalsa_utf8_sanitize(gchar **text, gboolean fallback,
 		       gchar const **target)
 {
+    gchar *p;
+
     if (target)
 	*target = NULL;
-    if (!*text)
-        return TRUE;
-
-    if (g_utf8_validate(*text, -1, NULL))
+    if (!*text || g_utf8_validate(*text, -1, NULL))
 	return TRUE;
-    if (!fallback) {
-	gchar *p = *text;
-	while (!g_utf8_validate(p, -1, (const gchar **) &p))
-	    *p = '?';
-    } else {
-	/* */
-	gint b_written;
+
+    if (fallback) {
+	gsize b_written;
 	GError *conv_error = NULL;
-	const gchar *use_enc = libbalsa_get_codeset_name(*text, codeset);
-	gchar *p = g_convert(*text, strlen(*text), "utf-8", use_enc, NULL,
-			     &b_written, &conv_error);
+	const gchar *use_enc =
+            libbalsa_get_codeset_name(*text, sanitize_fallback_codeset);
+	p = g_convert(*text, strlen(*text), "utf-8", use_enc, NULL,
+                      &b_written, &conv_error);
 
 	if (p) {
 	    g_free(*text);
 	    *text = p;
 	    if (target)
 		*target = use_enc;
-	} else {
-	    gchar *p = *text;
-	    while (!g_utf8_validate(p, -1, (const gchar **) &p))
-		*p = '?';
-	    g_warning("conversion %s -> utf8 failed: %s", use_enc,
-		      conv_error->message);
-	    g_error_free(conv_error);
+	    return FALSE;
+	}
+	g_message("conversion %s -> utf8 failed: %s", use_enc,
+                  conv_error->message);
+	g_error_free(conv_error);
+    }
+    p = *text;
+    while (!g_utf8_validate(p, -1, (const gchar **) &p))
+	*p++ = '?';
+
+    return FALSE;
+}
+
+/* libbalsa_utf8_strstr() returns TRUE if s2 is a substring of s1.
+ * libbalsa_utf8_strstr is case insensitive
+ * this functions understands utf8 strings (as you might have guessed ;-)
+ */
+gboolean
+libbalsa_utf8_strstr(const gchar *s1, const gchar *s2)
+{
+    const gchar * p,* q;
+
+    /* convention : NULL string is contained in anything */
+    if (!s2) return TRUE;
+    /* s2 is non-NULL, so if s1==NULL we return FALSE :)*/
+    if (!s1) return FALSE;
+    /* OK both are non-NULL now*/
+    /* If s2 is the empty string return TRUE */
+    if (!*s2) return TRUE;
+    while (*s1) {
+	/* We look for the first char of s2*/
+	for (;*s1 &&
+		 g_unichar_toupper(g_utf8_get_char(s2))!=g_unichar_toupper(g_utf8_get_char(s1));
+	     s1 = g_utf8_next_char(s1));
+	if (*s1) {
+	    /* We found the first char let see if this potential match is an actual one */
+	    s1 = g_utf8_next_char(s1);
+	    q = s1;
+	    p = g_utf8_next_char(s2);
+	    while (*q && *p && 
+		   g_unichar_toupper(g_utf8_get_char(p))
+		   ==g_unichar_toupper(g_utf8_get_char(q))) {
+		p = g_utf8_next_char(p);
+		q = g_utf8_next_char(q);
+	    }
+	    /* We have a match if p has reached the end of s2, ie *p==0 */
+	    if (!*p) return TRUE;
 	}
     }
     return FALSE;
 }
 
-static gchar *std_codesets[LIBBALSA_NUM_CODESETS] = 
-     { "iso-8859-1", "iso-8859-2", "iso-8859-3", "iso-8859-4", "iso-8859-5", 
-       "iso-8859-6", "iso-8859-7", "iso-8859-8", "iso-8859-9", "iso-8859-10",
-       "iso-8859-11", "iso-8859-13", "iso-8859-14", "iso-8859-15", "koi-8r",
-       "koi-8u", "euc-jp", "euc-kr" };
-
-static gchar *win_codesets[LIBBALSA_NUM_CODESETS] =
-    { "windows-1252", "windows-1250", NULL, NULL, "windows-1251", 
-      "windows-1256", "windows-1253", "windows-1255", "windows-1254", NULL,
-      NULL, "windows-1257", NULL, NULL, NULL,
-      NULL, NULL, NULL };
+/* The LibBalsaCodeset enum is not used for anything currently, but this
+ * list must be the same length, and should probably be kept consistent: */
+LibBalsaCodesetInfo libbalsa_codeset_info[LIBBALSA_NUM_CODESETS] = {
+    {N_("west european"),       /* WEST_EUROPE          */
+     "iso-8859-1", "windows-1252"} ,
+    {N_("east european"),       /* EAST_EUROPE          */
+     "iso-8859-2", "windows-1250"} ,
+    {N_("south european"),      /* SOUTH_EUROPE         */
+     "iso-8859-3"} ,
+    {N_("north european"),      /* NORTH_EUROPE         */
+     "iso-8859-4"} ,
+    {N_("cyrillic"),            /* CYRILLIC             */
+     "iso-8859-5", "windows-1251"} ,
+    {N_("arabic"),              /* ARABIC               */
+     "iso-8859-6", "windows-1256"} ,
+    {N_("greek"),               /* GREEK                */
+     "iso-8859-7", "windows-1253"} ,
+    {N_("hebrew"),              /* HEBREW               */
+     "iso-8859-8", "windows-1255"} ,
+    {N_("turkish"),             /* TURKISH              */
+     "iso-8859-9", "windows-1254"} ,
+    {N_("nordic"),              /* NORDIC               */
+     "iso-8859-10"} ,
+    {N_("thai"),                /* THAI                 */
+     "iso-8859-11"} ,
+    {N_("baltic"),              /* BALTIC               */
+     "iso-8859-13", "windows-1257"} ,
+    {N_("celtic"),              /* CELTIC               */
+     "iso-8859-14"} ,
+    {N_("west europe (euro)"),  /* WEST_EUROPE_EURO     */
+     "iso-8859-15"} ,
+    {N_("russian"),             /* RUSSIAN              */
+     "koi-8r"} ,
+    {N_("ukranian"),            /* UKRAINE              */
+     "koi-8u"} ,
+    {N_("japanese"),            /* JAPAN                */
+     "iso-2022-jp"} ,
+    {N_("korean"),              /* KOREA                */
+     "euc-kr"} ,
+    {N_("east european"),       /* EAST_EUROPE_WIN      */
+     "windows-1250"} ,
+    {N_("cyrillic"),            /* CYRILLIC_WIN         */
+     "windows-1251"} ,
+    {N_("greek"),               /* GREEK_WIN            */
+     "windows-1253"} ,
+    {N_("hebrew"),              /* HEBREW_WIN           */
+     "windows-1255"} ,
+    {N_("arabic"),              /* ARABIC_WIN           */
+     "windows-1256"} ,
+    {N_("baltic"),              /* BALTIC_WIN           */
+     "windows-1257"} ,
+};
 
 /*
  * Return the name of a codeset according to Codeset. If txt is not NULL, is
@@ -1111,15 +1290,165 @@ static gchar *win_codesets[LIBBALSA_NUM_CODESETS] =
  * usually means that txt contains windows (not iso) characters.
  */
 static const gchar *
-libbalsa_get_codeset_name(const gchar *txt, LibBalsaCodeset Codeset)
+libbalsa_get_codeset_name(const gchar * txt, LibBalsaCodeset Codeset)
 {
-  if (txt && win_codesets[Codeset])
-    while (*txt) {
-      if (*(unsigned char *)txt >= 0x80 && *(unsigned char *)txt < 0x9f)
-	return win_codesets[Codeset];
-      txt++;
+    LibBalsaCodesetInfo *info = &libbalsa_codeset_info[Codeset];
+
+    if (txt && info->win) {
+        LibBalsaTextAttribute attr = libbalsa_text_attr_string(txt);
+        if (attr & LIBBALSA_TEXT_HI_CTRL)
+            return info->win;
     }
-  return std_codesets[Codeset];
+    return info->std;
+}
+
+/* Create a GtkComboBox with the national charsets as options;
+ * called when some text is found to be neither US-ASCII nor UTF-8, so
+ * the list includes neither of these. */
+GtkWidget *
+libbalsa_charset_button_new(void)
+{
+    GtkWidget *combo_box;
+    LibBalsaCodeset n, active = WEST_EUROPE;
+    const gchar *locale_charset;
+
+    combo_box = gtk_combo_box_new_text();
+    locale_charset = g_mime_locale_charset();
+
+    for (n = 0; n < LIBBALSA_NUM_CODESETS; n++) {
+        LibBalsaCodesetInfo *info = &libbalsa_codeset_info[n];
+        gchar *tmp = g_strdup_printf("%s (%s)", _(info->label), info->std);
+        gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), tmp);
+        g_free(tmp);
+
+	if (!g_ascii_strcasecmp(info->std, locale_charset))
+	    active = n;
+    }
+
+    /* locale_charset may be UTF-8, in which case it was not found,
+     * and the initial choice will be WEST_EUROPE (= 0). */
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), active);
+
+    return combo_box;
+}
+
+/* Helper */
+static void
+lb_text_attr(const gchar * text, gboolean * has_esc, gboolean * has_hi_bit,
+             gboolean * has_hi_ctrl)
+{
+    const guchar *p;
+
+    for (p = text; *p; p++) {
+        if (*p == 0x1b)
+            *has_esc = TRUE;
+        if (*p >= 0x80) {
+            *has_hi_bit = TRUE;
+            if (*p <= 0x9f)
+                *has_hi_ctrl = TRUE;
+        }
+    }
+}
+
+/* Return text attributes of a string. */
+LibBalsaTextAttribute
+libbalsa_text_attr_string(const gchar * string)
+{
+    LibBalsaTextAttribute attr;
+    gboolean has_esc = FALSE;
+    gboolean has_hi_bit = FALSE;
+    gboolean has_hi_ctrl = FALSE;
+    gboolean is_utf8 = TRUE;
+
+    lb_text_attr(string, &has_esc, &has_hi_bit, &has_hi_ctrl);
+    is_utf8 = g_utf8_validate(string, -1, NULL);
+
+    attr = 0;
+    if (has_esc)
+        attr |= LIBBALSA_TEXT_ESC;
+    if (has_hi_bit)
+        attr |= LIBBALSA_TEXT_HI_BIT;
+    if (has_hi_ctrl)
+        attr |= LIBBALSA_TEXT_HI_CTRL;
+    if (is_utf8 && has_hi_bit)
+        attr |= LIBBALSA_TEXT_HI_UTF8;
+
+    return attr;
+}
+
+/* Return text attributes of the contents of a file. */
+LibBalsaTextAttribute
+libbalsa_text_attr_file(const gchar * filename)
+{
+    LibBalsaTextAttribute attr;
+    FILE *fp;
+    gchar buf[80];
+    gchar *new_chars = buf;
+    gboolean has_esc = FALSE;
+    gboolean has_hi_bit = FALSE;
+    gboolean has_hi_ctrl = FALSE;
+    gboolean is_utf8 = TRUE;
+
+    fp = fopen(filename, "r");
+    if (!fp)
+        return 0;
+
+    while (fgets(new_chars, (sizeof buf) - (new_chars - buf), fp)) {
+	gboolean test_bits = !has_esc || !has_hi_bit || !has_hi_ctrl;
+
+	if (!test_bits && !is_utf8)
+	    break;
+
+        if (test_bits)
+            lb_text_attr(new_chars, &has_esc, &has_hi_bit, &has_hi_ctrl);
+
+        if (is_utf8) {
+            const gchar *end;
+
+            new_chars = buf;
+            if (!g_utf8_validate(buf, -1, &end)) {
+                if (g_utf8_get_char_validated(end, -1) == (gunichar) (-1))
+                    is_utf8 = FALSE;
+                else
+                    /* copy any remaining bytes, including the
+                     * terminating '\0', to start of buffer */
+                    while ((*new_chars = *end++) != '\0')
+                        new_chars++;
+            }
+        }
+    }
+
+    fclose(fp);
+
+    attr = 0;
+    if (has_esc)
+        attr |= LIBBALSA_TEXT_ESC;
+    if (has_hi_bit)
+        attr |= LIBBALSA_TEXT_HI_BIT;
+    if (has_hi_ctrl)
+        attr |= LIBBALSA_TEXT_HI_CTRL;
+    if (is_utf8 && has_hi_bit)
+        attr |= LIBBALSA_TEXT_HI_UTF8;
+
+    return attr;
+}
+
+/* Check whether a file is all ascii or utf-8, and return charset
+ * accordingly (NULL if it's neither).
+ * This function is called only as a last resort when a message is being
+ * prepared for sending.  The charset should always be set when the file
+ * is being attached.
+ */
+const gchar *
+libbalsa_file_get_charset(const gchar * filename)
+{
+    LibBalsaTextAttribute attr = libbalsa_text_attr_file(filename);
+
+    if (!(attr & LIBBALSA_TEXT_HI_BIT))
+	return "us-ascii";
+    if (attr & LIBBALSA_TEXT_HI_UTF8)
+	return "utf-8";
+    return NULL;
 }
 
 /* libbalsa_insert_with_url:
@@ -1200,102 +1529,148 @@ get_url_reg(void)
     return url_reg;
 }
 
-void
+static regex_t *
+get_ml_url_reg(void)
+{
+    static regex_t *url_reg = NULL;
+    
+    if (!url_reg) {
+        /* one-time compilation of a constant url_str expression */
+        static const char url_str[] =
+#ifdef HAVE_PCRE
+            "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#\\w]|[ \\t]*[\\r\\n]+[ \\t]*)+>";
+#else
+	    "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]]|[ \t]*[\r\n]+[ \t]*)+>";
+#endif
+
+	url_reg = g_new(regex_t, 1);
+        if (regcomp(url_reg, url_str, REG_EXTENDED | REG_ICASE) != 0)
+            g_warning("libbalsa_insert_with_url: "
+                      "multiline url regex compilation failed.");
+    }
+    
+    return url_reg;
+}
+
+gboolean
 libbalsa_insert_with_url(GtkTextBuffer * buffer,
                          const char *chars,
+			 const char *all_chars,
                          GtkTextTag * tag,
-                         void (*callback) (GtkTextBuffer *,
-                                           GtkTextIter *,
-                                           const gchar *,
-                                           gpointer),
-                         gpointer callback_data)
+                         LibBalsaUrlInsertInfo *url_info)
 {
     gchar *p, *buf;
+    const gchar *all_p;
     GtkTextIter iter;
 
     buf = p = g_strdup(chars);
+    all_p = all_chars;
     gtk_text_buffer_get_iter_at_mark(buffer, &iter,
                                      gtk_text_buffer_get_insert(buffer));
 
-    if (prescanner(p)) {
-        GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
-        GtkTextTag *url_tag = gtk_text_tag_table_lookup(table, "url");
-        gint offset = 0;
-        regex_t *url_reg = get_url_reg();
-        regmatch_t url_match;
-        gint match = regexec(url_reg, p, 1, &url_match, 0);
+    /* if there shouldn't be a callback for URL's we don't need to detect
+       them... */
+    if (url_info) {
+	GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
+	GtkTextTag *url_tag = gtk_text_tag_table_lookup(table, "url");
 
-        while (!match) {
-            gchar *buf;
+	if (url_info->ml_url) {
+	    gchar *url_end = strchr(p, '>');
 
-            if (url_match.rm_so) {
-                buf = g_strndup(p, url_match.rm_so);
-                gtk_text_buffer_insert_with_tags(buffer, &iter,
-                                                 buf, -1, tag, NULL);
-                g_free(buf);
-            }
+	    if (url_end) {
+		url_info->ml_url_buffer =
+		    g_string_append_len(url_info->ml_url_buffer, p, url_end - p);
+		gtk_text_buffer_insert_with_tags(buffer, &iter,
+						 url_info->ml_url_buffer->str, -1,
+						 url_tag, tag, NULL);
+		if (url_info->callback)
+		    url_info->callback(buffer, &iter, url_info->ml_url, url_info->callback_data);
+		g_string_free(url_info->ml_url_buffer, TRUE);
+		g_free(url_info->ml_url);
+		url_info->ml_url_buffer = NULL;
+		url_info->ml_url = NULL;
+		p = url_end;
+	    } else {
+		url_info->ml_url_buffer =
+		    g_string_append(url_info->ml_url_buffer, p);
+		url_info->ml_url_buffer =
+		    g_string_append_c(url_info->ml_url_buffer, '\n');
+		return TRUE;
+	    }
+	}
 
-            buf = g_strndup(p + url_match.rm_so,
-                            url_match.rm_eo - url_match.rm_so);
-            gtk_text_buffer_insert_with_tags(buffer, &iter, buf, -1,
-                                             url_tag, tag, NULL);
+	if (prescanner(p)) {
+	    gint offset = 0;
+	    regex_t *url_reg = get_url_reg();
+	    regmatch_t url_match;
+	    gint match = regexec(url_reg, p, 1, &url_match, 0);
 
-            /* remember the URL and its position within the text */
-            if (callback)
-                callback(buffer, &iter, buf, callback_data);
-            g_free(buf);
+	    while (!match) {
+		gchar *buf;
 
-            p += url_match.rm_eo;
-            offset += url_match.rm_eo;
-            if (prescanner(p))
-                match = regexec(url_reg, p, 1, &url_match, 0);
-            else
-                match = -1;
-        }
+		if (url_match.rm_so) {
+		    /* check if we hit a multi-line URL... (see RFC 1738) */
+		    if (all_p && (p[url_match.rm_so - 1] == '<' ||
+				  (url_match.rm_so > 4 &&
+				   g_ascii_strcasecmp(p + url_match.rm_so - 5, "<URL:"))) &&
+			!strchr(p + url_match.rm_eo, '>')) {
+			regex_t *ml_url_reg = get_ml_url_reg();
+			regmatch_t ml_url_match;
+		    
+			if (!regexec(ml_url_reg,
+				     all_chars + offset + url_match.rm_eo, 1,
+				     &ml_url_match, 0) && ml_url_match.rm_so == 0) {
+			    GString *ml_url = g_string_new("");
+			    const gchar *ml_p = all_chars + offset + url_match.rm_so;
+			    gint ml_cnt =
+				url_match.rm_eo - url_match.rm_so + ml_url_match.rm_eo - 1;
+
+			    for (; ml_cnt; (ml_p++, ml_cnt--))
+				if (*ml_p > ' ')
+				    ml_url = g_string_append_c(ml_url, *ml_p);
+			    url_info->ml_url = ml_url->str;
+			    g_string_free(ml_url, FALSE);
+			}
+		    }
+
+		    buf = g_strndup(p, url_match.rm_so);
+		    gtk_text_buffer_insert_with_tags(buffer, &iter,
+						     buf, -1, tag, NULL);
+		    g_free(buf);
+		}
+
+		if (url_info->ml_url) {
+		    url_info->ml_url_buffer = g_string_new(p + url_match.rm_so);
+		    url_info->ml_url_buffer =
+			g_string_append_c(url_info->ml_url_buffer, '\n');
+		    return TRUE;
+		}
+
+		buf = g_strndup(p + url_match.rm_so,
+				url_match.rm_eo - url_match.rm_so);
+		gtk_text_buffer_insert_with_tags(buffer, &iter, buf, -1,
+						 url_tag, tag, NULL);
+
+		/* remember the URL and its position within the text */
+		if (url_info->callback)
+		    url_info->callback(buffer, &iter, buf, url_info->callback_data);
+		g_free(buf);
+
+		p += url_match.rm_eo;
+		offset += url_match.rm_eo;
+		if (prescanner(p))
+		    match = regexec(url_reg, p, 1, &url_match, 0);
+		else
+		    match = -1;
+	    }
+	}
     }
 
     if (*p)
         gtk_text_buffer_insert_with_tags(buffer, &iter, p, -1, tag, NULL);
     g_free(buf);
-}
 
-/* Helper for "Select All" callbacks: if the currently focused widget
- * supports any concept of "select-all", do it.
- *
- * It would be nice if all such widgets had a "select-all" signal, but
- * they don't; in fact, the only one that does (GtkTreeView) is
- * broken--if we emit it when the tree is not in multiple selection
- * mode, bad stuff happens.
- */
-void
-libbalsa_window_select_all(GtkWindow * window)
-{
-    GtkWidget *focus_widget = gtk_window_get_focus(window);
-
-    if (!focus_widget)
-	return;
-
-    if (GTK_IS_TEXT_VIEW(focus_widget)) {
-        GtkTextBuffer *buffer =
-            gtk_text_view_get_buffer(GTK_TEXT_VIEW(focus_widget));
-        GtkTextIter start, end;
-
-        gtk_text_buffer_get_bounds(buffer, &start, &end);
-        gtk_text_buffer_place_cursor(buffer, &start);
-        gtk_text_buffer_move_mark_by_name(buffer, "selection_bound", &end);
-    } else if (GTK_IS_EDITABLE(focus_widget)) {
-        gtk_editable_select_region(GTK_EDITABLE(focus_widget), 0, -1);
-    } else if (GTK_IS_TREE_VIEW(focus_widget)) {
-        GtkTreeSelection *selection =
-            gtk_tree_view_get_selection(GTK_TREE_VIEW(focus_widget));
-        if (gtk_tree_selection_get_mode(selection) ==
-            GTK_SELECTION_MULTIPLE)
-            gtk_tree_selection_select_all(selection);
-#ifdef    HAVE_GTKHTML
-    } else if (libbalsa_html_can_select(focus_widget)) {
-	libbalsa_html_select_all(focus_widget);
-#endif /* HAVE_GTKHTML */
-    }
+    return FALSE;
 }
 
 void
@@ -1367,3 +1742,291 @@ libbalsa_match_regex(const gchar * line, regex_t * rex, guint * count,
 	*index = p - line;
     return c > 0;
 }
+
+
+#define compare_stat(osb, nsb)  ( (osb.st_dev != nsb.st_dev || osb.st_ino != nsb.st_ino || osb.st_rdev != nsb.st_rdev) ? -1 : 0 )
+
+int 
+libbalsa_safe_open (const char *path, int flags)
+{
+  struct stat osb, nsb;
+  int fd;
+ 
+  if ((fd = open (path, flags, 0600)) < 0)
+    return fd;
+ 
+  /* make sure the file is not symlink */
+  if (lstat (path, &osb) < 0 || fstat (fd, &nsb) < 0 ||
+      compare_stat(osb, nsb) == -1)
+      {
+	  g_warning("safe_open(): %s is a symlink!\n", path);
+	  close (fd);
+	  return (-1);
+      }
+ 
+  return (fd);
+}
+
+/* 
+ * This function is supposed to do nfs-safe renaming of files.
+ * 
+ * Warning: We don't check whether src and target are equal.
+ */
+
+int 
+libbalsa_safe_rename (const char *src, const char *target)
+{
+  struct stat ssb, tsb;
+
+  if (!src || !target)
+    return -1;
+
+  if (link (src, target) != 0)
+  {
+
+    /*
+     * Coda does not allow cross-directory links, but tells
+     * us it's a cross-filesystem linking attempt.
+     * 
+     * However, the Coda rename call is allegedly safe to use.
+     * 
+     * With other file systems, rename should just fail when 
+     * the files reside on different file systems, so it's safe
+     * to try it here.
+     *
+     */
+
+    if (errno == EXDEV)
+      return rename (src, target);
+    
+    return -1;
+  }
+
+  /*
+   * Stat both links and check if they are equal.
+   */
+  
+  if (stat (src, &ssb) == -1)
+  {
+    return -1;
+  }
+  
+  if (stat (target, &tsb) == -1)
+  {
+    return -1;
+  }
+
+  /* 
+   * pretend that the link failed because the target file
+   * did already exist.
+   */
+
+  if (compare_stat (ssb, tsb) == -1)
+  {
+    errno = EEXIST;
+    return -1;
+  }
+
+  /*
+   * Unlink the original link.  Should we really ignore the return
+   * value here? XXX
+   */
+
+  unlink (src);
+
+  return 0;
+}
+
+
+#define MAXLOCKATTEMPT 5
+
+/* Args:
+ *      excl            if excl != 0, request an exclusive lock
+ *      dot             if dot != 0, try to dotlock the file
+ *      timeout         should retry locking?
+ */
+int 
+libbalsa_lock_file (const char *path, int fd, int excl, int dot, int timeout)
+{
+#if defined (USE_FCNTL) || defined (USE_FLOCK)
+    int count;
+    int attempt;
+    struct stat prev_sb;
+#endif
+    int r = 0;
+
+#ifdef USE_FCNTL
+    struct flock lck;
+
+    memset (&lck, 0, sizeof (struct flock));
+    lck.l_type = excl ? F_WRLCK : F_RDLCK;
+    lck.l_whence = SEEK_SET;
+
+    count = 0;
+    attempt = 0;
+    while (fcntl (fd, F_SETLK, &lck) == -1)
+	{
+	    struct stat sb;
+	    g_print("%s(): fcntl errno %d.\n", __FUNCTION__, errno);
+    if (errno != EAGAIN && errno != EACCES)
+	{
+	    libbalsa_information
+		(LIBBALSA_INFORMATION_DEBUG, "fcntl failed, errno=%d.", errno);
+	    return -1;
+	}
+ 
+    if (fstat (fd, &sb) != 0)
+	sb.st_size = 0;
+     
+    if (count == 0)
+	prev_sb = sb;
+ 
+    /* only unlock file if it is unchanged */
+    if (prev_sb.st_size == sb.st_size && ++count >= (timeout?MAXLOCKATTEMPT:0))
+	{
+	    if (timeout)
+		libbalsa_information
+		    (LIBBALSA_INFORMATION_WARNING,
+		     _("Timeout exceeded while attempting fcntl lock!"));
+	    return (-1);
+	}
+ 
+    prev_sb = sb;
+ 
+    libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
+			 _("Waiting for fcntl lock... %d"), ++attempt);
+    sleep (1);
+}
+#endif /* USE_FCNTL */
+ 
+#ifdef USE_FLOCK
+count = 0;
+attempt = 0;
+while (flock (fd, (excl ? LOCK_EX : LOCK_SH) | LOCK_NB) == -1)
+{
+    struct stat sb;
+    if (errno != EWOULDBLOCK)
+	{
+	    libbalsa_message ("flock: %s", strerror(errno));
+	    r = -1;
+	    break;
+	}
+ 
+    if (fstat(fd,&sb) != 0 )
+	sb.st_size=0;
+     
+    if (count == 0)
+	prev_sb=sb;
+ 
+    /* only unlock file if it is unchanged */
+    if (prev_sb.st_size == sb.st_size && ++count >= (timeout?MAXLOCKATTEMPT:0))
+	{
+	    if (timeout)
+		libbalsa_message (_("Timeout exceeded while attempting flock lock!"));
+	    r = -1;
+	    break;
+	}
+ 
+    prev_sb = sb;
+ 
+    libbalsa_message (_("Waiting for flock attempt... %d"), ++attempt);
+    sleep (1);
+}
+#endif /* USE_FLOCK */
+ 
+#ifdef USE_DOTLOCK
+if (r == 0 && dot)
+     r = dotlock_file (path, fd, timeout);
+#endif /* USE_DOTLOCK */
+ 
+     if (r == -1)
+{
+    /* release any other locks obtained in this routine */
+ 
+#ifdef USE_FCNTL
+    lck.l_type = F_UNLCK;
+    fcntl (fd, F_SETLK, &lck);
+#endif /* USE_FCNTL */
+ 
+#ifdef USE_FLOCK
+    flock (fd, LOCK_UN);
+#endif /* USE_FLOCK */
+ 
+    return (-1);
+}
+ 
+return 0;
+}
+
+int 
+libbalsa_unlock_file (const char *path, int fd, int dot)
+{
+#ifdef USE_FCNTL
+    struct flock unlockit = { F_UNLCK, 0, 0, 0 };
+
+    memset (&unlockit, 0, sizeof (struct flock));
+    unlockit.l_type = F_UNLCK;
+    unlockit.l_whence = SEEK_SET;
+    fcntl (fd, F_SETLK, &unlockit);
+#endif
+
+#ifdef USE_FLOCK
+    flock (fd, LOCK_UN);
+#endif
+
+#ifdef USE_DOTLOCK
+    if (dot)
+	undotlock_file (path, fd);
+#endif
+
+    return 0;
+}
+
+#ifndef HAVE_GLIB22
+gboolean
+libbalsa_str_has_prefix(const gchar * str, const gchar * prefix)
+{
+    g_return_val_if_fail(str != NULL, FALSE);
+    g_return_val_if_fail(prefix != NULL, FALSE);
+
+    while (*prefix == *str && *prefix) {
+	++prefix;
+	++str;
+    }
+
+    return *prefix == '\0';
+}
+#endif				/* HAVE_GLIB22 */
+
+
+/* libbalsa_ia_rfc2821_equal
+   compares two addresses according to rfc2821: local-part@domain is equal,
+   if the local-parts are case sensitive equal, but the domain case-insensitive
+*/
+gboolean
+libbalsa_ia_rfc2821_equal(const InternetAddress * a,
+			  const InternetAddress * b)
+{
+    const gchar *a_atptr, *b_atptr;
+    gint a_atpos, b_atpos;
+
+    if (!a || !b || a->type != INTERNET_ADDRESS_NAME ||
+	b->type != INTERNET_ADDRESS_NAME)
+        return FALSE;
+
+    /* first find the "@" in the two addresses */
+    a_atptr = strchr(a->value.addr, '@');
+    b_atptr = strchr(b->value.addr, '@');
+    if (!a_atptr || !b_atptr)
+        return FALSE;
+    a_atpos = a_atptr - a->value.addr;
+    b_atpos = b_atptr - b->value.addr;
+
+    /* now compare the strings */
+    if (!a_atpos || !b_atpos || a_atpos != b_atpos || 
+        strncmp(a->value.addr, b->value.addr, a_atpos) ||
+        g_ascii_strcasecmp(a_atptr, b_atptr))
+        return FALSE;
+    else
+        return TRUE;
+}
