@@ -23,13 +23,9 @@
 #ifndef __LIBBALSA_MISC_H__
 #define __LIBBALSA_MISC_H__
 
-#ifdef BALSA_USE_THREADS
-#include <pthread.h>
-pthread_t libbalsa_get_main_thread(void);
-#endif /* BALSA_USE_THREADS */
-
 #include <stdio.h>
 #include <gtk/gtk.h>
+#include <gmime/gmime.h>
 
 #if ENABLE_ESMTP
 #include <auth-client.h>
@@ -61,13 +57,49 @@ enum _LibBalsaCodeset {
     WEST_EUROPE_EURO,   /* iso-8859-15 */
     RUSSIAN,            /* koi-8r */
     UKRAINE,            /* koi-8u */
-    JAPAN,              /* euc-jp */
+    JAPAN,              /* iso-2022-jp */
     KOREA,              /* euc-kr */
+    EAST_EUROPE_WIN,    /* windows-1250 */
+    CYRILLIC_WIN,       /* windows-1251 */
+    GREEK_WIN,          /* windows-1253 */
+    HEBREW_WIN,         /* windows-1255 */
+    ARABIC_WIN,         /* windows-1256 */
+    BALTIC_WIN,         /* windows-1257 */
     LIBBALSA_NUM_CODESETS
 };
 
+typedef enum _LibBalsaTextAttribute LibBalsaTextAttribute;
+enum _LibBalsaTextAttribute {
+    LIBBALSA_TEXT_ESC     = 1 << 0,     /* ESC char(s)     */
+    LIBBALSA_TEXT_HI_BIT  = 1 << 1,     /* 8-bit char(s)   */
+    LIBBALSA_TEXT_HI_CTRL = 1 << 2,     /* 0x80 - 0x9f     */
+    LIBBALSA_TEXT_HI_UTF8 = 1 << 3      /* 8-bit utf-8     */
+};
+
+typedef struct _LibBalsaCodesetInfo LibBalsaCodesetInfo;
+struct _LibBalsaCodesetInfo {
+    const gchar *label;
+    const gchar *std;
+    const gchar *win;
+};
+
+typedef void (*libbalsa_url_cb_t) (GtkTextBuffer *, GtkTextIter *,
+				   const gchar *, gpointer);
+typedef struct _LibBalsaUrlInsertInfo LibBalsaUrlInsertInfo;
+struct _LibBalsaUrlInsertInfo {
+    libbalsa_url_cb_t callback;
+    gpointer callback_data;
+    gchar *ml_url;
+    GString *ml_url_buffer;
+};
+
+extern LibBalsaCodesetInfo libbalsa_codeset_info[];
+GtkWidget *libbalsa_charset_button_new(void);
+LibBalsaTextAttribute libbalsa_text_attr_string(const gchar * string);
+LibBalsaTextAttribute libbalsa_text_attr_file(const gchar * filename);
+const gchar *libbalsa_file_get_charset(const gchar * filename);
+
 gchar *libbalsa_lookup_mime_type(const gchar * path);
-gchar *libbalsa_make_string_from_list(const GList *);
 
 size_t libbalsa_readfile(FILE * fp, char **buf);
 size_t libbalsa_readfile_nostat(FILE * fp, char **buf);
@@ -75,6 +107,7 @@ size_t libbalsa_readfile_nostat(FILE * fp, char **buf);
 gchar *libbalsa_get_hostname(void);
 gchar *libbalsa_get_domainname(void);
 gchar *libbalsa_urlencode(const gchar* str);
+gchar *libbalsa_urldecode(const gchar * str);
 
 gboolean libbalsa_find_word(const gchar * word, const gchar * str);
 void libbalsa_wrap_string(gchar * str, int width);
@@ -97,21 +130,31 @@ gboolean libbalsa_delete_directory_contents(const gchar *path);
 gchar *libbalsa_truncate_string(const gchar * str, gint length, gint dots);
 gchar *libbalsa_expand_path(const gchar *path);
 void libbalsa_contract_path(gchar *path);
-void libbalsa_mktemp(gchar * name);
+gboolean libbalsa_mktempdir(gchar ** name);
+LibBalsaCodeset libbalsa_set_fallback_codeset(LibBalsaCodeset codeset);
 gboolean libbalsa_utf8_sanitize(gchar ** text, gboolean fallback,
-                                LibBalsaCodeset codeset, gchar const **target);
-void libbalsa_insert_with_url(GtkTextBuffer * buffer,
-                              const char *chars,
-                              GtkTextTag * tag,
-                              void (*callback) (GtkTextBuffer *,
-                                                GtkTextIter *,
-                                                const gchar *,
-                                                gpointer),
-                              gpointer callback_data);
-void libbalsa_window_select_all(GtkWindow * window);
+                                gchar const **target);
+gboolean libbalsa_utf8_strstr(const gchar *s1,const gchar *s2);
+gboolean libbalsa_insert_with_url(GtkTextBuffer * buffer,
+				  const char *chars,
+				  const char *all_chars,
+				  GtkTextTag * tag,
+				  LibBalsaUrlInsertInfo *url_info);
 void libbalsa_unwrap_selection(GtkTextBuffer * buffer, regex_t * rex);
 gboolean libbalsa_match_regex(const gchar * line, regex_t * rex,
 			      guint * count, guint * index);
 
+int libbalsa_safe_open (const char *path, int flags);
+int libbalsa_lock_file (const char *path, int fd, int excl, int dot, int timeout);
+int libbalsa_unlock_file (const char *path, int fd, int dot);
+int libbalsa_safe_rename (const char *src, const char *target);
+#ifdef HAVE_GLIB22
+#define libbalsa_str_has_prefix(str, prefix) g_str_has_prefix((str), (prefix))
+#else				/* HAVE_GLIB22 */
+gboolean libbalsa_str_has_prefix(const gchar * str, const gchar * prefix);
+#endif				/* HAVE_GLIB22 */
+
+gboolean libbalsa_ia_rfc2821_equal(const InternetAddress * a,
+				   const InternetAddress * b);
 
 #endif				/* __LIBBALSA_MISC_H__ */
