@@ -23,6 +23,8 @@
 #ifndef __LIBBALSA_SERVER_H__
 #define __LIBBALSA_SERVER_H__
 
+#include "imap/libimap.h"
+
 #define LIBBALSA_TYPE_SERVER \
     (libbalsa_server_get_type())
 #define LIBBALSA_SERVER(obj) \
@@ -45,19 +47,26 @@ typedef enum {
     LIBBALSA_SERVER_UNKNOWN
 } LibBalsaServerType;
 
+typedef enum {
+    LIBBALSA_TLS_DISABLED,
+    LIBBALSA_TLS_ENABLED,
+    LIBBALSA_TLS_REQUIRED
+} LibBalsaTlsMode;
+
 struct _LibBalsaServer {
     GObject object;
 
     LibBalsaServerType type;
 
     gchar *host;
-#ifdef USE_SSL
-    gboolean use_ssl;
-#endif
-
     gchar *user;
-    gchar *passwd; /* NULL means "ask for it". Empty ("") is a legal one */
-    gboolean remember_passwd;
+    gchar *passwd;
+    /* We include SSL support in UI unconditionally to preserve config
+     * between SSL and non-SSL builds. We just fail if SSL is requested
+     * in non-SSL build. */
+    LibBalsaTlsMode tls_mode;
+    unsigned use_ssl:1;
+    unsigned remember_passwd:1;
 };
 
 struct _LibBalsaServerClass {
@@ -66,11 +75,7 @@ struct _LibBalsaServerClass {
     void (*set_username) (LibBalsaServer * server, const gchar * name);
     void (*set_password) (LibBalsaServer * server, const gchar * passwd);
     void (*set_host) (LibBalsaServer * server,
-		      const gchar * host
-#ifdef USE_SSL
-		      , gboolean use_ssl
-#endif
-		      );
+		      const gchar * host, gboolean use_ssl);
     gchar *(*get_password) (LibBalsaServer * server);
 };
 
@@ -80,15 +85,17 @@ void libbalsa_server_set_username(LibBalsaServer * server,
 				  const gchar * username);
 void libbalsa_server_set_password(LibBalsaServer * server,
 				  const gchar * passwd);
-void libbalsa_server_set_host(LibBalsaServer * server, const gchar * host
-#ifdef USE_SSL
-			      , gboolean use_ssl
-#endif
-			      );
+void libbalsa_server_set_host(LibBalsaServer * server, const gchar * host,
+                              gboolean use_ssl);
 gchar *libbalsa_server_get_password(LibBalsaServer * server,
 				    LibBalsaMailbox * mbox);
 
 void libbalsa_server_load_config(LibBalsaServer * server);
 void libbalsa_server_save_config(LibBalsaServer * server);
 
+
+void libbalsa_server_user_cb(ImapUserEventType ue, void *arg, ...);
+
+void libbalsa_server_connect_signals(LibBalsaServer * server, GCallback cb,
+                                     gpointer cb_data);
 #endif				/* __LIBBALSA_SERVER_H__ */
