@@ -21,6 +21,8 @@
 
 #include "balsa-initdruid.h"
 
+#include "i18n.h"
+#include "libbalsa-conf.h"
 #include "save-restore.h"
 
 #include "balsa-druid-page-welcome.h"
@@ -40,8 +42,10 @@ balsa_initdruid_init(GnomeDruid * druid)
 
     balsa_druid_page_welcome(druid, default_logo);
     balsa_druid_page_user(druid, default_logo);
+#if !defined(ENABLE_TOUCH_UI)
     balsa_druid_page_directory(druid, default_logo);
     balsa_druid_page_defclient(druid, default_logo);
+#endif
     balsa_druid_page_finish(druid, default_logo);
 }
 
@@ -57,6 +61,8 @@ balsa_initdruid(GtkWindow * window)
     gtk_container_add(GTK_CONTAINER(window), GTK_WIDGET(druid));
     g_signal_connect(G_OBJECT(druid), "cancel",
                      G_CALLBACK(balsa_initdruid_cancel), NULL);
+    g_signal_connect(G_OBJECT(druid), "destroy",
+                     G_CALLBACK(exit), NULL);
     g_object_ref(G_OBJECT(window));
 
     balsa_initdruid_init(druid);
@@ -79,7 +85,7 @@ balsa_initdruid_cancel(GnomeDruid * druid)
     gtk_widget_destroy(dialog);
 
     if (reply == GTK_RESPONSE_YES) {
-        gnome_config_drop_all();
+        libbalsa_conf_drop_all();
         exit(0);
     }
 }
