@@ -38,10 +38,11 @@
 #include "address-book-ldap.h"
 #endif
 #include "address-book-vcard.h"
+#include "address-book-gpe.h"
 
 struct ABMainWindow {
     GtkWidget *entry_list; /* GtkListView widget */
-    GtkWidget *edit_box;
+    GtkWidget *edit_box, *apply_button, *remove_button, *cancel_button;
     GtkWidget *edit_widget;
     GtkWidget *entries[NUM_FIELDS];
 
@@ -59,6 +60,8 @@ static gint bab_save_session(GnomeClient * client, gint phase,
                              gpointer client_data);
 static gint bab_kill_session(GnomeClient * client, gpointer client_data);
 
+static void ab_set_edit_widget(GtkWidget *w, gboolean can_remove);
+
 #define BALSA_CONFIG_PREFIX "balsa/"
 #define ADDRESS_BOOK_SECTION_PREFIX "address-book-"
 static void
@@ -88,6 +91,8 @@ bab_config_init(void)
 
 }
 
+static void ab_warning(const char *fmt, ...);
+
 enum {
     LIST_COLUMN_NAME,
     LIST_COLUMN_ADDRESS,
@@ -142,7 +147,8 @@ bab_load_cb(LibBalsaAddressBook *libbalsa_ab,
 }
 
 static gboolean
-bab_set_address_book(LibBalsaAddressBook *ab, GtkWidget* list)
+bab_set_address_book(LibBalsaAddressBook *ab, GtkWidget* list,
+                     const gchar *filter)
 {
     LibBalsaABErr ab_err;
     GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
@@ -152,11 +158,11 @@ bab_set_address_book(LibBalsaAddressBook *ab, GtkWidget* list)
 
 
     gtk_list_store_clear(GTK_LIST_STORE(model));
-    if( (ab_err=libbalsa_address_book_load(ab, NULL,
+    if( (ab_err=libbalsa_address_book_load(ab, filter,
                                            (LibBalsaAddressBookLoadFunc)
                                            bab_load_cb, model))
         != LBABERR_OK) {
-        printf("error loading vcard addres book from %s: %d\n", 
+        printf("error loading address book from %s: %d\n", 
                ab->name, ab_err);
     }
 
@@ -164,91 +170,176 @@ bab_set_address_book(LibBalsaAddressBook *ab, GtkWidget* list)
 }
 
 static void
-select_address_book_cb(gpointer callback_data, guint callback_action,
-                       GtkWidget *w)
+select_address_book_cb(GtkRadioAction * action, GtkRadioAction * current,
+                       gpointer callback_data)
 {
-    GList *l = g_list_nth(contacts_app.address_book_list,
-                          GPOINTER_TO_INT(callback_data));
-    if(!l) return;
+    GList *l;
+
+    if (action != current)
+        return;
+    l = g_list_nth(contacts_app.address_book_list,
+                   GPOINTER_TO_INT(callback_data));
+    if (!l)
+        return;
+    ab_set_edit_widget(NULL, FALSE);
     bab_set_address_book(LIBBALSA_ADDRESS_BOOK(l->data),
-                         contacts_app.entry_list);
+                         contacts_app.entry_list, NULL);
 }
 
-static GtkItemFactoryEntry menu_items[] = {
-  { "/_File",         NULL,         NULL, 0, "<Branch>" },
-  { "/File/_New",     "<control>N", NULL, 0, NULL },
-  { "/File/_Open",    "<control>O", (GtkItemFactoryCallback)NULL,
-0, NULL },
-  { "/File/_Save",    "<control>S", NULL, 0, NULL },
-  { "/File/Save _As", NULL,         NULL, 0, NULL },
-  { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
-  { "/File/_Quit",     "<control>Q", (GtkItemFactoryCallback)gtk_main_quit, 
-    0, NULL },
-  { "/File/sep2",     NULL,         NULL, 0, "<Separator>" },
-  { "/_Entry",         NULL,        NULL, 0, "<Branch>" },
-  { "/Entry/_New Person", NULL,   (GtkItemFactoryCallback)NULL, 0, NULL },
-  { "/Entry/_New Group",  NULL,   (GtkItemFactoryCallback)NULL, 0, NULL },
-  { "/Entry/_Delete",  NULL,   (GtkItemFactoryCallback)NULL, 0, NULL },
-  { "/Entry/_Edit...", NULL,   (GtkItemFactoryCallback)NULL, 0, NULL },
-  { "/_Help",          NULL,         NULL, 0, "<LastBranch>" },
-  { "/_Help/_About",   NULL,         NULL, 0, NULL },
+static void
+address_changed_cb(GtkWidget *w, gpointer data)
+{
+    struct ABMainWindow * aw = (struct ABMainWindow*)data;
+    gtk_widget_set_sensitive(aw->apply_button,  TRUE);
+    gtk_widget_set_sensitive(aw->cancel_button, TRUE);
+}
+
+
+static void
+edit_new_person_cb(GtkAction * action, gpointer user_data)
+{
+    GtkWidget *ew;
+    contacts_app.displayed_address = NULL;
+    ew = libbalsa_address_get_edit_widget(NULL, contacts_app.entries,
+                                          G_CALLBACK(address_changed_cb),
+                                          &contacts_app);
+    ab_set_edit_widget(ew, FALSE);
+    gtk_widget_set_sensitive(contacts_app.remove_button, FALSE);
+}
+
+/* Normal items */
+static GtkActionEntry entries[] = {
+    {"FileMenu", NULL, "_File"},
+    {"EntryMenu", NULL, "_Entry"},
+    {"HelpMenu", NULL, "_Help"},
+    {"New", GTK_STOCK_NEW, "_New", NULL, "New file", NULL},
+    {"Open", GTK_STOCK_OPEN, "_Open", NULL, "Open a file", NULL},
+    {"Save", GTK_STOCK_SAVE, "_Save", NULL, "Save file", NULL},
+    {"SaveAs", GTK_STOCK_SAVE_AS, "Save _As", "<shift><control>S",
+     "Save file as", NULL},
+    {"Quit", GTK_STOCK_QUIT, "_Quit", NULL, "Exit the program",
+     gtk_main_quit},
+    {"NewPerson", GTK_STOCK_NEW, "_New Person", "<shift><control>N",
+     "Add new person", G_CALLBACK(edit_new_person_cb)},
+    {"NewGroup", GTK_STOCK_NEW, "New _Group", "<control>G",
+     "Add new group", NULL},
+    {"About",
+#if GTK_CHECK_VERSION(2, 6, 0)
+     GTK_STOCK_ABOUT,
+#else
+     GNOME_STOCK_ABOUT,
+#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
+     "_About", NULL, NULL, NULL}
 };
 
+static const char *ui_description =
+"<ui>"
+"  <menubar name='MainMenu'>"
+"    <menu action='FileMenu'>"
+"      <menuitem action='New'/>"
+"      <menuitem action='Open'/>"
+"      <menuitem action='Save'/>"
+"      <menuitem action='SaveAs'/>"
+"      <separator/>"
+"      <menuitem action='Quit'/>"
+"      <separator/>"
+"    </menu>"
+"    <menu action='EntryMenu'>"
+"      <menuitem action='NewPerson'/>"
+"      <menuitem action='NewGroup'/>"
+"    </menu>"
+"    <menu action='HelpMenu'>"
+"      <menuitem action='About'/>"
+"    </menu>"
+"  </menubar>"
+"</ui>";
+
 static void
-get_main_menu(GtkWidget  *window, GtkWidget **menubar, GList* address_books)
+get_main_menu(GtkWidget * window, GtkWidget ** menubar,
+              GList * address_books)
 {
-    GtkItemFactory *item_factory;
+    GtkActionGroup *action_group;
+    GtkUIManager *ui_manager;
     GtkAccelGroup *accel_group;
-    gint nmenu_items = sizeof(menu_items)/sizeof(menu_items[0]);
+    GError *error;
     GList *ab;
     int cnt;
-    gchar *first_ab_path = NULL;
+    GSList *group = NULL;
 
-    accel_group = gtk_accel_group_new();
-    item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
-                                       accel_group);
-    gtk_item_factory_create_items (item_factory, nmenu_items,
-                                   menu_items, window);
-    
-    for(cnt=1, ab= address_books; ab; ab = ab->next, cnt++) {
-        LibBalsaAddressBook *address_book = LIBBALSA_ADDRESS_BOOK(ab->data);
-        GtkItemFactoryEntry gife;
-        gife.path = g_strdup_printf("/File/_%d:%s", cnt,
-                                    address_book->name);
-        gife.accelerator = cnt<=9 
-            ? g_strdup_printf("<control>%d", cnt) : NULL;
-        gife.callback = select_address_book_cb;
-        gife.callback_action = 0;
-        gife.item_type = first_ab_path ? first_ab_path : "<RadioItem>";
-        gife.extra_data = NULL;
-        gtk_item_factory_create_item(item_factory, &gife, 
-                                     GINT_TO_POINTER(cnt-1), 1);
-        g_free(gife.accelerator);
-        g_free(gife.path);
-        if(!first_ab_path)
-            first_ab_path = g_strdup_printf("/File/1:%s",
-                                    address_book->name);
+    action_group = gtk_action_group_new("MenuActions");
+    gtk_action_group_add_actions(action_group, entries,
+                                 G_N_ELEMENTS(entries), window);
+
+    ui_manager = gtk_ui_manager_new();
+    gtk_ui_manager_insert_action_group(ui_manager, action_group, 0);
+
+    accel_group = gtk_ui_manager_get_accel_group(ui_manager);
+    gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
+
+    error = NULL;
+    if (!gtk_ui_manager_add_ui_from_string(ui_manager, ui_description,
+                                           -1, &error)) {
+        g_message("building menus failed: %s", error->message);
+        g_error_free(error);
+        return;
     }
-    g_free(first_ab_path);
-    /* Attach the new accelerator group to the window. */
-    gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
-    
+
+    for (cnt = 1, ab = address_books; ab; ab = ab->next, cnt++) {
+        LibBalsaAddressBook *address_book;
+	gchar *label;
+        guint merge_id;
+        gchar *accelerator;
+        GtkRadioAction *radio_action;
+
+        address_book = LIBBALSA_ADDRESS_BOOK(ab->data);
+
+        label = g_strdup_printf("_%d:%s", cnt, address_book->name);
+        radio_action = gtk_radio_action_new(address_book->name, label,
+                                            NULL, NULL, cnt - 1);
+	g_free(label);
+        if (group)
+            gtk_radio_action_set_group(radio_action, group);
+        else {
+            group = gtk_radio_action_get_group(radio_action);
+	    gtk_action_activate(GTK_ACTION(radio_action));
+	}
+        g_signal_connect(G_OBJECT(radio_action), "changed",
+                         G_CALLBACK(select_address_book_cb),
+                         GINT_TO_POINTER(cnt - 1));
+
+        accelerator =
+            cnt <= 9 ? g_strdup_printf("<control>%d", cnt) : NULL;
+        gtk_action_group_add_action_with_accel(action_group,
+                                               GTK_ACTION(radio_action),
+                                               accelerator);
+        g_free(accelerator);
+
+        merge_id = gtk_ui_manager_new_merge_id(ui_manager);
+        gtk_ui_manager_add_ui(ui_manager, merge_id,
+                              "/ui/MainMenu/FileMenu/",
+                              address_book->name, address_book->name,
+                              GTK_UI_MANAGER_AUTO, FALSE);
+    }
+
     if (menubar)
-        /* Finally, return the actual menu bar created by the item factory. */
-        *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
+        /* Finally, return the actual menu bar created by the UIManager. */
+        *menubar = gtk_ui_manager_get_widget(ui_manager, "/MainMenu");
 }
 
-
 static void
-ab_set_edit_widget(GtkWidget *w)
+ab_set_edit_widget(GtkWidget *w, gboolean can_remove)
 {
-    g_return_if_fail(w);
-
     if(contacts_app.edit_widget)
         gtk_widget_destroy(contacts_app.edit_widget);
     contacts_app.edit_widget = w;
-    gtk_container_add(GTK_CONTAINER(contacts_app.edit_box), w);
-    gtk_widget_show_all(w);
+    if(w) {
+        gtk_box_pack_start(GTK_BOX(contacts_app.edit_box), w,
+                           FALSE, FALSE, 1);
+        gtk_widget_show_all(w);
+    }
+    gtk_widget_set_sensitive(contacts_app.apply_button,  FALSE);
+    gtk_widget_set_sensitive(contacts_app.remove_button, can_remove);
+    gtk_widget_set_sensitive(contacts_app.cancel_button, FALSE);
 }
 
 static void
@@ -260,17 +351,18 @@ list_selection_changed_cb(GtkTreeSelection *selection, gpointer data)
     GtkWidget *ew;
     LibBalsaAddress *address;
 
-    gtk_tree_selection_get_selected(selection, &model, &iter);
+    if(!gtk_tree_selection_get_selected(selection, &model, &iter))
+        return;
     gtk_tree_model_get_value(model, &iter, LIST_COLUMN_ADDRESS, &gv);
     address = LIBBALSA_ADDRESS(g_value_get_object(&gv));
     if(address) {
-        ew = libbalsa_address_get_edit_widget(address, contacts_app.entries);
-        ab_set_edit_widget(ew);
-    } else ab_set_edit_widget(NULL);
+        ew = libbalsa_address_get_edit_widget(address, contacts_app.entries,
+                                              G_CALLBACK(address_changed_cb),
+                                              data);
+        ab_set_edit_widget(ew, TRUE);
+    } else ab_set_edit_widget(NULL, FALSE);
     g_value_unset(&gv);
     contacts_app.displayed_address = address;
-    printf("selection_changed_cb: %s\n",
-           contacts_app.displayed_address->first_name);
 }
 
 
@@ -322,57 +414,135 @@ apply_button_cb(GtkWidget *w, gpointer data)
 {
     LibBalsaAddress * newval =
         libbalsa_address_new_from_edit_entries(contacts_app.entries);
-    libbalsa_address_book_modify_address(contacts_app.address_book,
-                                         contacts_app.displayed_address,
-                                         newval);
-    /* FIXME: error handling here! */
+    LibBalsaABErr err = 
+        contacts_app.displayed_address 
+        ? libbalsa_address_book_modify_address(contacts_app.address_book,
+                                               contacts_app.displayed_address,
+                                               newval)
+        : libbalsa_address_book_add_address(contacts_app.address_book,
+                                            newval);
+    if(err == LBABERR_OK) {
+        gtk_widget_set_sensitive(contacts_app.apply_button,  FALSE);
+        gtk_widget_set_sensitive(contacts_app.remove_button, TRUE);
+        gtk_widget_set_sensitive(contacts_app.cancel_button, FALSE);
+    } else 
+        ab_warning("Cannot add: %s\n",
+                   libbalsa_address_book_strerror(contacts_app.address_book,
+                                                  err));                   
     g_object_unref(newval);
 }
 
 static void
 remove_button_cb(GtkWidget *w, gpointer data)
 {
-    /* FIXME: removal code really needs some work.
-     * in particular, the list of entries is not updated after 
-     * address removal. */
-    libbalsa_address_book_remove_address(contacts_app.address_book,
-                                         contacts_app.displayed_address);
-    contacts_app.displayed_address = NULL;
-    /* FIXME: error handling here! */    
+    LibBalsaABErr err = 
+        libbalsa_address_book_remove_address(contacts_app.address_book,
+                                             contacts_app.displayed_address);
+    if(err == LBABERR_OK) {
+        GtkTreeIter       iter;
+        GtkTreeSelection *selection;
+        GtkTreeView  *v = GTK_TREE_VIEW(contacts_app.entry_list);
+        GtkTreeModel *m = gtk_tree_view_get_model(v);
+        selection       = gtk_tree_view_get_selection(v);
+        if(gtk_tree_selection_get_selected(selection, &m, &iter))
+            gtk_list_store_remove(GTK_LIST_STORE(m), &iter);
+        ab_set_edit_widget(NULL, FALSE);
+        contacts_app.displayed_address = NULL;
+    } else 
+        ab_warning("Cannot remove: %s\n",
+                   libbalsa_address_book_strerror(contacts_app.address_book,
+                                                  err));
 }
 static void
 cancel_button_cb(GtkWidget *w, gpointer data)
 {
+    struct ABMainWindow *abmw = (struct ABMainWindow*)data;
+    if(abmw->displayed_address) {
+        GtkWidget *ew =
+            libbalsa_address_get_edit_widget(abmw->displayed_address,
+                                             abmw->entries,
+                                             G_CALLBACK(address_changed_cb),
+                                             data);
+        ab_set_edit_widget(ew, TRUE);
+    } else ab_set_edit_widget(NULL, FALSE);
 }
 
 #define ELEMENTS(x) (sizeof(x)/sizeof((x)[0])) 
 static GtkWidget*
-bab_get_edit_button_box(void)
+bab_get_edit_button_box(struct ABMainWindow *abmw)
 {
-    GtkWidget *box, *button;
+    GtkWidget *box;
     box = gtk_hbutton_box_new();
     gtk_container_add(GTK_CONTAINER(box),
-                      button=gtk_button_new_from_stock(GTK_STOCK_APPLY));
-    g_signal_connect(G_OBJECT(button), "clicked",
+                      abmw->apply_button = 
+                      gtk_button_new_from_stock(GTK_STOCK_APPLY));
+    g_signal_connect(G_OBJECT(abmw->apply_button), "clicked",
                      G_CALLBACK(apply_button_cb), (gpointer) NULL);
-     gtk_container_add(GTK_CONTAINER(box),
-                      button=gtk_button_new_from_stock(GTK_STOCK_REMOVE));
-    g_signal_connect(G_OBJECT(button), "clicked",
+    gtk_container_add(GTK_CONTAINER(box),
+                      abmw->remove_button
+                      =gtk_button_new_from_stock(GTK_STOCK_REMOVE));
+    g_signal_connect(G_OBJECT(abmw->remove_button), "clicked",
                      G_CALLBACK(remove_button_cb), (gpointer) NULL);
     gtk_container_add(GTK_CONTAINER(box),
-                      button=gtk_button_new_from_stock(GTK_STOCK_CANCEL));
-    g_signal_connect(G_OBJECT(button), "clicked",
-                     G_CALLBACK(cancel_button_cb), (gpointer) NULL);
+                      abmw->cancel_button = 
+                      gtk_button_new_from_stock(GTK_STOCK_CANCEL));
+    g_signal_connect(G_OBJECT(abmw->cancel_button), "clicked",
+                     G_CALLBACK(cancel_button_cb), abmw);
     return box;
 }
 
+static void
+bab_filter_entry_activate(GtkWidget *entry, GtkWidget *button)
+{
+    const gchar *filter = gtk_entry_get_text(GTK_ENTRY(entry));
+    bab_set_address_book(contacts_app.address_book, contacts_app.entry_list,
+                         filter);
+    gtk_widget_set_sensitive(button, FALSE);
+}
 
+static void
+bab_filter_entry_changed(GtkWidget *entry, GtkWidget *button)
+{
+    gtk_widget_set_sensitive(button, TRUE);
+}
+
+static GtkWidget*
+bab_get_filter_box(void)
+{
+    GtkWidget *search_hbox = gtk_hbox_new(FALSE, 1);
+    GtkWidget *find_label, *find_entry, *button;
+
+    gtk_widget_show(search_hbox);
+    find_label = gtk_label_new_with_mnemonic(_("F_ilter:"));
+    gtk_widget_show(find_label);
+    gtk_box_pack_start(GTK_BOX(search_hbox), find_label, FALSE, FALSE, 1);
+    find_entry = gtk_entry_new();
+    gtk_widget_show(find_entry);
+    gtk_box_pack_start(GTK_BOX(search_hbox), find_entry, TRUE, TRUE, 1);
+    gtk_widget_show(search_hbox);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(find_label), find_entry);
+    button = gtk_button_new();
+    gtk_container_add(GTK_CONTAINER(button),
+                      gtk_image_new_from_stock(GTK_STOCK_OK,
+                                               GTK_ICON_SIZE_BUTTON));
+    gtk_box_pack_start(GTK_BOX(search_hbox), button, FALSE, FALSE, 1);
+
+    g_signal_connect(G_OBJECT(find_entry), "activate",
+                     G_CALLBACK(bab_filter_entry_activate),
+                     button);
+    g_signal_connect_swapped(G_OBJECT(button), "clicked",
+                             G_CALLBACK(bab_filter_entry_activate),
+                             find_entry);
+    g_signal_connect(G_OBJECT(find_entry), "changed",
+                             G_CALLBACK(bab_filter_entry_changed),
+                             button);
+    return search_hbox;
+}
 
 static GtkWidget*
 bab_window_new()
 {
-    GtkWidget* menubar, *main_vbox, *search_hbox, *cont_box, *vbox, *scroll;
-    GtkWidget *find_label, *find_entry;
+    GtkWidget* menubar, *main_vbox, *cont_box, *vbox, *scroll;
     GtkWidget *wnd = gnome_app_new("Contacts", "Contacts");
     GList *first_ab;
 
@@ -382,19 +552,9 @@ bab_window_new()
     main_vbox = gtk_vbox_new(FALSE, 1);
 
     /* Entry widget for finding an address */
-    search_hbox = gtk_hbox_new(FALSE, 1);
-    gtk_widget_show(search_hbox);
-    gtk_box_pack_start(GTK_BOX(main_vbox), search_hbox, FALSE, FALSE, 1);
-
-    find_label = gtk_label_new_with_mnemonic(_("F_ilter:"));
-    gtk_widget_show(find_label);
-    gtk_box_pack_start(GTK_BOX(search_hbox), find_label, FALSE, FALSE, 1);
-    find_entry = gtk_entry_new();
-    gtk_widget_show(find_entry);
-    gtk_box_pack_start(GTK_BOX(search_hbox), find_entry, TRUE, TRUE, 1);
-    gtk_widget_show(search_hbox);
-    gtk_label_set_mnemonic_widget(GTK_LABEL(find_label), find_entry);
-
+    gtk_box_pack_start(GTK_BOX(main_vbox),
+                       bab_get_filter_box(), FALSE, FALSE, 1);
+ 
     cont_box = gtk_hbox_new(FALSE, 1);
     gtk_box_pack_start(GTK_BOX(main_vbox), cont_box, TRUE,TRUE, 1);
 
@@ -404,7 +564,7 @@ bab_window_new()
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     gtk_box_pack_start(GTK_BOX(cont_box), scroll, TRUE,TRUE, 1);
     
-    contacts_app.entry_list = bab_window_list_new(NULL);
+    contacts_app.entry_list = bab_window_list_new(&contacts_app);
     gtk_container_add(GTK_CONTAINER(scroll), contacts_app.entry_list);
 
     vbox = gtk_vbox_new(FALSE, 1);
@@ -412,8 +572,9 @@ bab_window_new()
     contacts_app.edit_box = gtk_vbox_new(FALSE, 1);
     gtk_box_pack_start(GTK_BOX(vbox), contacts_app.edit_box,
                        TRUE,TRUE, 1);
-    gtk_box_pack_start(GTK_BOX(vbox), bab_get_edit_button_box(),
-                       TRUE,TRUE, 1);
+    gtk_box_pack_start(GTK_BOX(vbox),
+                       bab_get_edit_button_box(&contacts_app),
+                       FALSE, FALSE, 1);
     /*
     g_signal_connect(G_OBJECT(find_entry), "changed",
 		     G_CALLBACK(balsa_ab_window_find), ab);
@@ -424,7 +585,7 @@ bab_window_new()
     first_ab = g_list_first(contacts_app.address_book_list);
     if(first_ab)
         bab_set_address_book(LIBBALSA_ADDRESS_BOOK(first_ab->data),
-                             contacts_app.entry_list);
+                             contacts_app.entry_list, NULL);
     return wnd;
 }
 
@@ -434,6 +595,23 @@ bab_delete_ok(void)
     return FALSE;
 }
 /* -------------------------- main --------------------------------- */
+static GtkWidget *ab_window = NULL;
+static void
+ab_warning(const char *fmt, ...)
+{
+    GtkWidget *d;
+    va_list va_args;
+    char *msg;
+    va_start(va_args, fmt);
+    msg =  g_strdup_vprintf(fmt, va_args);
+    va_end(va_args);
+    d = gtk_message_dialog_new(GTK_WINDOW(ab_window),
+                               GTK_DIALOG_DESTROY_WITH_PARENT,
+                               GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, msg);
+    gtk_dialog_run(GTK_DIALOG(d));
+    gtk_widget_destroy(d);
+}
+
 static void
 bab_init(void)
 {
@@ -442,6 +620,9 @@ bab_init(void)
     LIBBALSA_TYPE_ADDRESS_BOOK_LDIF;
 #if ENABLE_LDAP
     LIBBALSA_TYPE_ADDRESS_BOOK_LDAP;
+#endif
+#if HAVE_SQLITE
+    LIBBALSA_TYPE_ADDRESS_BOOK_GPE;
 #endif
     memset(&contacts_app, 0, sizeof(contacts_app));
 }
@@ -449,7 +630,6 @@ bab_init(void)
 int
 main(int argc, char *argv[])
 {
-    GtkWidget *window;
     GnomeClient *client;
 #ifdef GTKHTML_HAVE_GCONF
     GError *gconf_error;
@@ -485,10 +665,10 @@ main(int argc, char *argv[])
     /* load address book data */
     bab_config_init();
 
-    window = bab_window_new();
-    g_signal_connect(G_OBJECT(window), "destroy",
+    ab_window = bab_window_new();
+    g_signal_connect(G_OBJECT(ab_window), "destroy",
                      G_CALLBACK(bab_cleanup), NULL);
-    g_signal_connect(G_OBJECT(window), "delete-event",
+    g_signal_connect(G_OBJECT(ab_window), "delete-event",
                      G_CALLBACK(bab_delete_ok), NULL);
 
     /* session management */
@@ -498,7 +678,7 @@ main(int argc, char *argv[])
     g_signal_connect(G_OBJECT(client), "die",
 		     G_CALLBACK(bab_kill_session), NULL);
 
-    gtk_widget_show_all(window);
+    gtk_widget_show_all(ab_window);
 
     gdk_threads_enter();
     gtk_main();
