@@ -23,7 +23,12 @@
 
 #include "libbalsa.h"
 
-LibBalsaAddressBook *balsa_address_book_config_new(LibBalsaAddressBook *
-						   address_book);
+typedef void BalsaAddressBookCallback(LibBalsaAddressBook * address_book,
+                                      gboolean append);
+void balsa_address_book_config_new(LibBalsaAddressBook * address_book,
+				   BalsaAddressBookCallback callback,
+                                   GtkWindow * parent);
+GtkWidget *balsa_address_book_add_menu(BalsaAddressBookCallback callback,
+                                       GtkWindow * parent);
 
 #endif				/* __ADDRESS_BOOK_CONFIG_H__ */
