@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,9 @@
 #include "main-window.h"
 #include "toolbar-factory.h"
 
+/* misc.h for LibBalsaCodeset */
+#include "misc.h"
+
 #if ENABLE_ESMTP
 #include <libesmtp.h>			/* part of libESMTP */
 #include <auth-client.h>		/* part of libESMTP */
@@ -53,31 +56,28 @@
 #define NUM_DEFAULT_WIDTH 30
 #define STATUS_DEFAULT_WIDTH 16
 #define ATTACHMENT_DEFAULT_WIDTH 16
-#define FROM_DEFAULT_WIDTH 160
-#define SUBJECT_DEFAULT_WIDTH 250
+#define FROM_DEFAULT_WIDTH 128
+#define SUBJECT_DEFAULT_WIDTH 180
 #define DATE_DEFAULT_WIDTH 128
 #define SIZE_DEFAULT_WIDTH 40
 
 /* default width settings for the mailbox list columns, not fully utilized yet */
 #define MBNAME_DEFAULT_WIDTH 80
 
-#define NEWMSGCOUNT_DEFAULT_WIDTH 45
-#define TOTALMSGCOUNT_DEFAULT_WIDTH 45
+#define NEWMSGCOUNT_DEFAULT_WIDTH 20
+#define TOTALMSGCOUNT_DEFAULT_WIDTH 25
 #define INFO_FIELD_LENGTH 10
 
-/* Default colour for mailboxes with unread messages */
-#define MBLIST_UNREAD_COLOR "rgb:FFFF/0000/0000"
-
 /*
  * Default colour for quoted text
  * oh no, I used the US spelling.
  */
-#define MAX_QUOTED_COLOR 6
-#define DEFAULT_QUOTED_COLOR "rgb:0000/5000/5000"
-#define DEFAULT_QUOTE_REGEX  "^(([ \tA-Z])\1*[|>:}#])"
+#define MAX_QUOTED_COLOR 2
+#define DEFAULT_QUOTED_COLOR "#055"
+#define DEFAULT_QUOTE_REGEX  "^[ \\t]*[|>:}#]"
 
-#define DEFAULT_URL_COLOR    "rgb:A000/0000/0000"
-#define DEFAULT_BAD_ADDRESS_COLOR    "rgb:8000/8000/0000"
+#define DEFAULT_URL_COLOR    "dark blue"
+#define DEFAULT_BAD_ADDRESS_COLOR    "red"
 
 #define MAILBOX_MANAGER_WIDTH 350
 #define MAILBOX_MANAGER_HEIGHT 400
@@ -85,15 +85,13 @@
 #define MESSAGEBOX_WIDTH 450
 #define MESSAGEBOX_HEIGHT 150
 
-#define DEFAULT_MESSAGE_FONT "-*-fixed-medium-r-semicondensed-*-13-*-*-*-c-*-iso8859-1"
-#define DEFAULT_SUBJECT_FONT "-*-fixed-bold-r-normal-*-*-*-*-*-c-*-iso8859-1"
-#define DEFAULT_DATE_FORMAT "%Y.%m.%d %H:%M"
+#define DEFAULT_MESSAGE_FONT "monospace 10"
+#define DEFAULT_SUBJECT_FONT "helvetica Bold 10"
+#define DEFAULT_DATE_FORMAT "%x %X"
 #define DEFAULT_PAPER_SIZE "A4"
-#define DEFAULT_PRINT_HEADER_FONT "Helvetica"
-#define DEFAULT_PRINT_HEADER_SIZE 11.0
-#define DEFAULT_PRINT_FOOTER_SIZE 7.0
-#define DEFAULT_PRINT_BODY_FONT "Courier"
-#define DEFAULT_PRINT_BODY_SIZE 10.0
+#define DEFAULT_PRINT_HEADER_FONT "Bitstream Charter Bold 12"
+#define DEFAULT_PRINT_BODY_FONT "Courier Regular 11"
+#define DEFAULT_PRINT_FOOTER_FONT "Bitstream Charter Bold Italic 8"
 #define DEFAULT_SELECTED_HDRS "from to date cc subject"
 #define DEFAULT_MESSAGE_TITLE_FORMAT "Message from %F: %s"
 #define DEFAULT_ENCODING ENC8BIT
@@ -105,6 +103,8 @@
 #define DEFAULT_CHECK_SIG FALSE
 #define DEFAULT_CHECK_QUOTED FALSE
 
+#define DEFAULT_BROKEN_CODESET "0"   /* == west european */
+
 
 enum {
     WHILERETR,
@@ -152,6 +152,12 @@ enum _BalsaMDNReply {
     BALSA_MDN_REPLY_ALWAYS,
 };
 
+typedef enum _MwActionAfterMove MwActionAfterMove;
+enum _MwActionAfterMove {
+    NEXT_UNREAD,
+    NEXT,
+    CLOSE
+};
 
 /* global balsa application structure */
 extern struct BalsaApplication {
@@ -171,11 +177,13 @@ extern struct BalsaApplication {
     gchar *smtp_certificate_passphrase;
 #endif
 
-    /* IMAP folder scanning */
+    /* folder scanning */
+    guint local_scan_depth;
     guint imap_scan_depth;
 
     BalsaWindow *main_window;
     BalsaMBList *mblist;
+    GtkTreeStore *mblist_tree_store;
 
     LibBalsaMailbox *inbox;
     GList *inbox_input;		/* mailboxes such as POP3, etc that will be appending into inbox */
@@ -184,7 +192,7 @@ extern struct BalsaApplication {
     LibBalsaMailbox *outbox;
     LibBalsaMailbox *trash;
 
-    GNode *mailbox_nodes;
+    BalsaMailboxNode *root_node;
 
     /* timer for mm_exists callback */
     gint new_messages_timer;
@@ -205,10 +213,16 @@ extern struct BalsaApplication {
 
     /* automatically close mailboxes after XX minutes */
     gboolean close_mailbox_auto;
-    gint close_mailbox_timeout;
+    gint close_mailbox_timeout; /* seconds */
+
+    /* automatically expunge mailboxes after XX hours */
+    gboolean expunge_auto;
+    gint expunge_timeout;	/* seconds */
+
     gint check_imap;
     gint check_imap_inbox;
     gint quiet_background_check;
+    gint msg_size_limit; /* for POP mailboxes; in kB */
 
     /* GUI settings */
     gint mw_width;
@@ -216,10 +230,14 @@ extern struct BalsaApplication {
     gint mblist_width;
     gint sw_width; /* sendmsg window */
     gint sw_height;
-    int toolbar_count;
-    BalsaToolbarType *toolbar_ids; 
-    char ***toolbars;     /* toolbar_ids[i] is a type of toolbars[i] */
+    gint message_window_width;
+    gint message_window_height;
+
+    /* toolbars */
     int toolbar_wrap_button_text;
+    GSList *main_window_toolbar_current;
+    GSList *compose_window_toolbar_current;
+    GSList *message_window_toolbar_current;
 
     /* file paths */
     gchar *attach_dir;
@@ -246,7 +264,6 @@ extern struct BalsaApplication {
     /* Colour of mailboxes with unread messages in mailbox list */
     GdkVisual *visual;
     GdkColormap *colormap;
-    GdkColor mblist_unread_color;
 
     /* Colour of quoted text. */
     gchar *quote_regex;
@@ -258,32 +275,35 @@ extern struct BalsaApplication {
     /* label color of bad addresses */
     GdkColor bad_address_color;
 
-    GtkToolbarStyle toolbar_style;
-    GnomeMDIMode mdi_style;
     guint pwindow_option;
     gboolean wordwrap;
     gint wraplength;
     gboolean browse_wrap;
     gint browse_wrap_length;
     ShownHeaders shown_headers;
+    gboolean show_all_headers;
     gchar *selected_headers;
-    gchar *message_title_format;
-    BalsaIndexThreadingType threading_type;
     gboolean expand_tree;
     gboolean show_mblist;
     gboolean show_notebook_tabs;
     gboolean alternative_layout;
     gboolean view_message_on_open;
-    gboolean line_length;
     gboolean pgdownmod;
     gint pgdown_percent;
+#if defined(ENABLE_TOUCH_UI)
+    gboolean do_file_format_check; /* do file format check on attaching */
+    gboolean enable_view_filter;   /* enable quick view filter */
+#endif
 
     gboolean empty_trash_on_exit;
     gboolean previewpane;
+    gboolean source_escape_specials;
     gboolean debug;
 
+    /* what to do with message window after moving the message */
+    MwActionAfterMove mw_action_after_move;
+
     /* external editor */
-    gchar *extern_editor_command;
     gboolean edit_headers;
 
     /* arp --- string to prefix "replied to" messages. */
@@ -303,8 +323,10 @@ extern struct BalsaApplication {
     gint check_mail_upon_startup;
     gint remember_open_mboxes;
     gint open_unread_mailbox;
-    GList *open_mailbox_list;	/* data is a pointer to the mailbox */
 
+    /* list of currently open mailboxes */
+    GList *open_mailbox_list;  /* data is a pointer to the mailbox */
+    
     /* font used to display messages */
     gchar *message_font;
     gchar *subject_font;
@@ -315,18 +337,21 @@ extern struct BalsaApplication {
 
     /* printing */
     gchar* paper_size; /* A4 or Letter */
-    gchar* print_header_font;  /* font for printing headers and footers */
-    gfloat print_header_size;  /* size fpr printing headers */
-    gfloat print_footer_size;  /* size for printing footers */
+    gchar* margin_left;
+    gchar* margin_top;
+    gchar* margin_right;
+    gchar* margin_bottom;
+    gchar* print_unit;
+    gchar* print_layout;
+    gchar* paper_orientation;
+    gchar* page_orientation;
+    gchar* print_header_font;  /* font for printing headers */
     gchar* print_body_font;    /* font for printing text parts */
-    gfloat print_body_size;    /* size for printing text parts */
+    gchar* print_footer_font;  /* font for printing footers */
     gboolean print_highlight_cited;
 
-    /* compose: shown headers */
+    /* compose */
     gchar *compose_headers;
-
-    /* compose: request a disposition notification */
-    gboolean req_dispnotify;   
     gboolean always_queue_sent_mail;
     gboolean copy_to_sentbox;
 
@@ -358,48 +383,56 @@ extern struct BalsaApplication {
     /* how to act if a MDN request is received */
     BalsaMDNReply mdn_reply_clean;
     BalsaMDNReply mdn_reply_notclean;
-    gboolean recognize_rfc2646_format_flowed;
-    gboolean send_rfc2646_format_flowed;
+
+    /* how to handle multipart/alternative */
+    gboolean display_alt_plain;
+
+    /* how to handle broken mails with 8-bit chars */
+    gboolean convert_unknown_8bit;
+    LibBalsaCodeset convert_unknown_8bit_codeset;
+
+#ifdef HAVE_GPGME
+    /* gpgme stuff */
+    gboolean has_openpgp;
+    gboolean has_smime;
+#endif 
 
     GList *folder_mru;
     GList *fcc_mru;
-    gint drag_default_is_move;
-    gboolean delete_immediately;
-    gboolean hide_deleted;
+    gboolean expunge_on_close;
+
+    /* use as default email client for GNOME */
+    int default_client;
 
 } balsa_app;
 
 #define BALSA_IS_MAILBOX_SPECIAL(a) ((a)==balsa_app.inbox || (a)==balsa_app.trash || (a)==balsa_app.outbox||(a)==balsa_app.draftbox || (a)==balsa_app.sentbox)
 
 void balsa_app_init(void);
-gboolean do_load_mailboxes(void);
+void balsa_app_destroy(void);
 void update_timer(gboolean update, guint minutes);
 
 gchar *ask_password(LibBalsaServer * server, LibBalsaMailbox * mbox);
-GtkWidget *gnome_stock_button_with_label(const char *icon,
+GtkWidget *balsa_stock_button_with_label(const char *icon,
 					 const char *label);
 gboolean open_mailboxes_idle_cb(gchar * names[]);
 
 /* Search functions */
-GNode *balsa_find_mailbox(GNode * root, LibBalsaMailbox * mailbox);
-GNode *balsa_find_mbnode(GNode * root, BalsaMailboxNode * mbnode);
-GNode *balsa_find_dir(GNode * root, const gchar * path);
-GNode *balsa_find_path(GNode * root, const gchar * path);
-GNode *balsa_find_url(GNode * root, const gchar * url);
+BalsaMailboxNode *balsa_find_mailbox(LibBalsaMailbox * mailbox);
+BalsaMailboxNode *balsa_find_dir(LibBalsaServer *server, const gchar * path);
+BalsaMailboxNode *balsa_find_url(const gchar * url);
 LibBalsaMailbox *balsa_find_mailbox_by_url(const gchar * url);
-LibBalsaMailbox *balsa_find_mailbox_by_name(const gchar * name);
-
-void  balsa_remove_children_mailbox_nodes(GNode* gnode);
+LibBalsaMailbox *balsa_find_sentbox_by_url(const gchar * url);
+gboolean balsa_find_iter_by_data(GtkTreeIter * iter, gpointer data);
 BalsaIndex* balsa_find_index_by_mailbox(LibBalsaMailbox* mailbox);
 
-GtkWidget *create_label(const gchar * label, GtkWidget * table, 
-			       gint row, guint *keyval);
-GtkWidget *create_entry(GnomeDialog *mcw, GtkWidget * table, 
-			GtkSignalFunc func, gpointer data, gint row, 
-			const gchar * initval, const guint keyval);
-GtkWidget *create_check(GnomeDialog *mcw, const gchar * label, 
-			GtkWidget * table, gint row, gboolean initval);
-gchar *balsa_charset_from_message_font(void);
+void  balsa_remove_children_mailbox_nodes(BalsaMailboxNode * mbnode);
 
+GtkWidget *create_label(const gchar * label, GtkWidget * table, gint row);
+GtkWidget *create_entry(GtkDialog *mcw, GtkWidget * table, 
+			GtkSignalFunc func, gpointer data, gint row, 
+			const gchar * initval, GtkWidget* hotlabel);
+GtkWidget *create_check(GtkDialog *mcw, const gchar * label, 
+			GtkWidget * table, gint row, gboolean initval);
 
 #endif				/* __BALSA_APP_H__ */
