@@ -53,26 +53,24 @@ extern "C" {
         GtkWidget *delete_item;
         GtkWidget *undelete_item;
         GtkWidget *move_to_trash_item;
+        GtkWidget *toggle_item;
         GtkWidget *move_to_item;
 
         BalsaMailboxNode* mailbox_node;
         LibBalsaMessage* first_new_message;
         LibBalsaMessage* current_message;
+	gboolean current_message_is_deleted;
         gboolean prev_message;
         gboolean next_message;
-
-        GTimeVal last_use;
-
-	gchar *date_string;
-	gboolean line_length;
+	GArray *selected;
+        gchar *sos_filter; /* SenderOrSubject filter string, if any */
 
         /* signal handler ids */
         gulong selection_changed_id;
         gulong row_expanded_id;
         gulong row_collapsed_id;
 
-        /* A table with a row reference for each message. */
-        GHashTable *ref_table;
+	LibBalsaMailboxSearchIter *search_iter;
     };
 
     struct _BalsaIndexClass {
@@ -104,12 +102,17 @@ extern "C" {
 /* sets the mail stream; if it's a new stream, then it's 
  * contents is loaded into the index */
     gboolean balsa_index_load_mailbox_node(BalsaIndex * bindex,
-                                           BalsaMailboxNode * mbnode);
+                                           BalsaMailboxNode * mbnode,
+					   GError **err);
+    void balsa_index_scroll_on_open(BalsaIndex *index);
     void balsa_index_update_tree(BalsaIndex *bindex, gboolean expand);
     void balsa_index_set_threading_type(BalsaIndex * bindex, int thtype);
+    void balsa_index_set_sos_filter(BalsaIndex *bindex,
+                                    const gchar *sos_filter,
+                                    LibBalsaCondition *flag_filter);
 
 /* move or copy a list of messages */
-    void balsa_index_transfer(BalsaIndex * index, GList * messages,
+    void balsa_index_transfer(BalsaIndex * index, GArray * msgnos,
                               LibBalsaMailbox * to_mailbox, gboolean copy);
 
 /* select up/down the index */
@@ -117,9 +120,11 @@ extern "C" {
     void balsa_index_select_next_unread(BalsaIndex * bindex);
     void balsa_index_select_next_flagged(BalsaIndex * bindex);
     void balsa_index_select_previous(BalsaIndex *);
+    void balsa_index_select(BalsaIndex * index, LibBalsaMessage * message);
 
-    void balsa_index_find(BalsaIndex * bindex, FilterOpType op,
-                          GSList * conditions, gboolean previous);
+    void balsa_index_find(BalsaIndex * bindex,
+			  LibBalsaMailboxSearchIter * search_iter,
+                          gboolean previous, gboolean wrap);
 
 /* balsa index page stuff */
     void balsa_message_reply(GtkWidget * widget, gpointer user_data);
@@ -133,11 +138,9 @@ extern "C" {
     void balsa_message_continue(GtkWidget * widget, gpointer data);
 
     void balsa_message_move_to_trash(GtkWidget * widget, gpointer user_data);
-    void balsa_message_delete(GtkWidget * widget, gpointer user_data);
-    void balsa_message_undelete(GtkWidget * widget, gpointer user_data);
 
-    void balsa_message_toggle_flagged(GtkWidget * widget, gpointer user_data);
-    void balsa_message_toggle_new(GtkWidget * widget, gpointer user_data);
+    void balsa_index_toggle_flag(BalsaIndex *index,
+                                 LibBalsaMessageFlag flag);
 
     void balsa_index_reset(BalsaIndex * index);
     gint balsa_find_notebook_page_num(LibBalsaMailbox * mailbox);
@@ -151,17 +154,17 @@ extern "C" {
     void balsa_index_refresh_date (BalsaIndex * index);
     void balsa_index_refresh_size (BalsaIndex * index);
 
-    /* Change the display of all indexes when balsa_app.hide_deleted is
-     * changed */
-    void balsa_index_hide_deleted(gboolean hide);
-
-    /* Threading Stuff, implementation is in balsa-index-threading.c */
-    void balsa_index_threading(BalsaIndex* bindex,
-			       LibBalsaMailboxThreadingType th_type);
-
     /* Remove duplicate messages. */
     void balsa_index_remove_duplicates(BalsaIndex * index);
 
+    /* Expunge deleted messages. */
+    void balsa_index_expunge(BalsaIndex * index);
+ 
+    /* Message window */ 	 
+    guint balsa_index_next_msgno(BalsaIndex * index, guint current_msgno);
+    guint balsa_index_previous_msgno(BalsaIndex * index,
+                                     guint current_msgno);
+
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
