@@ -41,6 +41,8 @@
 #include "balsa-app.h"
 #include "save-restore.h"
 #include "mailbox-filter.h"
+#include "i18n.h"
+#include "libbalsa-conf.h"
 
 /* Defined in filter-edit-dialog.c*/
 extern option_list fe_search_type[4];
@@ -81,7 +83,7 @@ GtkWidget *fe_matching_fields_from;
 GtkWidget *fe_matching_fields_subject;
 GtkWidget *fe_matching_fields_cc;
 /* Combo list for user headers and check button*/
-GtkCombo * fe_user_header;
+GtkWidget * fe_user_header;
 GtkWidget * fe_matching_fields_us_head;
 
 /* widget for the conditions */
@@ -226,8 +228,12 @@ unique_filter_name(const gchar * name)
     selection = gtk_tree_view_get_selection(fe_filters_list);
     for (valid = gtk_tree_model_get_iter_first(model, &iter); valid;
          valid = gtk_tree_model_iter_next(model, &iter)) {
+	gboolean matches;
+
         gtk_tree_model_get(model, &iter, 0, &row_text, -1);
-        if (strcmp(name, row_text) == 0
+	matches = strcmp(name, row_text) == 0;
+	g_free(row_text);
+        if (matches
             && !gtk_tree_selection_iter_is_selected(selection, &iter))
             return FALSE;
     }
@@ -258,6 +264,18 @@ fe_regexs_selection_changed(GtkTreeSelection *selection,
     gtk_widget_set_sensitive(fe_regex_remove_button, selected);
 }
 
+/* Helper. */
+static gint
+fe_combo_box_get_value(GtkWidget * combo_box)
+{
+    struct fe_combo_box_info *info =
+        g_object_get_data(G_OBJECT(combo_box), BALSA_FE_COMBO_BOX_INFO);
+    /* Retrieve the selected item in the menu */
+    gint active = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_box));
+
+    return GPOINTER_TO_INT(g_slist_nth_data(info->values, active));
+}
+
 /*
  * fe_typesmenu_cb()
  *
@@ -267,7 +285,8 @@ fe_regexs_selection_changed(GtkTreeSelection *selection,
 static void
 fe_typesmenu_cb(GtkWidget* widget, gpointer data)
 {
-    ConditionMatchType type=GPOINTER_TO_INT(data);
+    ConditionMatchType type =
+	(ConditionMatchType) fe_combo_box_get_value(widget);
 
     condition_has_changed=TRUE;
     gtk_notebook_set_current_page(GTK_NOTEBOOK(fe_type_notebook),
@@ -323,12 +342,9 @@ update_condition_list_label(void)
 static ConditionMatchType
 get_condition_type(void)
 {
-    GtkWidget * menu;
-
-    /* Retrieve the selected item in the search type menu */
-    menu=gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(fe_search_option_menu))));
     /* Set the type associated with the selected item */
-    return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu),"value"));
+    return (ConditionMatchType)
+        fe_combo_box_get_value(fe_search_option_menu);
 }
 
 /* fe_negate_condition :handle pressing on the "Contain/Does not
@@ -338,7 +354,7 @@ fe_negate_condition(GtkWidget * widget, gpointer data)
 {
     condition_not = !condition_not;
     switch (get_condition_type()) {
-    case CONDITION_SIMPLE: 
+    case CONDITION_STRING: 
         fe_update_label(fe_type_simple_label, &simple_label); break;
     case CONDITION_REGEX:  
         fe_update_label(fe_type_regex_label,  &regex_label);  break;
@@ -347,6 +363,8 @@ fe_negate_condition(GtkWidget * widget, gpointer data)
     case CONDITION_FLAG:   
         fe_update_label(fe_type_flag_label,   &flags_label);
     case CONDITION_NONE:
+    case CONDITION_AND:
+    case CONDITION_OR:
         /* to avoid warnings */
 	break;
     }
@@ -382,9 +400,11 @@ fe_match_fields_buttons_cb(GtkWidget * widget, gpointer data)
 static void
 fe_match_field_user_header_cb(GtkWidget * widget)
 {
-    gtk_widget_set_sensitive(GTK_WIDGET(fe_user_header),
-			     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_matching_fields_us_head)));
-    condition_has_changed=TRUE;
+    GtkToggleButton *button =
+        GTK_TOGGLE_BUTTON(fe_matching_fields_us_head);
+    gtk_widget_set_sensitive(fe_user_header,
+                             gtk_toggle_button_get_active(button));
+    condition_has_changed = TRUE;
 }
 
 void
@@ -402,6 +422,43 @@ fe_add_new_user_header(const gchar * str)
                              (GCompareFunc) g_ascii_strcasecmp);
 }
 
+static void
+fe_user_header_set_active(const gchar * user_header)
+{
+    if (user_header && *user_header) {
+	GList *lst;
+        gint i;
+
+        for (lst = fe_user_headers_list, i = 0; lst; lst = lst->next, i++) {
+            if (g_ascii_strcasecmp(user_header, lst->data) == 0) {
+                gtk_combo_box_set_active(GTK_COMBO_BOX(fe_user_header), i);
+                break;
+            }
+        }
+    }
+}
+
+static void
+fe_user_header_add(const gchar * user_header)
+{
+    if (user_header && *user_header) {
+        GList *lst;
+
+        /* First clear the combo box... */
+        for (lst = fe_user_headers_list; lst; lst = lst->next)
+            gtk_combo_box_remove_text(GTK_COMBO_BOX(fe_user_header), 0);
+
+        fe_add_new_user_header(user_header);
+
+        /* ...then remake it with the new string... */
+        for (lst = fe_user_headers_list; lst; lst = lst->next)
+            gtk_combo_box_append_text(GTK_COMBO_BOX(fe_user_header),
+                                      lst->data);
+
+        fe_user_header_set_active(user_header);
+    }
+}
+
 /* conditon_validate is responsible of validating
  * the changes to the current condition, according to the widgets
  * Performs sanity check on the widgets 
@@ -413,7 +470,6 @@ fe_add_new_user_header(const gchar * str)
 static gboolean
 condition_validate(LibBalsaCondition* new_cnd)
 {
-    LibBalsaConditionRegex * new_reg;
     gchar * str,* p;
     const gchar *c_str;
     gint row, col;
@@ -441,24 +497,22 @@ condition_validate(LibBalsaCondition* new_cnd)
             CONDITION_SETMATCH(new_cnd,CONDITION_MATCH_CC);
         if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_matching_fields_us_head))) {
 	    CONDITION_SETMATCH(new_cnd,CONDITION_MATCH_US_HEAD);
-            str = gtk_editable_get_chars(GTK_EDITABLE(fe_user_header->entry),
-                                         0, -1);
+            str =
+                gtk_editable_get_chars(GTK_EDITABLE
+                                       (GTK_BIN(fe_user_header)->child), 0,
+                                       -1);
 	    if (!str[0]) {
                 balsa_information(LIBBALSA_INFORMATION_ERROR,
                                   _("You must specify the name of the "
                                     "user header to match on"));
 		return FALSE;
 	    }
-	    fe_add_new_user_header(str);
-	    /* This piece of code replaces the combo list
-	       by a new one that contains the new string the user has entered
-	       it seems that we must reset the text to the correct string,
-	    */
-	    gtk_combo_set_popdown_strings(fe_user_header,fe_user_headers_list);
-	    gtk_entry_set_text(GTK_ENTRY(fe_user_header->entry),str);
+            if (gtk_combo_box_get_active(GTK_COMBO_BOX(fe_user_header)) < 0)
+		/* User has changed the entry. */
+		fe_user_header_add(str);
 	    g_free(str);
 	}
-        else if (new_cnd->match_fields==CONDITION_EMPTY) {
+        else if (new_cnd->match.string.fields==CONDITION_EMPTY) {
             balsa_information(LIBBALSA_INFORMATION_ERROR,
                               _("You must specify at least one "
                                 "field for matching"));
@@ -466,7 +520,7 @@ condition_validate(LibBalsaCondition* new_cnd)
         }
     }
     switch (new_cnd->type) {
-    case CONDITION_SIMPLE:
+    case CONDITION_STRING:
         c_str = gtk_entry_get_text(GTK_ENTRY(fe_type_simple_entry));
         if (!c_str || c_str[0]=='\0') {
             balsa_information(LIBBALSA_INFORMATION_ERROR,
@@ -492,7 +546,7 @@ condition_validate(LibBalsaCondition* new_cnd)
                                   _("Low date is incorrect"));
                 return FALSE;
             }
-            new_cnd->match.interval.date_low=mktime(&date);
+            new_cnd->match.date.date_low=mktime(&date);
         }
         c_str = gtk_entry_get_text(GTK_ENTRY(fe_type_date_high_entry));
         if (c_str && c_str[0]!='\0') {
@@ -503,10 +557,10 @@ condition_validate(LibBalsaCondition* new_cnd)
                                   _("High date is incorrect"));
                 return FALSE;
             }
-            new_cnd->match.interval.date_high=mktime(&date);
+            new_cnd->match.date.date_high=mktime(&date);
         }
-        if (new_cnd->match.interval.date_low >
-            new_cnd->match.interval.date_high) {
+        if (new_cnd->match.date.date_low >
+            new_cnd->match.date.date_high) {
             balsa_information(LIBBALSA_INFORMATION_ERROR,
                               _("Low date is greater than high date"));
             return FALSE;
@@ -514,23 +568,30 @@ condition_validate(LibBalsaCondition* new_cnd)
         break;
     case CONDITION_FLAG:
     case CONDITION_NONE:
+    case CONDITION_AND:
+    case CONDITION_OR:
         /* to avoid warnings */
 	break;
     }
 
     /* Sanity checks OK, retrieve datas from widgets */
 
-    new_cnd->condition_not=condition_not;
+    new_cnd->negate = condition_not;
     if (CONDITION_CHKMATCH(new_cnd,CONDITION_MATCH_US_HEAD))
-	new_cnd->user_header=g_strdup(gtk_entry_get_text(GTK_ENTRY(fe_user_header->entry)));
+        new_cnd->match.string.user_header =
+            gtk_editable_get_chars(GTK_EDITABLE
+                                   (GTK_BIN(fe_user_header)->child), 0,
+                                   -1);
     /* Set the type specific fields of the condition */
     switch (new_cnd->type) {
-    case CONDITION_SIMPLE:
-        new_cnd->match.string =
+    case CONDITION_STRING:
+        new_cnd->match.string.string =
             g_strdup(gtk_entry_get_text(GTK_ENTRY(fe_type_simple_entry)));
         break;
 
     case CONDITION_REGEX:
+#if 0
+        FIXME;
         do {
             gchar* str;
             gtk_tree_model_get(model, &iter, 0, &str, -1);
@@ -538,8 +599,8 @@ condition_validate(LibBalsaCondition* new_cnd)
             libbalsa_condition_regex_set(new_reg, str);
             libbalsa_condition_prepend_regex(new_cnd, new_reg);
         } while (gtk_tree_model_iter_next(model, &iter));
+#endif
         break;
-
     case CONDITION_DATE:
         break;
 
@@ -551,6 +612,8 @@ condition_validate(LibBalsaCondition* new_cnd)
                     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_type_flag_buttons[row*2+col])) ? 1 << (row*2+col+1): 0;
 
     case CONDITION_NONE:
+    case CONDITION_AND: /*FIXME: verify this! */
+    case CONDITION_OR:
         /* To avoid warnings :) */
 	break;
     }
@@ -600,19 +663,15 @@ fill_condition_widgets(LibBalsaCondition* cnd)
 {
     GtkTreeModel *model =
         gtk_tree_view_get_model(fe_type_regex_list);
-    GtkTreeIter iter;
-    GtkTreeSelection *selection =
-        gtk_tree_view_get_selection(fe_type_regex_list);
-    GSList * regex;
     gchar str[20];
-    struct tm * date;
+    struct tm date;
     gint row,col;
     gboolean andmask;
     static gchar xformat[] = "%x"; /* to suppress error in strftime */
     
-    condition_not=cnd->condition_not;
+    condition_not=cnd->negate;
     /* Clear all widgets */
-    if (cnd->type!=CONDITION_SIMPLE)
+    if (cnd->type!=CONDITION_STRING)
         gtk_entry_set_text(GTK_ENTRY(fe_type_simple_entry),"");
 
     if (cnd->type!=CONDITION_REGEX)
@@ -623,8 +682,8 @@ fill_condition_widgets(LibBalsaCondition* cnd)
     gtk_notebook_set_current_page(GTK_NOTEBOOK(fe_type_notebook),
                                   cnd->type - 1);
 
-    gtk_option_menu_set_history(GTK_OPTION_MENU(fe_search_option_menu), 
-                                cnd->type-1);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(fe_search_option_menu),
+                             cnd->type - 1);
 
     /* First update matching fields
      * but if type is date or flag, these are meaning less so we disable them */
@@ -643,21 +702,24 @@ fill_condition_widgets(LibBalsaCondition* cnd)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_matching_fields_us_head),
                                  CONDITION_CHKMATCH(cnd,CONDITION_MATCH_US_HEAD) && andmask);
     if (CONDITION_CHKMATCH(cnd,CONDITION_MATCH_US_HEAD) && andmask) {
-	gtk_widget_set_sensitive(GTK_WIDGET(fe_user_header),TRUE);
-	gtk_entry_set_text(GTK_ENTRY(fe_user_header->entry),cnd->user_header ? cnd->user_header : "");
+	gtk_widget_set_sensitive(fe_user_header, TRUE);
+	fe_user_header_set_active(cnd->match.string.user_header);
     }
     else {
-	gtk_widget_set_sensitive(GTK_WIDGET(fe_user_header),FALSE);
-	gtk_entry_set_text(GTK_ENTRY(fe_user_header->entry),"");
+	gtk_widget_set_sensitive(fe_user_header,FALSE);
+	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(fe_user_header)->child),"");
     }	
     /* Next update type specific fields */
     switch (cnd->type) {
-    case CONDITION_SIMPLE:
+    case CONDITION_STRING:
         gtk_entry_set_text(GTK_ENTRY(fe_type_simple_entry),
-                           cnd->match.string==NULL ? "" : cnd->match.string);
+                           cnd->match.string.string 
+                           ? cnd->match.string.string : "");
         fe_update_label(fe_type_simple_label, &simple_label);
         break;
     case CONDITION_REGEX:
+#if 0
+        FIXME;
         for (regex = cnd->match.regexs; regex; regex = g_slist_next(regex)) {
             gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
             gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0,
@@ -671,18 +733,19 @@ fill_condition_widgets(LibBalsaCondition* cnd)
         } else
             gtk_widget_set_sensitive(fe_regex_remove_button, FALSE);
         fe_update_label(fe_type_regex_label, &regex_label);
+#endif
         break;
     case CONDITION_DATE:
-        if (cnd->match.interval.date_low==0) str[0]='\0';
+        if (cnd->match.date.date_low==0) str[0]='\0';
         else {
-            date=localtime(&cnd->match.interval.date_low);
-            strftime(str, sizeof(str), xformat, date);
+            localtime_r(&cnd->match.date.date_low, &date);
+            strftime(str, sizeof(str), xformat, &date);
         }
         gtk_entry_set_text(GTK_ENTRY(fe_type_date_low_entry),str);
-        if (cnd->match.interval.date_high==0) str[0]='\0';
+        if (cnd->match.date.date_high==0) str[0]='\0';
         else {
-            date=localtime(&cnd->match.interval.date_high);
-            strftime(str,sizeof(str), xformat, date);
+            localtime_r(&cnd->match.date.date_high, &date);
+            strftime(str,sizeof(str), xformat, &date);
         }
         gtk_entry_set_text(GTK_ENTRY(fe_type_date_high_entry),str);
         fe_update_label(fe_type_date_label, &date_label);
@@ -695,10 +758,13 @@ fill_condition_widgets(LibBalsaCondition* cnd)
         fe_update_label(fe_type_flag_label,   &flags_label);
         break;
     case CONDITION_NONE:
+    case CONDITION_AND:
+    case CONDITION_OR:
         /* To avoid warnings :), we should never get there */
 	break;
     }
-    gtk_menu_set_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(fe_search_option_menu))),cnd->type-1);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(fe_search_option_menu),
+                             cnd->type - 1);
 }            /* end fill_condition_widget */
 
 static void
@@ -962,6 +1028,7 @@ static
 void build_condition_dialog(GtkWidget * condition_dialog)
 {
     GtkWidget * table,* label,* button,* page,* box;
+    GList *list;
 
     page = gtk_table_new(7, 2, FALSE);
     /* builds the toggle buttons to specify fields concerned by the conditions of
@@ -1039,14 +1106,14 @@ void build_condition_dialog(GtkWidget * condition_dialog)
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
     g_signal_connect(G_OBJECT(fe_matching_fields_us_head), "toggled",
                      G_CALLBACK(fe_match_field_user_header_cb), NULL);
-    fe_user_header = GTK_COMBO(gtk_combo_new());
-    gtk_combo_set_value_in_list(fe_user_header,FALSE,FALSE);
-    gtk_combo_set_case_sensitive(fe_user_header,FALSE);
-    gtk_combo_set_popdown_strings(fe_user_header,fe_user_headers_list);
-    g_signal_connect(G_OBJECT(fe_user_header->entry), "changed",
+    fe_user_header = gtk_combo_box_entry_new_text();
+    for (list = fe_user_headers_list; list; list = list->next)
+        gtk_combo_box_append_text(GTK_COMBO_BOX(fe_user_header),
+                                  list->data);
+    g_signal_connect(G_OBJECT(fe_user_header), "changed",
                      G_CALLBACK(fe_condition_changed_cb), NULL);
     gtk_table_attach(GTK_TABLE(table),
-                     GTK_WIDGET(fe_user_header),
+                     fe_user_header,
                      1, 2, 3, 4,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
     
@@ -1192,72 +1259,85 @@ fe_condition_remove_pressed(GtkWidget * widget, gpointer data)
  * to replace filters name that have changed in the filters list of mailboxes
  * and to invalidate previously constructed list of filters, to force reloading
  */
-static
-gboolean update_filters_mailbox(GNode * node,gpointer throwaway)
+static gboolean
+update_filters_mailbox(GtkTreeModel * model, GtkTreePath * path,
+		       GtkTreeIter * iter, gpointer data)
 {
-    BalsaMailboxNode *mbnode = (BalsaMailboxNode *) node->data;
-    gchar * tmp;
+    BalsaMailboxNode *mbnode;
+    LibBalsaMailbox *mailbox;
+    gchar *tmp;
 
-    if (mbnode->mailbox) {
-        /* First we free the filters list (which is now obsolete) */
-        g_slist_free(mbnode->mailbox->filters);
-        mbnode->mailbox->filters=NULL;
-        /* Second we replace old filters name by the new ones
-         * Note : deleted filters are also removed */
-        if (!filters_names_changes) 
-            return FALSE;
-        tmp=mailbox_filters_section_lookup(mbnode->mailbox->url ? mbnode->mailbox->url : mbnode->mailbox->name);
-        if (tmp) {
-            gchar **filters_names=NULL;
-            gboolean def;
-            guint nb_filters;
-            
-            gnome_config_push_prefix(tmp);
-            gnome_config_get_vector_with_default(MAILBOX_FILTERS_KEY,&nb_filters,&filters_names,&def);
-            if (!def) {
-                guint i;
-                GList * lst;
-                
-                for (i=0;i<nb_filters;) {
-                    for (lst=filters_names_changes;
-                         lst &&
-                         strcmp(((filters_names_rec *)lst->data)->old_name,filters_names[i])!=0;
-                         lst=g_list_next(lst));
+    gtk_tree_model_get(model, iter, 0, &mbnode, -1);
+    mailbox = mbnode->mailbox;
+    g_object_unref(mbnode);
+    if (!mailbox)
+	return FALSE;
 
-                        if (lst) {
-                            g_free(filters_names[i]);
-                            if (((filters_names_rec *)lst->data)->new_name) {
-                                /* Name changing */
-                                filters_names[i++]=g_strdup(((filters_names_rec *)lst->data)->new_name);
-                            }
-                            else {
-                                /* Name removing */
-                                guint j;
-                                
-                                for (j=i;j<nb_filters-1;j++)
-                                    filters_names[j]=filters_names[j+1];
-                                /* We put NULL to be sure that
-                                 * g_strfreev does not free already
-                                 * freed memory. */
-                                filters_names[--nb_filters]=NULL;
-                            }
-                        }
-                        else i++;
-                }
-            }
-            
-            if (nb_filters) {
-                gnome_config_set_vector(MAILBOX_FILTERS_KEY,nb_filters,(const gchar **)filters_names);      
-                gnome_config_pop_prefix();
-            }
-            else {
-                gnome_config_pop_prefix();
-                gnome_config_clean_section(tmp);
-            }
-            g_strfreev(filters_names);
-            g_free(tmp);
-        }
+    /* First we free the filters list (which is now obsolete) */
+    g_slist_foreach(mailbox->filters, (GFunc) g_free, NULL);
+    g_slist_free(mailbox->filters);
+    mailbox->filters = NULL;
+    /* Second we replace old filters name by the new ones
+     * Note : deleted filters are also removed */
+    if (!filters_names_changes)
+	return FALSE;
+    tmp = mailbox_filters_section_lookup(mailbox->url ?
+					 mailbox->url : mailbox->name);
+    if (tmp) {
+	gchar **filters_names = NULL;
+	gboolean def;
+	guint nb_filters;
+
+	libbalsa_conf_push_prefix(tmp);
+	libbalsa_conf_get_vector_with_default(MAILBOX_FILTERS_KEY,
+					     &nb_filters, &filters_names,
+					     &def);
+	if (!def) {
+	    guint i;
+	    GList *lst;
+
+	    for (i = 0; i < nb_filters;) {
+		for (lst = filters_names_changes;
+		     lst &&
+		     strcmp(((filters_names_rec *) lst->data)->old_name,
+			    filters_names[i]) != 0;
+		     lst = g_list_next(lst));
+
+		if (lst) {
+		    g_free(filters_names[i]);
+		    if (((filters_names_rec *) lst->data)->new_name) {
+			/* Name changing */
+			filters_names[i++] =
+			    g_strdup(((filters_names_rec *) lst->data)->
+				     new_name);
+		    } else {
+			/* Name removing */
+			guint j;
+
+			for (j = i; j < nb_filters - 1; j++)
+			    filters_names[j] = filters_names[j + 1];
+			/* We put NULL to be sure that
+			 * g_strfreev does not free already
+			 * freed memory. */
+			filters_names[--nb_filters] = NULL;
+		    }
+		} else
+		    i++;
+	    }
+	}
+
+	if (nb_filters) {
+	    libbalsa_conf_set_vector(MAILBOX_FILTERS_KEY, nb_filters,
+				    (const gchar **) filters_names);
+	    libbalsa_conf_pop_prefix();
+	} else {
+	    libbalsa_conf_pop_prefix();
+	    libbalsa_conf_clean_section(tmp);
+	}
+	g_strfreev(filters_names);
+	g_free(tmp);
     }
+
     return FALSE;
 }
 
@@ -1340,11 +1420,9 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
                           GINT_TO_POINTER(TRUE));
 
         /* Update mailboxes filters */
-        balsa_mailbox_nodes_lock(TRUE);
-        g_node_traverse(balsa_app.mailbox_nodes,
-                        G_LEVEL_ORDER,
-                        G_TRAVERSE_ALL, 10, update_filters_mailbox, NULL);
-        balsa_mailbox_nodes_unlock(TRUE);
+	gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
+			       (GtkTreeModelForeachFunc)
+			       update_filters_mailbox, NULL);
 
         config_filters_save();
 
@@ -1379,8 +1457,10 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
 void
 fe_action_selected(GtkWidget * widget, gpointer data)
 {
+    FilterActionType type =
+        (FilterActionType) fe_combo_box_get_value(widget);
     gtk_widget_set_sensitive(GTK_WIDGET(fe_mailboxes),
-                             GPOINTER_TO_INT(data)!=FILTER_TRASH);
+                             type != FILTER_TRASH);
     set_button_sensitivities(TRUE);
 }                       /* end fe_action_selected() */
 
@@ -1584,7 +1664,6 @@ fe_new_pressed(GtkWidget * widget, gpointer data)
     /* Fill the filter with default values */
 
     fil->name=g_strdup(new_item);
-    fil->conditions_op=FILTER_OP_OR;
 
     FILTER_SETFLAG(fil,FILTER_COMPILED);
     fil->action=FILTER_MOVE;
@@ -1637,7 +1716,12 @@ fe_delete_pressed(GtkWidget * widget, gpointer data)
         gtk_entry_set_text(GTK_ENTRY(fe_name_entry),"");
         gtk_entry_set_text(GTK_ENTRY(fe_popup_entry),"");
         /*gtk_option_menu_set_history(GTK_OPTION_MENU(fe_mailboxes), 0); */
+#if GTK_CHECK_VERSION(2, 6, 0)
+        gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(fe_sound_entry),
+                                      "");
+#else /* GTK_CHECK_VERSION(2, 6, 0) */
         gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fe_sound_entry))),"");
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
         gtk_list_store_clear(GTK_LIST_STORE
                              (gtk_tree_view_get_model
                               (fe_conditions_list)));
@@ -1666,8 +1750,8 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
     GtkTreeIter cond_iter;
     LibBalsaFilter *fil,*old;
     const gchar *temp;
-    GtkWidget * menu;
     FilterActionType action;
+    ConditionMatchType condition_op;
 
     if (!gtk_tree_selection_get_selected(selection, &model, &iter))
         return;
@@ -1685,12 +1769,9 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
         return;
     }
     
-    /* Retrieve the selected item in the action menu */
-    menu = gtk_menu_get_active(GTK_MENU
-                               (gtk_option_menu_get_menu
-                                (GTK_OPTION_MENU(fe_action_option_menu))));
     /* Set the type associated with the selected item */
-    action=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu),"value"));
+    action =
+        (FilterActionType) fe_combo_box_get_value(fe_action_option_menu);
     
     if (!gtk_tree_model_get_iter_first(cond_model, &cond_iter)) {
         balsa_information(LIBBALSA_INFORMATION_ERROR,
@@ -1712,32 +1793,30 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
 
     /* Retrieve the selected item in the op codes menu */
 
-    menu = gtk_menu_get_active(GTK_MENU
-                               (gtk_option_menu_get_menu
-                                (GTK_OPTION_MENU
-                                 (fe_op_codes_option_menu))));
-
     /* Set the op-codes associated with the selected item */
-
-    fil->conditions_op =
-        GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu),"value"));
+    condition_op =
+        (FilterOpType) fe_combo_box_get_value(fe_op_codes_option_menu) ==
+        FILTER_OP_OR ? CONDITION_OR : CONDITION_AND;
 
     /* Retrieve all conditions for that filter */
 
     FILTER_SETFLAG(fil,FILTER_VALID);
 
-    /* Here I set back FILTER_COMPILED, that way, modified filters with no Regex condition
-     * won't have to recalculate regex (that they don't have actually :)
-     * but modified filters with regex condition will have their compiled flag unset by filter_append_condition,
-     * So that's OK
+    /* Here I set back FILTER_COMPILED, that way, modified filters
+     * with no Regex condition won't have to recalculate regex (that
+     * they don't have actually :) but modified filters with regex
+     * condition will have their compiled flag unset by
+     * filter_append_condition, So that's OK
      */
     FILTER_SETFLAG(fil,FILTER_COMPILED);
-
+    gtk_tree_model_get_iter_first(cond_model, &cond_iter);
     do {
         LibBalsaCondition *cond;
 
         gtk_tree_model_get(cond_model, &cond_iter, 1, &cond, -1);
-        libbalsa_filter_prepend_condition(fil, libbalsa_condition_clone(cond));
+        libbalsa_filter_prepend_condition(fil,
+                                          libbalsa_condition_clone(cond),
+                                          condition_op);
     } while (gtk_tree_model_iter_next(cond_model, &cond_iter));
 
     if (filter_errno!=FILTER_NOERR) {
@@ -1814,6 +1893,31 @@ fe_revert_pressed(GtkWidget * widget, gpointer data)
  * Callback function handling the selection of a row in the filter list
  * so that we can refresh the notebook page
  */
+static void
+fill_condition_list(GtkTreeModel *model, LibBalsaCondition *condition,
+		    ConditionMatchType type)
+{
+    GtkTreeIter iter;
+    if (!condition)
+	return;
+    if (condition->type == CONDITION_OR
+	|| condition->type == CONDITION_AND)  {
+	/* A nested boolean operator must be the same as the top level
+	 * operator. */
+	if (condition->type != type)
+	    /* We'll silently ignore a mismatch. */
+	    return;
+	fill_condition_list(model, condition->match.andor.left, type);
+	fill_condition_list(model, condition->match.andor.right, type);
+	return;
+    }
+
+    gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
+    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
+                       0, _(fe_search_type[condition->type-1].text),
+                       1, libbalsa_condition_clone(condition),
+                       -1);
+}
 
 void
 fe_filters_list_selection_changed(GtkTreeSelection * selection,
@@ -1822,8 +1926,7 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
     GtkTreeModel *model;
     GtkTreeIter iter;
     LibBalsaFilter* fil;
-    LibBalsaCondition* cnd;
-    GSList *list;
+    int pos;
 
     if (!gtk_tree_selection_get_selected(selection, &model, &iter)) {
         if (gtk_tree_model_iter_n_children(model, NULL) == 0)
@@ -1842,14 +1945,19 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
                        ? fil->popup_text : "");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button),
                                  fil->sound!=NULL);
+#if GTK_CHECK_VERSION(2, 6, 0)
+    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(fe_sound_entry),
+                                  fil->sound != NULL ? fil->sound : "");
+#else /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fe_sound_entry))),
                        fil->sound!=NULL ? fil->sound : "");
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     
-    gtk_option_menu_set_history(GTK_OPTION_MENU(fe_action_option_menu), 
-                                fil->action-1);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(fe_op_codes_option_menu), 
-                                fil->conditions_op-1);
-
+    gtk_combo_box_set_active(GTK_COMBO_BOX(fe_action_option_menu),
+                             fil->action - 1);
+    pos = (fil->condition
+           && fil->condition->type == CONDITION_AND) ? 1 : 0;
+    gtk_combo_box_set_active(GTK_COMBO_BOX(fe_op_codes_option_menu), pos);
     if (fil->action!=FILTER_TRASH && fil->action_string)
         balsa_mblist_mru_option_menu_set(fe_mailboxes,
                                          fil->action_string);
@@ -1862,16 +1970,9 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
 
     /* Populate the conditions list */
     filter_errno=FILTER_NOERR;
-    for (list=fil->conditions;
-         list && filter_errno==FILTER_NOERR;list=g_slist_next(list)) {
-        cnd=(LibBalsaCondition*) list->data;
-        gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter,
-                           0, _(fe_search_type[cnd->type-1].text),
-                           1, libbalsa_condition_clone(cnd),
-                           -1);
-    }
-
+    fill_condition_list(model, fil->condition,
+			fil->condition ?
+			fil->condition->type : CONDITION_OR);
     if (filter_errno!=FILTER_NOERR)
         gtk_widget_destroy(fe_window);
 
