@@ -26,6 +26,7 @@
 #include "filter-edit.h"
 #include "filter-funcs.h"
 #include "message.h"
+#include "i18n.h"
 
 #define FILTER_EDIT_ENTRY_MAX_LENGTH 256
 #define FILTER_EDIT_PADDING 6
@@ -85,23 +86,23 @@ GtkWidget * fe_condition_delete_button,* fe_condition_edit_button;
 /* ******************************** */
 
 option_list fe_search_type[] = {
-    {N_("Simple"), CONDITION_SIMPLE, NULL},
-    {N_("Regular Expression"), CONDITION_REGEX, NULL},
-    {N_("Date interval"),CONDITION_DATE,NULL},
-    {N_("Flag condition"),CONDITION_FLAG,NULL}
+    {N_("Simple"), CONDITION_STRING},
+    {N_("Regular Expression"), CONDITION_REGEX},
+    {N_("Date interval"), CONDITION_DATE},
+    {N_("Flag condition"), CONDITION_FLAG}
 };
 
 option_list fe_actions[] = {
-    {N_("Copy to folder:"), FILTER_COPY, NULL},
-    {N_("Move to folder:"), FILTER_MOVE, NULL},
-    {N_("Print on printer:"), FILTER_PRINT, NULL},
-    {N_("Run program:"), FILTER_RUN, NULL},
-    {N_("Send to Trash"), FILTER_TRASH, NULL}
+    {N_("Copy to folder:"), FILTER_COPY},
+    {N_("Move to folder:"), FILTER_MOVE},
+    {N_("Print on printer:"), FILTER_PRINT},
+    {N_("Run program:"), FILTER_RUN},
+    {N_("Send to Trash"), FILTER_TRASH}
 };
 
 option_list fe_op_codes[] = {
-    {N_("OR"), FILTER_OP_OR, NULL},
-    {N_("AND"), FILTER_OP_AND, NULL}
+    {N_("OR"), FILTER_OP_OR},
+    {N_("AND"), FILTER_OP_AND}
 };
 
 /* ******************************** */
@@ -125,36 +126,40 @@ fe_enable_right_page(gboolean enabled)
  *    GtkOptionMenu - the menu created
  */
 
+static void
+fe_combo_box_info_free(struct fe_combo_box_info * info)
+{
+    g_slist_free(info->values);
+    g_free(info);
+}
+
 GtkWidget *
 fe_build_option_menu(option_list options[], gint num, GCallback func)
 {
-    GtkWidget *option_menu;
-    GtkWidget *menu;
+    GtkWidget *combo_box;
+    struct fe_combo_box_info *info;
     int i;
 
     if (num < 1)
 	return (NULL);
 
-    menu = gtk_menu_new();
+    combo_box = gtk_combo_box_new_text();
+    info = g_new(struct fe_combo_box_info, 1);
+    info->values = NULL;
 
     for (i = 0; i < num; i++) {
-	options[i].widget =
-	    gtk_menu_item_new_with_label(_(options[i].text));
-	g_object_set_data(G_OBJECT(options[i].widget), "value",
-			  GINT_TO_POINTER(options[i].value));
-
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), options[i].widget);
-	if (func)
-	    g_signal_connect(G_OBJECT(options[i].widget), "activate",
-			     func, GINT_TO_POINTER(options[i].value));
-	gtk_widget_show(options[i].widget);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),
+                                  _(options[i].text));
+	info->values =
+	    g_slist_append(info->values, GINT_TO_POINTER(options[i].value));
     }
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), 0);
+    if (func)
+	g_signal_connect(G_OBJECT(combo_box), "changed", func, NULL);
+    g_object_set_data_full(G_OBJECT(combo_box), BALSA_FE_COMBO_BOX_INFO,
+                           info, (GDestroyNotify) fe_combo_box_info_free);
 
-    option_menu = gtk_option_menu_new();
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), 0);
-
-    return (option_menu);
+    return combo_box;
 }				/* end fe_build_option_menu */
 
 /*
@@ -213,7 +218,7 @@ build_left_side(void)
     gtk_container_add(GTK_CONTAINER(bbox), fe_new_button);
     /* delete button */
     fe_delete_button =
-        balsa_stock_button_with_label(GNOME_STOCK_TRASH, _("_Delete"));
+        balsa_stock_button_with_label(GTK_STOCK_DELETE, _("_Delete"));
     g_signal_connect(G_OBJECT(fe_delete_button), "clicked",
 		     G_CALLBACK(fe_delete_pressed), NULL);
     gtk_container_add(GTK_CONTAINER(bbox), fe_delete_button);
@@ -348,8 +353,14 @@ build_action_page(GtkWindow * window)
 		     0, 1, 0, 1,
 		     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
 
+#if GTK_CHECK_VERSION(2, 6, 0)
+    fe_sound_entry =
+        gtk_file_chooser_button_new(_("Use Sound..."),
+                                    GTK_FILE_CHOOSER_ACTION_OPEN);
+#else /* GTK_CHECK_VERSION(2, 6, 0) */
     fe_sound_entry =
 	gnome_file_entry_new("filter_sounds", _("Use Sound..."));
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_table_attach(GTK_TABLE(table), fe_sound_entry, 1, 2, 0, 1,
 		     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
     /* fe_sound_entry is initially sensitive, so to be consistent 
@@ -357,10 +368,15 @@ build_action_page(GtkWindow * window)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button), TRUE);
     g_signal_connect(G_OBJECT(fe_sound_button), "toggled",
                      G_CALLBACK(fe_button_toggled), fe_sound_entry);
+#if GTK_CHECK_VERSION(2, 6, 0)
+    g_signal_connect(G_OBJECT(fe_sound_entry), "selection-changed",
+                     G_CALLBACK(fe_action_changed), NULL);
+#else /* GTK_CHECK_VERSION(2, 6, 0) */
     g_signal_connect(G_OBJECT
                      (gnome_file_entry_gtk_entry
                       (GNOME_FILE_ENTRY(fe_sound_entry))), "changed",
                      G_CALLBACK(fe_action_changed), NULL);
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
 
     fe_popup_button = gtk_check_button_new_with_label(_("Popup text:"));
     gtk_table_attach(GTK_TABLE(table),
@@ -459,6 +475,26 @@ build_right_side(GtkWindow * window)
     return rightside;
 }				/* end build_right_side() */
 
+/* Helper */
+static void
+fe_collect_user_headers(LibBalsaCondition * condition)
+{
+    switch (condition->type) {
+    case CONDITION_STRING:
+        if (CONDITION_CHKMATCH(condition, CONDITION_MATCH_US_HEAD)) {
+	    gchar *user_header = condition->match.string.user_header;
+	    if (user_header && *user_header)
+		fe_add_new_user_header(user_header);
+	}
+        break;
+    case CONDITION_AND:
+    case CONDITION_OR:
+        fe_collect_user_headers(condition->match.andor.left);
+        fe_collect_user_headers(condition->match.andor.right);
+    default:
+        break;
+    }
+}
 
 /*
  * filters_edit_dialog()
@@ -472,7 +508,7 @@ filters_edit_dialog(void)
     GtkWidget *hbox;
     GtkWidget *piece;
     LibBalsaFilter * cpfil,* fil;
-    GSList * cnds,* filter_list;
+    GSList * filter_list;
     GtkTreeModel *model;
     GtkTreeIter iter;
 
@@ -538,7 +574,7 @@ filters_edit_dialog(void)
 	cpfil->flags=fil->flags;
 	if (fil->sound) cpfil->sound=g_strdup(fil->sound);
 	if (fil->popup_text) cpfil->popup_text=g_strdup(fil->popup_text);
-	cpfil->conditions_op=fil->conditions_op;
+	/* FIXME: cpfil->conditions_op=fil->conditions_op; */
 	cpfil->flags=fil->flags;
 
 	/* We have to unset the "compiled" flag, because we don't copy
@@ -552,18 +588,15 @@ filters_edit_dialog(void)
 
 	FILTER_CLRFLAG(cpfil,FILTER_COMPILED);
 	/* Copy conditions */
-      	for (cnds=fil->conditions; cnds; cnds=g_slist_next(cnds)) {
-            LibBalsaCondition *c = (LibBalsaCondition*)cnds->data;
-	    cpfil->conditions = 
-                g_slist_prepend(cpfil->conditions,libbalsa_condition_clone(c));
+        if (fil->condition)
+            /* Copy conditions */
+            cpfil->condition = libbalsa_condition_clone(fil->condition);
+        else
+            balsa_information(LIBBALSA_INFORMATION_WARNING,
+                              _("Filter \"%s\" has no condition."),
+                              fil->name);
 
-	    /* If this condition is a match on a user header,
-	       add the user header name to the combo list */
-	    if (CONDITION_CHKMATCH(c,CONDITION_MATCH_US_HEAD) &&
-		c->user_header && c->user_header[0])
-		fe_add_new_user_header(c->user_header);
-        }
-	cpfil->conditions=g_slist_reverse(cpfil->conditions);
+	fe_collect_user_headers(fil->condition);
 
 	cpfil->action=fil->action;
 	if (fil->action_string) 
@@ -574,8 +607,6 @@ filters_edit_dialog(void)
                            0, cpfil->name, 1, cpfil, -1);
     }
 
-    /* To make sure we have at least one item in the combo list */
-    fe_add_new_user_header("X-Mailer");
     if (filter_errno!=FILTER_NOERR) {
 	filter_perror(filter_strerror(filter_errno));
 	gtk_widget_destroy(GTK_WIDGET(fe_window));
