@@ -26,12 +26,15 @@
 #include "config.h"
 
 #include <string.h>
+
 #include "mailbox-filter.h"
 #include "filter-funcs.h"
 #include "filter-run.h"
 #include "balsa-app.h"
 #include "save-restore.h"
 
+#include "i18n.h"	/* Must come after balsa-app.h. */
+
 /* Global vars */
 
 extern GList * fr_dialogs_opened;
@@ -87,20 +90,54 @@ run_filters_on_mailbox(GtkTreeView * filter_list, LibBalsaMailbox * mbox)
 {
     GSList *filters = build_selected_filters_list(filter_list, TRUE);
     GSList *lst;
+    guint sent_to_trash;
 
-    if (!filters) return TRUE;
-    if (!filters_prepare_to_run(filters))
+    if (!filters)
+	return TRUE;
+    if (!filters_prepare_to_run(filters)) {
+	g_slist_free(filters);
 	return FALSE;
-    libbalsa_mailbox_match(mbox, filters);
-    for (lst=filters;lst;lst = g_slist_next(lst))
-	if (((LibBalsaFilter*) lst->data)->matching_messages)
-	    break;
-    
-    if (lst) {
-	if (libbalsa_filter_apply(filters))
-	    enable_empty_trash(TRASH_FULL);
+    }
+
+    sent_to_trash = 0;
+    for (lst = filters; lst; lst = g_slist_next(lst)) {
+	LibBalsaFilter *filter = lst->data;
+	LibBalsaMailboxSearchIter *search_iter;
+	guint msgno;
+	GArray *messages;
+
+	if (!filter->condition)
+	    continue;
+	search_iter = libbalsa_mailbox_search_iter_new(filter->condition);
+
+	/* Build a list of matching messages;
+	 * - to use the existing search_iter methods, we go repeatedly
+	 *   to the mailbox;
+	 * - for the local mailboxes, that may be reasonable;
+	 * - for imap, it can surely be improved--the backend caches the
+	 * search results (in the search-iter), so we don't go
+	 * repeatedly to the server, but each test is a hash-table
+	 * lookup; the question is how to design an api that handles
+	 * cleanly both this search and mailbox searching.
+	 */
+	messages = g_array_new(FALSE, FALSE, sizeof(guint));
+	for (msgno = 1; msgno <= libbalsa_mailbox_total_messages(mbox);
+	     msgno++)
+	    if (libbalsa_mailbox_message_match(mbox, msgno, search_iter))
+		g_array_append_val(messages, msgno);
+	libbalsa_mailbox_search_iter_free(search_iter);
+
+	libbalsa_mailbox_register_msgnos(mbox, messages);
+	sent_to_trash +=
+	    libbalsa_filter_mailbox_messages(filter, mbox, messages);
+	libbalsa_mailbox_unregister_msgnos(mbox, messages);
+	g_array_free(messages, TRUE);
     }
     g_slist_free(filters);
+
+    if (sent_to_trash)
+	enable_empty_trash(balsa_app.main_window, TRASH_FULL);
+
     return TRUE;
 }
 
@@ -116,6 +153,7 @@ static
 void save_filters(BalsaFilterRunDialog * p)
 {
     if (p->filters_modified) {
+	g_slist_foreach(p->mbox->filters, (GFunc) g_free, NULL);
 	g_slist_free(p->mbox->filters);
 	p->mbox->filters=build_selected_filters_list(p->selected_filters,FALSE);
 	config_mailbox_filters_save(p->mbox);
@@ -207,7 +245,7 @@ fr_add_pressed_func(GtkTreeModel * model, GtkTreePath * path,
                             "mailbox (that causes recursion)."),
                           fil->name, p->mbox->name);
 
-    if (!libbalsa_mailbox_can_match(p->mbox, fil->conditions))
+    if (!libbalsa_mailbox_can_match(p->mbox, fil->condition))
 	balsa_information(LIBBALSA_INFORMATION_WARNING,
 			  _("The filter \"%s\" is not compatible with "
 			    "the mailbox type of \"%s\".\n"
