@@ -29,12 +29,14 @@
 #include "mailbox-node.h"
 #include "save-restore.h"
 #include "pref-manager.h"
+#include "imap-server.h"
+#include "i18n.h"
 
 typedef struct _CommonDialogData CommonDialogData;
 typedef struct _FolderDialogData FolderDialogData;
 typedef struct _SubfolderDialogData SubfolderDialogData;
 
-typedef void (*CommonDialogFunc)(CommonDialogData * cdd);
+typedef gboolean (*CommonDialogFunc)(CommonDialogData * cdd);
 
 #define FOLDER_CONF_COMMON \
     GtkDialog *dialog; \
@@ -49,11 +51,11 @@ struct _CommonDialogData {
 
 struct _FolderDialogData {
     FOLDER_CONF_COMMON;
-    GtkWidget *folder_name, *server, *port, *username, *remember,
+    BalsaServerConf bsc;
+    GtkWidget *folder_name, *port, *username, *remember,
         *password, *subscribed, *list_inbox, *prefix;
-#ifdef USE_SSL
-    GtkWidget *use_ssl;
-#endif
+    GtkWidget *use_ssl, *tls_mode;
+    GtkWidget *connection_limit, *enable_persistent, *has_bugs;
 };
 
 /* FIXME: identity_name will leak on cancelled folder edition */
@@ -61,7 +63,7 @@ struct _FolderDialogData {
 struct _SubfolderDialogData {
     FOLDER_CONF_COMMON;
     BalsaMailboxConfView *mcv;
-    GtkWidget *parent_folder, *folder_name;
+    GtkWidget *parent_folder, *folder_name, *host_label;
     gchar *old_folder, *old_parent;
     BalsaMailboxNode *parent;   /* (new) parent of the mbnode.  */
     /* Used for renaming and creation */
@@ -88,6 +90,12 @@ folder_conf_response(GtkDialog * dialog, int response,
 {
     GError *err = NULL;
 
+    /* If mbnode's parent gets rescanned, mbnode will be finalized,
+     * which triggers folder_conf_destroy_cdd, and recursively calls
+     * folder_conf_response, which results in cdd being freed before
+     * we're done with it; we ref mbnode to avoid that. */
+    if (cdd->mbnode)
+	g_object_ref(cdd->mbnode);
     switch (response) {
     case GTK_RESPONSE_HELP:
         gnome_help_display("balsa", folder_config_section, &err);
@@ -97,20 +105,26 @@ folder_conf_response(GtkDialog * dialog, int response,
                     err->message);
             g_error_free(err);
         }
+	if (cdd->mbnode)
+	    g_object_unref(cdd->mbnode);
         return;
     case GTK_RESPONSE_OK:
-        cdd->ok(cdd);
-        /* Fall over */
+        if(!cdd->ok(cdd))
+            break;
+        /* ...or fall over */
     default:
         gtk_widget_destroy(GTK_WIDGET(cdd->dialog));
         cdd->dialog = NULL;
-        if (cdd->mbnode)
+        if (cdd->mbnode) {
             /* Clearing the data signifies that the dialog has been
              * destroyed. It also triggers a call to
-             * folder_conf_destroy_cdd. */
-            g_object_set_data(G_OBJECT(cdd->mbnode),
+             * folder_conf_destroy_cdd, which will free cdd, so we cache
+             * cdd->mbnode. */
+	    BalsaMailboxNode *mbnode = cdd->mbnode;
+            g_object_set_data(G_OBJECT(mbnode),
                               BALSA_FOLDER_CONF_IMAP_KEY, NULL);
-        else
+	    g_object_unref(mbnode);
+	} else
             /* Cancelling, without creating a mailbox node. Nobody owns
              * the xDialogData, so we'll free it here. */
             g_free(cdd);
@@ -129,37 +143,12 @@ validate_folder(GtkWidget *w, FolderDialogData * fcw)
     gboolean sensitive = TRUE;
     if (!*gtk_entry_get_text(GTK_ENTRY(fcw->folder_name)))
 	sensitive = FALSE;
-    else if (!*gtk_entry_get_text(GTK_ENTRY(fcw->server)))
+    else if (!*gtk_entry_get_text(GTK_ENTRY(fcw->bsc.server)))
 	sensitive = FALSE;
 
     gtk_dialog_set_response_sensitive(fcw->dialog, GTK_RESPONSE_OK, sensitive);
 }
 
-#ifdef USE_SSL
-static void imap_use_ssl_cb(GtkToggleButton * button,
-                            FolderDialogData * fcw);
-
-/* imap_use_ssl_cb:
- * set default text in the `port' entry box according to the state of
- * the `Use SSL' checkbox
- *
- * callback on toggling fcw->use_ssl
- * */
-static void
-imap_use_ssl_cb(GtkToggleButton * button, FolderDialogData * fcw)
-{
-    char *colon, *newhost;
-    const gchar* host = gtk_entry_get_text(GTK_ENTRY(fcw->server));
-    gchar* port = gtk_toggle_button_get_active(button) ? "993" : "143";
-
-    if( (colon=strchr(host,':')) != NULL) 
-        *colon = '\0';
-    newhost = g_strconcat(host, ":", port, NULL);
-    gtk_entry_set_text(GTK_ENTRY(fcw->server), newhost);
-    g_free(newhost);
-}
-#endif
-
 static void
 remember_cb(GtkToggleButton * button, FolderDialogData * fcw)
 {
@@ -167,39 +156,50 @@ remember_cb(GtkToggleButton * button, FolderDialogData * fcw)
                              gtk_toggle_button_get_active(button));
 }
 
-static void
+static gboolean
 folder_conf_clicked_ok(FolderDialogData * fcw)
 {
     gboolean insert;
     LibBalsaServer *s;
-    GNode *gnode;
+    const gchar *username;
+    const gchar *host;
+
+    host = gtk_entry_get_text(GTK_ENTRY(fcw->bsc.server));
+    username = gtk_entry_get_text(GTK_ENTRY(fcw->username));
 
     if (fcw->mbnode) {
         insert = FALSE;
         s = fcw->mbnode->server;
     } else {
         insert = TRUE;
-        s = LIBBALSA_SERVER(libbalsa_server_new(LIBBALSA_SERVER_IMAP));
+	s = LIBBALSA_SERVER(libbalsa_imap_server_new(username, host));
+        g_signal_connect(G_OBJECT(s), "get-password",
+                         G_CALLBACK(ask_password), NULL);
     }
 
-    libbalsa_server_set_host(s, gtk_entry_get_text(GTK_ENTRY(fcw->server))
-#ifdef USE_SSL
-                             ,
-                             gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
-                                                          (fcw->use_ssl))
+    s->tls_mode = balsa_server_conf_get_tls_mode(&fcw->bsc);
+    libbalsa_imap_server_set_max_connections
+        (LIBBALSA_IMAP_SERVER(s),
+         gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON
+                                          (fcw->connection_limit)));
+#if !defined(ENABLE_TOUCH_UI)
+    libbalsa_imap_server_enable_persistent_cache
+        (LIBBALSA_IMAP_SERVER(s),
+         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->enable_persistent)));
 #endif
-        );
-    libbalsa_server_set_username(s,
-                                 gtk_entry_get_text(GTK_ENTRY
-                                                    (fcw->username)));
+    libbalsa_imap_server_set_bug
+        (LIBBALSA_IMAP_SERVER(s), ISBUG_FETCH,
+         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->has_bugs)));
+    libbalsa_server_set_username(s, username);
     s->remember_passwd =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->remember));
     libbalsa_server_set_password(s,
                                  gtk_entry_get_text(GTK_ENTRY
                                                     (fcw->password)));
-
     if (!fcw->mbnode) {
         fcw->mbnode = balsa_mailbox_node_new_imap_folder(s, NULL);
+	/* mbnode will be unrefed in folder_conf_response. */
+	g_object_ref(fcw->mbnode);
         /* The mailbox node takes over ownership of the
          * FolderDialogData. */
         g_object_set_data_full(G_OBJECT(fcw->mbnode),
@@ -217,22 +217,29 @@ folder_conf_clicked_ok(FolderDialogData * fcw)
     fcw->mbnode->list_inbox =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->list_inbox));
 
+    /* Set host after all other server changes, as it triggers
+     * save-to-config for any folder or mailbox using this server. */
+    libbalsa_server_set_host(s, host, 
+                             balsa_server_conf_get_use_ssl(&fcw->bsc));
+
     if (insert) {
-        balsa_mailbox_nodes_lock(TRUE);
-        g_node_append(balsa_app.mailbox_nodes,
-                      gnode = g_node_new(fcw->mbnode));
-        balsa_mailbox_node_append_subtree(fcw->mbnode, gnode);
-        balsa_mailbox_nodes_unlock(TRUE);
-        balsa_mblist_repopulate(balsa_app.mblist_tree_store);
+	balsa_mblist_mailbox_node_append(NULL, fcw->mbnode);
+        balsa_mailbox_node_append_subtree(fcw->mbnode);
         config_folder_add(fcw->mbnode, NULL);
+	g_signal_connect_swapped(s, "set-host",
+		                 G_CALLBACK(config_folder_update),
+				 fcw->mbnode);
         update_mail_servers();
     } else {
         balsa_mailbox_node_rescan(fcw->mbnode);
-        config_folder_update(fcw->mbnode);
+	/* Calling libbalsa_server_set_host already triggered an update,
+	 * so we do not need to here:
+	 * config_folder_update(fcw->mbnode); */
+	balsa_mblist_mailbox_node_redraw(fcw->mbnode);
     }
+    return TRUE;
 }
 
-
 /* folder_conf_imap_node:
    show the IMAP Folder configuration dialog for given mailbox node.
    If mn is NULL, setup it with default values for folder creation.
@@ -240,7 +247,7 @@ folder_conf_clicked_ok(FolderDialogData * fcw)
 void
 folder_conf_imap_node(BalsaMailboxNode *mn)
 {
-    GtkWidget *frame, *table, *label;
+    GtkWidget *notebook, *table, *label, *advanced;
     FolderDialogData *fcw;
     static FolderDialogData *fcw_new;
     LibBalsaServer *s;
@@ -256,7 +263,6 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     }
 
     s = mn ? mn->server : NULL;
-    default_server = libbalsa_guess_imap_server();
 
     fcw = g_new(FolderDialogData, 1);
     fcw->ok = (CommonDialogFunc) folder_conf_clicked_ok;
@@ -266,27 +272,56 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
                    (_("Remote IMAP folder"),
                     GTK_WINDOW(balsa_app.main_window),
                     GTK_DIALOG_DESTROY_WITH_PARENT,
-                    mn ? _("_Update") : _("_Create"), GTK_RESPONSE_OK,
+                    mn ? _("_Update") : _("C_reate"), GTK_RESPONSE_OK,
                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                     GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL));
+    g_object_add_weak_pointer(G_OBJECT(fcw->dialog),
+                              (gpointer) &fcw->dialog);
     gtk_window_set_wmclass(GTK_WINDOW(fcw->dialog), 
 			   "folder_config_dialog", "Balsa");
-    if (mn)
+    if (mn) {
         g_object_set_data_full(G_OBJECT(mn),
                                BALSA_FOLDER_CONF_IMAP_KEY, fcw, 
                                (GDestroyNotify) folder_conf_destroy_cdd);
-    else {
+    } else {
         fcw_new = fcw;
         g_object_add_weak_pointer(G_OBJECT(fcw->dialog),
                                   (gpointer) &fcw_new);
     }
 
-    frame = gtk_frame_new(_("Remote IMAP folder set"));
+    notebook = gtk_notebook_new();
     gtk_box_pack_start(GTK_BOX(fcw->dialog->vbox),
-                       frame, TRUE, TRUE, 0);
+                       notebook, TRUE, TRUE, 0);
     table = gtk_table_new(9, 2, FALSE);
-    gtk_container_add(GTK_CONTAINER(frame), table);
- 
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table,
+                             gtk_label_new_with_mnemonic(_("_Basic")));
+    advanced = balsa_server_conf_get_advanced_widget(&fcw->bsc, s, 3);
+    /* Limit number of connections */
+    fcw->connection_limit = 
+        balsa_server_conf_add_spinner
+        (&fcw->bsc, _("_Max number of connections"), 1, 40, 1,
+         s 
+         ? libbalsa_imap_server_get_max_connections(LIBBALSA_IMAP_SERVER(s))
+         : 20);
+#if !defined(ENABLE_TOUCH_UI)
+    fcw->enable_persistent = 
+        balsa_server_conf_add_checkbox(&fcw->bsc,
+                                       _("Enable _persistent cache"));
+    if(!s || 
+       libbalsa_imap_server_has_persistent_cache(LIBBALSA_IMAP_SERVER(s)))
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fcw->enable_persistent),
+                                     TRUE);
+#endif
+    fcw->has_bugs = 
+        balsa_server_conf_add_checkbox(&fcw->bsc,
+                                       _("Enable _bug workarounds"));
+    if(s &&
+       libbalsa_imap_server_has_bug(LIBBALSA_IMAP_SERVER(s), ISBUG_FETCH))
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fcw->has_bugs),
+                                     TRUE);
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), advanced,
+                             gtk_label_new_with_mnemonic(_("_Advanced")));
+
     /* INPUT FIELD CREATION */
     label = create_label(_("Descriptive _Name:"), table, 0);
     fcw->folder_name = create_entry(fcw->dialog, table,
@@ -294,11 +329,14 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
                                    fcw, 0, mn ? mn->name : NULL, 
 				   label);
 
+    default_server = libbalsa_guess_imap_server();
     label = create_label(_("_Server:"), table, 1);
-    fcw->server = create_entry(fcw->dialog, table,
+    fcw->bsc.server = create_entry(fcw->dialog, table,
                               GTK_SIGNAL_FUNC(validate_folder),
                               fcw, 1, s ? s->host : default_server,
 			      label);
+    fcw->bsc.default_ports = IMAP_DEFAULT_PORTS;
+    g_free(default_server);
 
     label= create_label(_("Use_r name:"), table, 3);
     fcw->username = create_entry(fcw->dialog, table,
@@ -318,21 +356,13 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
 
     fcw->subscribed = create_check(fcw->dialog, _("Subscribed _folders only"), 
                                   table, 6, mn ? mn->subscribed : FALSE);
-    fcw->list_inbox = create_check(fcw->dialog, _("_Always show INBOX"), 
+    fcw->list_inbox = create_check(fcw->dialog, _("Always show _INBOX"), 
                                   table, 7, mn ? mn->list_inbox : TRUE); 
 
     label = create_label(_("Pr_efix"), table, 8);
     fcw->prefix = create_entry(fcw->dialog, table, NULL, NULL, 8,
 			      mn ? mn->dir : NULL, label);
     
-#ifdef USE_SSL
-    fcw->use_ssl = create_check(fcw->dialog,
-			       _("Use SS_L (IMAPS)"),
-			       table, 9, s ? s->use_ssl : FALSE);
-    g_signal_connect(G_OBJECT(fcw->use_ssl), "toggled",
-                     G_CALLBACK(imap_use_ssl_cb), fcw);
-#endif
-
     gtk_widget_show_all(GTK_WIDGET(fcw->dialog));
 
     validate_folder(NULL, fcw);
@@ -341,6 +371,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     gtk_dialog_set_default_response(fcw->dialog, 
                                     mn ? GTK_RESPONSE_OK 
                                     : GTK_RESPONSE_CANCEL);
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
 
     g_signal_connect(G_OBJECT(fcw->dialog), "response",
                      G_CALLBACK(folder_conf_response), fcw);
@@ -397,6 +428,7 @@ browse_button_select_row_cb(GtkTreeSelection * selection,
                                       GTK_RESPONSE_OK, selected);
     if (selected)
         gtk_tree_model_get(model, &iter, 0, &bbd->mbnode, -1);
+    /* bbd->mbnode is unreffed when bbd is freed. */
 }
 
 static void
@@ -420,25 +452,41 @@ browse_button_response(GtkDialog * dialog, gint response,
         if (mbnode->dir)
             gtk_entry_set_text(GTK_ENTRY(bbd->sdd->parent_folder),
                                mbnode->dir);
+        if(mbnode->server)
+            gtk_label_set_label(GTK_LABEL(bbd->sdd->host_label),
+                                mbnode->server->host);
     }
-
+    validate_sub_folder(NULL, bbd->sdd);
     gtk_widget_set_sensitive(bbd->button, TRUE);
     gtk_widget_destroy(GTK_WIDGET(dialog));
 }
 
 static gboolean
 folder_selection_func(GtkTreeSelection * selection, GtkTreeModel * model,
-                      GtkTreePath * path, gboolean path_currently_selected,
-                      SubfolderDialogData * sdd)
+		      GtkTreePath * path, gboolean path_currently_selected,
+		      SubfolderDialogData * sdd)
 {
     GtkTreeIter iter;
     BalsaMailboxNode *mbnode;
+    gboolean retval;
 
     gtk_tree_model_get_iter(model, &iter, path);
     gtk_tree_model_get(model, &iter, 0, &mbnode, -1);
+    retval = (mbnode->server
+	      && mbnode->server->type == LIBBALSA_SERVER_IMAP
+	      && (sdd->mbnode == NULL
+		  || sdd->mbnode->server == mbnode->server));
+    g_object_unref(mbnode);
 
-    return (mbnode->server && mbnode->server->type == LIBBALSA_SERVER_IMAP
-            && (sdd->mbnode == NULL || sdd->mbnode->server == mbnode->server));
+    return retval;
+}
+
+static void
+browse_button_data_free(BrowseButtonData *bbd)
+{
+    if (bbd->mbnode)
+	g_object_unref(bbd->mbnode);
+    g_free(bbd);
 }
 
 static void
@@ -480,7 +528,8 @@ browse_button_cb(GtkWidget * widget, SubfolderDialogData * sdd)
     bbd->dialog = GTK_DIALOG(dialog);
     bbd->button = widget;
     bbd->mbnode = NULL;
-    g_object_weak_ref(G_OBJECT(dialog), (GWeakNotify) g_free, bbd);
+    g_object_weak_ref(G_OBJECT(dialog),
+		      (GWeakNotify) browse_button_data_free, bbd);
     g_signal_connect(G_OBJECT(selection), "changed",
                      G_CALLBACK(browse_button_select_row_cb), bbd);
     g_signal_connect(G_OBJECT(tree_view), "row-activated",
@@ -500,15 +549,17 @@ browse_button_cb(GtkWidget * widget, SubfolderDialogData * sdd)
     /* OK button is insensitive until some row is selected. */
     gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
                                       GTK_RESPONSE_OK, FALSE);
+
     g_signal_connect(G_OBJECT(dialog), "response",
                      G_CALLBACK(browse_button_response), bbd);
     gtk_widget_show_all(GTK_WIDGET(dialog));
 }
 
-static void
+static gboolean
 subfolder_conf_clicked_ok(SubfolderDialogData * sdd)
 {
     gchar *parent, *folder;
+    gboolean ret = TRUE;
 
     parent =
         gtk_editable_get_chars(GTK_EDITABLE(sdd->parent_folder), 0, -1);
@@ -553,15 +604,23 @@ folder, parent);
                 gtk_widget_destroy(ask);
             }
             if (button == GTK_RESPONSE_OK) {
+                GError* err = NULL;
                 /* Close the mailbox before renaming,
                  * otherwise the rescan will try to close it
                  * under its old name.
                  */
                 balsa_window_close_mbnode(balsa_app.main_window,
                                           sdd->mbnode);
-                libbalsa_imap_rename_subfolder
-                    (LIBBALSA_MAILBOX_IMAP(sdd->mbnode->mailbox),
-                     parent, folder, sdd->mbnode->subscribed);
+                if(!libbalsa_imap_rename_subfolder
+                   (LIBBALSA_MAILBOX_IMAP(sdd->mbnode->mailbox),
+                    parent, folder, sdd->mbnode->subscribed, &err)) {
+                    balsa_information(LIBBALSA_INFORMATION_ERROR,
+                                      _("Folder rename failed. Reason: %s"),
+                                      err ? err->message : "unknown");
+                    g_clear_error(&err);
+                    ret = FALSE;
+                    goto error;
+                }
                 g_free(sdd->mbnode->dir);
                 sdd->mbnode->dir = g_strdup(parent);
 
@@ -569,22 +628,23 @@ folder, parent);
                 if (sdd->old_parent
                     && !strncmp(parent, sdd->old_parent, strlen(parent))) {
                     /* moved it up the tree */
-                    GNode *n = balsa_find_dir(balsa_app.mailbox_nodes,
-                                              parent);
-                    if (n)
-                        balsa_mailbox_node_rescan
-                            (BALSA_MAILBOX_NODE(n->data));
-                    else
+		    BalsaMailboxNode *mbnode =
+                        balsa_find_dir(sdd->parent->server, parent);
+                    if (mbnode) {
+                        balsa_mailbox_node_rescan(mbnode);
+			g_object_unref(mbnode);
+		    } else
                         printf("Parent not found!?\n");
                 } else if (sdd->old_parent
                            && !strncmp(parent, sdd->old_parent,
                                        strlen(sdd->old_parent))) {
                     /* moved it down the tree */
-                    GNode *n = balsa_find_dir(balsa_app.mailbox_nodes,
-                                              sdd->old_parent);
-                    if (n)
-                        balsa_mailbox_node_rescan
-                            (BALSA_MAILBOX_NODE(n->data));
+		    BalsaMailboxNode *mbnode =
+			balsa_find_dir(sdd->parent->server, sdd->old_parent);
+                    if (mbnode) {
+                        balsa_mailbox_node_rescan(mbnode);
+			g_object_unref(mbnode);
+		    }
                 } else {
                     /* moved it sideways: a chain of folders might
                      * go away, so we'd better rescan from higher up
@@ -598,16 +658,25 @@ folder, parent);
             }
         }
     } else {
+        GError *err = NULL;
         /* create and subscribe, if parent was. */
-        libbalsa_imap_new_subfolder(parent, folder,
-                                    sdd->parent->subscribed,
-                                    sdd->parent->server);
-
-        /* see it as server sees it: */
-        balsa_mailbox_node_rescan(sdd->parent);
+        if(libbalsa_imap_new_subfolder(parent, folder,
+                                       sdd->parent->subscribed,
+                                       sdd->parent->server, &err)) {
+            /* see it as server sees it: */
+            balsa_mailbox_node_rescan(sdd->parent);
+        } else {
+            balsa_information(LIBBALSA_INFORMATION_ERROR,
+                              _("Folder creation failed. Reason: %s"),
+                              err ? err->message : "unknown");
+            g_clear_error(&err);
+            ret = FALSE;
+        }
     }
+ error:
     g_free(parent);
     g_free(folder);
+    return ret;
 }
 
 /* folder_conf_imap_sub_node:
@@ -655,21 +724,23 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
         GTK_DIALOG(gtk_dialog_new_with_buttons
                    (_("Remote IMAP subfolder"), 
                     GTK_WINDOW(balsa_app.main_window),
-                    GTK_DIALOG_DESTROY_WITH_PARENT,
+                    GTK_DIALOG_DESTROY_WITH_PARENT, /* must NOT be modal */
                     mn ? _("_Update") : _("_Create"), GTK_RESPONSE_OK,
                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
                     GTK_STOCK_HELP,   GTK_RESPONSE_HELP,
                     NULL));
+    g_object_add_weak_pointer(G_OBJECT(sdd->dialog),
+                              (gpointer) &sdd->dialog);
     /* `Enter' key => Create: */
     gtk_dialog_set_default_response(GTK_DIALOG(sdd->dialog), GTK_RESPONSE_OK);
     gtk_window_set_wmclass(GTK_WINDOW(sdd->dialog), 
 			   "subfolder_config_dialog", "Balsa");
 
-    if (sdd->mbnode)
+    if (sdd->mbnode) {
         g_object_set_data_full(G_OBJECT(sdd->mbnode),
                                BALSA_FOLDER_CONF_IMAP_KEY, sdd, 
                                (GDestroyNotify) folder_conf_destroy_cdd);
-    else {
+        } else {
         sdd_new = sdd;
         g_object_add_weak_pointer(G_OBJECT(sdd->dialog),
                                   (gpointer) &sdd_new);
@@ -689,23 +760,30 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
 				   sdd, 0, sdd->old_folder, label);
 
     subtable = gtk_table_new(1, 3, FALSE);
-    label = create_label(_("_Subfolder of:"), table, 1);
-    sdd->parent_folder = create_entry(sdd->dialog, subtable,
-                                     GTK_SIGNAL_FUNC(validate_sub_folder),
-				     sdd, 0, sdd->old_parent, label);
-
+    label = create_label(_("Host:"), table, 1);
+    sdd->host_label = 
+        gtk_label_new(sdd->mbnode && sdd->mbnode->server
+                      ? sdd->mbnode->server->host : "");
+    gtk_table_attach(GTK_TABLE(subtable), sdd->host_label, 1, 2, 0, 1,
+		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 5);
     button = gtk_button_new_with_mnemonic(_("_Browse..."));
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(browse_button_cb), (gpointer) sdd);
     gtk_table_attach(GTK_TABLE(subtable), button, 2, 3, 0, 1,
-	GTK_EXPAND | GTK_FILL, GTK_FILL, 5, 5);
+	GTK_FILL, GTK_FILL, 5, 5);
     gtk_table_attach(GTK_TABLE(table), subtable, 1, 2, 1, 2,
 	GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 5);
 
+    label = create_label(_("_Subfolder of:"), table, 2);
+    sdd->parent_folder = create_entry(sdd->dialog, table,
+                                     GTK_SIGNAL_FUNC(validate_sub_folder),
+				     sdd, 2, sdd->old_parent, label);
+
+
     if (mn)
         sdd->mcv = mailbox_conf_view_new(mn->mailbox,
                                          GTK_WINDOW(sdd->dialog),
-                                         table, 2);
+                                         table, 3);
 
     gtk_widget_show_all(GTK_WIDGET(sdd->dialog));
 
@@ -721,7 +799,6 @@ void
 folder_conf_delete(BalsaMailboxNode* mbnode)
 {
     GtkWidget* ask;
-    GNode* gnode;
     gint response;
 
     if(!mbnode->config_prefix) {
@@ -751,21 +828,8 @@ folder_conf_delete(BalsaMailboxNode* mbnode)
     config_folder_delete(mbnode);
 
     /* Remove the node from balsa's mailbox list */
-    balsa_mailbox_nodes_lock(FALSE);
-    gnode = balsa_find_mbnode(balsa_app.mailbox_nodes, mbnode);
-    balsa_mailbox_nodes_unlock(FALSE);
-    if(gnode) {
-        balsa_mailbox_nodes_lock(TRUE);
-	balsa_remove_children_mailbox_nodes(gnode);
-        balsa_mailbox_nodes_unlock(TRUE);
-        balsa_mblist_remove_mailbox_node(balsa_app.mblist_tree_store,
-                                         mbnode);
-	g_node_unlink(gnode);
-	g_node_destroy(gnode);
-	g_object_unref(G_OBJECT(mbnode));
-	update_mail_servers();
-    } else g_warning("folder node %s (%p) not found in hierarchy.\n",
-		     mbnode->name, mbnode);
+    balsa_mblist_mailbox_node_remove(mbnode);
+    update_mail_servers();
 }
 
 void
