@@ -44,4 +44,32 @@ BalsaMailboxConfView *mailbox_conf_view_new(LibBalsaMailbox * mailbox,
                                             GtkWidget * table, gint row);
 void mailbox_conf_view_check(BalsaMailboxConfView * mcc,
                              LibBalsaMailbox * mailbox);
+
+
+typedef struct {
+    GtkWidget *use_ssl;
+    GtkWidget *tls_mode;
+    GtkTable  *table;      /* internal */
+    GtkWidget *tls_option; /* internal */
+    GtkWidget *server;     /* internal */
+    const gchar *default_ports;
+    unsigned   used_rows;  /* internal */
+} BalsaServerConf;
+#define IMAP_DEFAULT_PORTS "143 993 imap imaps"
+#define POP3_DEFAULT_PORTS "110 995 pop3 pop3s"
+
+GtkWidget* balsa_server_conf_get_advanced_widget(BalsaServerConf *bsc,
+                                                 LibBalsaServer *s,
+                                                 int extra_rows);
+GtkWidget*      balsa_server_conf_add_checkbox(BalsaServerConf *bsc,
+                                               const char *label);
+GtkWidget*      balsa_server_conf_add_spinner(BalsaServerConf *bsc,
+                                              const char *label,
+                                              gint lo, gint hi, gint step,
+                                              gint initial_value);
+void            balsa_server_conf_set_values(BalsaServerConf *bsc,
+                                             LibBalsaServer *server);
+gboolean        balsa_server_conf_get_use_ssl(BalsaServerConf *bsc);
+LibBalsaTlsMode balsa_server_conf_get_tls_mode(BalsaServerConf *bsc);
+
 #endif				/* __MAILBOX_CONF_H__ */
