@@ -44,11 +44,13 @@ struct _BalsaWindow {
 
     GtkWidget *progress_bar;
     GtkWidget *mblist;
+    GtkWidget *sos_entry;       /* SenderOrSubject filter entry */
     GtkWidget *notebook;
     GtkWidget *preview;		/* message is child */
     GtkWidget *hpaned;
     GtkWidget *vpaned;
     GtkWidget *current_index;
+    GtkWidget *filter_label;
     LibBalsaMessage *current_message;
 };
 
@@ -57,6 +59,7 @@ struct _BalsaWindowClass {
 
     void (*open_mbnode)  (BalsaWindow * window, BalsaMailboxNode * mbnode);
     void (*close_mbnode) (BalsaWindow * window, BalsaMailboxNode * mbnode);
+    void (*identities_changed) (BalsaWindow * window);
 };
 
 /*
@@ -72,19 +75,28 @@ typedef enum {
 GtkType balsa_window_get_type(void);
 GtkWidget *balsa_window_new(void);
 GtkWidget *balsa_window_find_current_index(BalsaWindow * window);
+void balsa_window_enable_mailbox_menus(BalsaWindow * window,
+				       BalsaIndex * index);
+LibBalsaCondition* balsa_window_get_view_filter(BalsaWindow *window);
 void balsa_window_refresh(BalsaWindow * window);
 void balsa_window_open_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
 void balsa_window_close_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
-void enable_empty_trash(TrashState status);
-void balsa_window_enable_continue(void);
+void balsa_identities_changed(BalsaWindow *bw);
+
+void balsa_window_set_filter_label(BalsaWindow * window, gboolean to_field);
+void balsa_window_update_tab(BalsaMailboxNode * mbnode);
+void enable_empty_trash(BalsaWindow * window, TrashState status);
+void balsa_window_enable_continue(BalsaWindow * window);
 void balsa_change_window_layout(BalsaWindow *window);
 gboolean mail_progress_notify_cb(void);
 gboolean send_progress_notify_cb(void);
 void check_new_messages_cb(GtkWidget *, gpointer data);
 void check_new_messages_real(GtkWidget *, gpointer data, int type);
-void empty_trash(void);
-void update_view_menu(void);
+void check_new_messages_count(LibBalsaMailbox * mailbox, gboolean notify);
+void empty_trash(BalsaWindow * window);
+void update_view_menu(BalsaWindow * window);
 BalsaToolbarModel *balsa_window_get_toolbar_model(void);
+void balsa_window_select_all(GtkWindow * window);
 
 /* functions to manipulate the progress bars of the window */
 void balsa_window_increase_activity(BalsaWindow* window);
