@@ -34,12 +34,12 @@
 #include "mailbox-node.h"
 
 #include "libbalsa.h"
+#include "i18n.h"
 
 typedef struct _MessageWindow MessageWindow;
 
 /* callbacks */
 static void destroy_message_window(GtkWidget * widget, MessageWindow * mw);
-static void mw_message_weak_notify(MessageWindow * mw, GObject * obj);
 static void close_message_window(GtkWidget * widget, gpointer data);
 
 static void replyto_message_cb(GtkWidget * widget, gpointer data);
@@ -62,9 +62,13 @@ static void wrap_message_cb(GtkWidget * widget, gpointer data);
 static void size_alloc_cb(GtkWidget * window, GtkAllocation * alloc);
 static void mw_set_buttons_sensitive(MessageWindow * mw);
 
+static void mw_set_selected(MessageWindow * mw, void (*select_func)(BalsaIndex *));
+
 static void copy_cb(GtkWidget * widget, MessageWindow * mw);
 static void select_all_cb(GtkWidget * widget, gpointer);
 
+static void next_message_cb(GtkWidget * widget, gpointer data);
+static void previous_message_cb(GtkWidget * widget, gpointer data);
 static void next_unread_cb(GtkWidget * widget, gpointer);
 static void next_flagged_cb(GtkWidget * widget, gpointer);
 static void print_cb(GtkWidget * widget, gpointer);
@@ -91,6 +95,7 @@ static GnomeUIInfo shown_hdrs_menu[] = {
 static GnomeUIInfo file_menu[] = {
     GNOMEUIINFO_MENU_PRINT_ITEM(print_cb, NULL),
     GNOMEUIINFO_SEPARATOR,
+#define MENU_FILE_CLOSE_POS 2
     GNOMEUIINFO_MENU_CLOSE_ITEM(close_message_window, NULL),
     GNOMEUIINFO_END
 };
@@ -142,57 +147,88 @@ static GnomeUIInfo message_menu[] = {
     {
      GNOME_APP_UI_ITEM, N_("_Reply..."), N_("Reply to this message"),
      replyto_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_REPLY, 'R', 0, NULL},
+     BALSA_PIXMAP_REPLY, 'R', 0, NULL},
     /* A */
     {
      GNOME_APP_UI_ITEM, N_("Reply to _All..."),
      N_("Reply to all recipients of this message"),
      replytoall_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_REPLY_ALL, 'A', 0, NULL},
+     BALSA_PIXMAP_REPLY_ALL, 'A', 0, NULL},
     /* G */
     {
      GNOME_APP_UI_ITEM, N_("Reply to _Group..."),
      N_("Reply to mailing list"),
      replytogroup_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_REPLY_GROUP, 'G', 0, NULL},
+     BALSA_PIXMAP_REPLY_GROUP, 'G', 0, NULL},
     /* F */
     {
      GNOME_APP_UI_ITEM, N_("_Forward attached..."),
      N_("Forward this message as attachment"),
      forward_message_attached_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_FORWARD, 'F', 0, NULL},
+     BALSA_PIXMAP_FORWARD, 'F', 0, NULL},
     {
      GNOME_APP_UI_ITEM, N_("Forward inline..."), 
      N_("Forward this message inline"),
      forward_message_inline_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_FORWARD, 'F', GDK_CONTROL_MASK, NULL},
+     BALSA_PIXMAP_FORWARD, 'F', GDK_CONTROL_MASK, NULL},
     GNOMEUIINFO_SEPARATOR,
+#define MENU_MESSAGE_NEXT_PART_POS 6
     {
      GNOME_APP_UI_ITEM, N_("Next Part"), N_("Next part in message"),
      next_part_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_NEXT, '.', GDK_CONTROL_MASK, NULL},
+     BALSA_PIXMAP_NEXT_PART, '.', GDK_CONTROL_MASK, NULL},
+#define MENU_MESSAGE_PREVIOUS_PART_POS 7
     {
      GNOME_APP_UI_ITEM, N_("Previous Part"),
      N_("Previous part in message"),
      previous_part_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_PREVIOUS, ',', GDK_CONTROL_MASK, NULL},
+     BALSA_PIXMAP_PREVIOUS_PART, ',', GDK_CONTROL_MASK, NULL},
     {
      GNOME_APP_UI_ITEM, N_("Save Current Part..."),
      N_("Save current part in message"),
      save_current_part_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_SAVE, 's', GDK_CONTROL_MASK, NULL},
+     GTK_STOCK_SAVE, 's', GDK_CONTROL_MASK, NULL},
     {
      GNOME_APP_UI_ITEM, N_("_View Source..."),
      N_("View source form of the message"),
      view_msg_source_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_SAVE, 'v', GDK_CONTROL_MASK, NULL},
+     BALSA_PIXMAP_BOOK_OPEN, 'v', GDK_CONTROL_MASK, NULL},
      GNOMEUIINFO_SEPARATOR,
-#define MENU_MESSAGE_NEXT_UNREAD_POS 11
+    /* N */
+#define MENU_MESSAGE_NEXT_POS 11
+    {
+     GNOME_APP_UI_ITEM, N_("_Next Message"), N_("Next message"),
+     next_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
+     BALSA_PIXMAP_NEXT, 'N', 0, NULL},
+    /* P */
+#define MENU_MESSAGE_PREVIOUS_POS 12
+    {
+     GNOME_APP_UI_ITEM, N_("_Previous Message"),
+     N_("Previous Message"),
+     previous_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
+     BALSA_PIXMAP_PREVIOUS, 'P', 0, NULL},
+    /* Ctrl + N */
+#define MENU_MESSAGE_NEXT_UNREAD_POS 13
     {
      GNOME_APP_UI_ITEM, N_("Next Unread Message"),
      N_("Next Unread Message"), next_unread_cb, NULL, NULL,
-     GNOME_APP_PIXMAP_STOCK, BALSA_PIXMAP_MENU_NEXT_UNREAD, 'N',
+     GNOME_APP_PIXMAP_STOCK, BALSA_PIXMAP_NEXT_UNREAD, 'N',
      GDK_CONTROL_MASK, NULL},
+    /* Ctrl + Alt + F */
+#define MENU_MESSAGE_NEXT_FLAGGED_POS 14
+    {
+     GNOME_APP_UI_ITEM, N_("Next Flagged Message"),
+     N_("Next Flagged Message"), next_flagged_cb, NULL, NULL,
+     GNOME_APP_PIXMAP_STOCK, BALSA_PIXMAP_NEXT_FLAGGED, 'F',
+     GDK_MOD1_MASK|GDK_CONTROL_MASK, NULL},
+     GNOMEUIINFO_SEPARATOR,
+     /* D */
+#define MENU_MESSAGE_TRASH_POS 16
+    {
+     GNOME_APP_UI_ITEM, N_("_Move to Trash"),
+     N_("Move the message to Trash mailbox"),
+     trash_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
+     GTK_STOCK_DELETE, 'D', 0, NULL},
     GNOMEUIINFO_END
 };
 
@@ -222,7 +258,13 @@ struct _MessageWindow {
     guint idle_handler_id;
 
     /* Pointers to our copies of widgets. */
+    GtkWidget *next;
+    GtkWidget *previous;
     GtkWidget *next_unread;
+    GtkWidget *next_flagged;
+    GtkWidget *next_part;
+    GtkWidget *previous_part;
+    GtkWidget *trash;
 #ifdef HAVE_GTKHTML
     GtkWidget *zoom_in;
     GtkWidget *zoom_out;
@@ -242,7 +284,8 @@ mru_menu_cb(gchar * url, gpointer data)
 static void
 message_window_move_message(MessageWindow * mw, LibBalsaMailbox * mailbox)
 {
-    GList *list;
+    GArray *messages;
+    LibBalsaMessage *original = mw->bindex->current_message;
 
     g_return_if_fail(mailbox != NULL);
 
@@ -250,11 +293,54 @@ message_window_move_message(MessageWindow * mw, LibBalsaMailbox * mailbox)
     if (mw->message->mailbox == mailbox)
 	return;
 
-    list = g_list_append(NULL, mw->message);
-    balsa_index_transfer(mw->bindex, list, mailbox, FALSE);
-    g_list_free_1(list);
-    
-    close_message_window(NULL, (gpointer) mw);
+    messages = g_array_new(FALSE, FALSE, sizeof(guint));
+    g_array_append_val(messages, mw->message->msgno);
+    balsa_index_transfer(mw->bindex, messages, mailbox, FALSE);
+    g_array_free(messages, TRUE);
+
+    if ( balsa_app.mw_action_after_move == NEXT_UNREAD )
+    {
+        /* Try selecting the next unread message.
+           If there are no more, close the window anyways. */
+        mw_set_selected( mw, balsa_index_select_next_unread );
+
+        if ( mw->bindex->current_message == original )
+            close_message_window(NULL, (gpointer) mw);
+    }
+    else if ( balsa_app.mw_action_after_move == NEXT )
+    {
+        mw_set_selected( mw, balsa_index_select_next );
+
+        if ( mw->bindex->current_message == original )
+            close_message_window(NULL, (gpointer) mw);
+    }
+    else
+    {
+        close_message_window(NULL, (gpointer) mw);
+    }
+}
+
+static void
+mw_set_part_buttons_sensitive(MessageWindow * mw)
+{
+    BalsaMessage *msg = BALSA_MESSAGE(mw->bmessage);
+    GtkWidget *toolbar;
+    gboolean enable;
+
+    if (!msg || !msg->treeview)
+	return;
+
+    toolbar = balsa_toolbar_get_from_gnome_app(GNOME_APP(mw->window));
+
+    enable = balsa_message_has_next_part(msg);
+    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT_PART,
+                                       enable);
+    gtk_widget_set_sensitive(mw->next_part, enable);
+
+    enable = balsa_message_has_previous_part(msg);
+    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_PREVIOUS_PART,
+                                       enable);
+    gtk_widget_set_sensitive(mw->previous_part, enable);
 }
 
 static gboolean
@@ -262,25 +348,36 @@ message_window_idle_handler(MessageWindow* mw)
 {
     BalsaMessage *msg = BALSA_MESSAGE(mw->bmessage);
     LibBalsaMessage *message = mw->message;
+    gchar *title;
+    gchar *from;
 
     gdk_threads_enter();
 
     mw->idle_handler_id = 0;
 
-    balsa_message_set(msg, message);
-
-    if(msg && msg->treeview) {
-	if (msg->info_count > 1) {
-            GtkWidget *toolbar =
-                balsa_toolbar_get_from_gnome_app(GNOME_APP(mw->window));
-	    balsa_toolbar_set_button_sensitive(toolbar,
-                                               BALSA_PIXMAP_PREVIOUS, TRUE);
-	    balsa_toolbar_set_button_sensitive(toolbar,
-                                               BALSA_PIXMAP_NEXT, TRUE);
-	}
+    if (!message->mailbox) {
+	gtk_widget_destroy(mw->window);
+	gdk_threads_leave();
+	return FALSE;
     }
 
-    g_object_unref(G_OBJECT(message)); 
+    /* set window title */
+    from = internet_address_list_to_string(message->headers->from, FALSE);
+    title = g_strdup_printf(_("Message from %s: %s"), from,
+                            LIBBALSA_MESSAGE_GET_SUBJECT(message));
+    g_free(from);
+    gtk_window_set_title(GTK_WINDOW(mw->window), title);
+    g_free(title);
+
+    if (!balsa_message_set(msg, message)) {
+	gtk_widget_destroy(mw->window);
+	gdk_threads_leave();
+	return FALSE;
+    }
+    balsa_message_grab_focus(BALSA_MESSAGE(mw->bmessage));
+
+    balsa_message_set_close(msg, TRUE);
+    mw_set_part_buttons_sensitive(mw);
 
     /* Update the style and message counts in the mailbox list */
     balsa_mblist_update_mailbox(balsa_app.mblist_tree_store,
@@ -299,19 +396,34 @@ static const struct callback_item {
     { BALSA_PIXMAP_REPLY_ALL,    replytoall_message_cb },
     { BALSA_PIXMAP_REPLY_GROUP,  replytogroup_message_cb },
     { BALSA_PIXMAP_FORWARD,      forward_message_default_cb },
-    { BALSA_PIXMAP_PREVIOUS,     previous_part_cb },
-    { BALSA_PIXMAP_NEXT,         next_part_cb },
+    { BALSA_PIXMAP_PREVIOUS,     previous_message_cb },
+    { BALSA_PIXMAP_PREVIOUS_PART,previous_part_cb },
+    { BALSA_PIXMAP_NEXT,         next_message_cb },
+    { BALSA_PIXMAP_NEXT_PART,    next_part_cb },
     { BALSA_PIXMAP_NEXT_UNREAD,  next_unread_cb },
     { BALSA_PIXMAP_NEXT_FLAGGED, next_flagged_cb },
-    { BALSA_PIXMAP_TRASH,        trash_cb },
-    { BALSA_PIXMAP_PRINT,        print_cb },
-    { BALSA_PIXMAP_SAVE,         save_current_part_cb },
+    { GTK_STOCK_DELETE,           trash_cb },
+    { GTK_STOCK_PRINT,           print_cb },
+    { GTK_STOCK_SAVE,            save_current_part_cb },
     { GTK_STOCK_CLOSE,           close_message_window },
     { BALSA_PIXMAP_SHOW_HEADERS, show_all_headers_tool_cb }
 };
 
 /* Standard buttons; "" means a separator. */
 static const gchar* message_toolbar[] = {
+#if defined(ENABLE_TOUCH_UI)
+    BALSA_PIXMAP_NEXT_UNREAD,
+    "",
+    BALSA_PIXMAP_REPLY,
+    BALSA_PIXMAP_REPLY_ALL,
+    BALSA_PIXMAP_FORWARD,
+    "",
+    GTK_STOCK_PRINT,
+    "",
+    GTK_STOCK_DELETE,
+    "",
+    GTK_STOCK_CLOSE
+#else /* ENABLE_TOUCH_UI */
     BALSA_PIXMAP_NEXT_UNREAD,
     "",
     BALSA_PIXMAP_REPLY,
@@ -319,13 +431,14 @@ static const gchar* message_toolbar[] = {
     BALSA_PIXMAP_REPLY_GROUP,
     BALSA_PIXMAP_FORWARD,
     "",
-    BALSA_PIXMAP_PREVIOUS,
-    BALSA_PIXMAP_NEXT,
-    BALSA_PIXMAP_SAVE,
+    BALSA_PIXMAP_PREVIOUS_PART,
+    BALSA_PIXMAP_NEXT_PART,
+    GTK_STOCK_SAVE,
     "",
-    BALSA_PIXMAP_PRINT,
+    GTK_STOCK_PRINT,
     "",
-    BALSA_PIXMAP_TRASH,
+    GTK_STOCK_DELETE
+#endif /* ENEBLE_TOUCH_UI */
 };
 
 /* Create the toolbar model for the message window's toolbar.
@@ -363,21 +476,34 @@ static void
 mw_set_message(MessageWindow *mw, LibBalsaMessage * message)
 {
     mw->message = message;
-    mw_set_buttons_sensitive(mw);
     g_object_set_data(G_OBJECT(message), BALSA_MESSAGE_WINDOW_KEY, mw);
-    g_object_weak_ref(G_OBJECT(message),
-		      (GWeakNotify) mw_message_weak_notify, mw);
 
     g_object_ref(message); /* protect from destroying */
     mw->idle_handler_id =
         g_idle_add((GSourceFunc) message_window_idle_handler, mw);
+
+    mw_set_buttons_sensitive(mw);
+}
+
+static void
+bindex_closed_cb(gpointer data, GObject *bindex)
+{
+    MessageWindow *mw = data;
+    mw->bindex = NULL;
+    gtk_widget_destroy(mw->window);
+}
+
+static void
+mw_disable_trash(MessageWindow * mw, GtkWidget * toolbar)
+{
+    balsa_toolbar_set_button_sensitive(toolbar, GTK_STOCK_DELETE, FALSE);
+    gtk_widget_set_sensitive(mw->trash, FALSE);
 }
 
 void
 message_window_new(LibBalsaMessage * message)
 {
     MessageWindow *mw;
-    gchar *title;
     BalsaToolbarModel *model;
     GtkWidget *toolbar;
     guint i;
@@ -401,10 +527,7 @@ message_window_new(LibBalsaMessage * message)
 
     mw = g_malloc0(sizeof(MessageWindow));
 
-    title = libbalsa_message_title(message,
-                                   balsa_app.message_title_format);
-    mw->window = gnome_app_new("balsa", title);
-    g_free(title);
+    mw->window = gnome_app_new("balsa", NULL);
 
     mw->headers_shown=balsa_app.shown_headers;
     mw->show_all_headers = FALSE;
@@ -417,8 +540,8 @@ message_window_new(LibBalsaMessage * message)
                                    mw);
     gnome_app_set_toolbar(GNOME_APP(mw->window), GTK_TOOLBAR(toolbar));
 
-    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_PREVIOUS, FALSE);
-    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT, FALSE);
+    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_PREVIOUS_PART, FALSE);
+    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT_PART, FALSE);
 
     gtk_window_set_wmclass(GTK_WINDOW(mw->window), "message", "Balsa");
 
@@ -428,7 +551,7 @@ message_window_new(LibBalsaMessage * message)
                      G_CALLBACK(size_alloc_cb), NULL);
     
     mw->bindex = balsa_find_index_by_mailbox(message->mailbox);
-    g_object_add_weak_pointer(G_OBJECT(mw->bindex), (gpointer) &mw->bindex);
+    g_object_weak_ref(G_OBJECT(mw->bindex), bindex_closed_cb, mw);
     g_signal_connect_swapped(G_OBJECT(mw->bindex), "index-changed",
 			     G_CALLBACK(mw_set_buttons_sensitive), mw);
 
@@ -436,7 +559,13 @@ message_window_new(LibBalsaMessage * message)
 
     /* Save the widgets that we need to change--they'll be overwritten
      * if another message window is opened. */
-    mw->next_unread = message_menu[MENU_MESSAGE_NEXT_UNREAD_POS].widget;
+    mw->next          = message_menu[MENU_MESSAGE_NEXT_POS].widget;
+    mw->previous      = message_menu[MENU_MESSAGE_PREVIOUS_POS].widget;
+    mw->next_unread   = message_menu[MENU_MESSAGE_NEXT_UNREAD_POS].widget;
+    mw->next_flagged  = message_menu[MENU_MESSAGE_NEXT_FLAGGED_POS].widget;
+    mw->next_part     = message_menu[MENU_MESSAGE_NEXT_PART_POS].widget;
+    mw->previous_part = message_menu[MENU_MESSAGE_PREVIOUS_PART_POS].widget;
+    mw->trash =         message_menu[MENU_MESSAGE_TRASH_POS].widget;
 #ifdef HAVE_GTKHTML
     mw->zoom_in  = view_menu[MENU_VIEW_ZOOM_IN].widget;
     mw->zoom_out = view_menu[MENU_VIEW_ZOOM_OUT].widget;
@@ -454,8 +583,12 @@ message_window_new(LibBalsaMessage * message)
                                     G_CALLBACK(mru_menu_cb), mw);
     move_menu = main_menu[MAIN_MENU_MOVE_POS].widget;
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(move_menu), submenu);
-    if(message->mailbox->readonly)
+    if(message->mailbox->readonly) {
 	gtk_widget_set_sensitive(move_menu, FALSE);
+	mw_disable_trash(mw, toolbar);
+    }
+    if (message->mailbox == balsa_app.trash)
+	mw_disable_trash(mw, toolbar);
     mw->bmessage = balsa_message_new();
     
     gnome_app_set_contents(GNOME_APP(mw->window), mw->bmessage);
@@ -482,6 +615,12 @@ message_window_new(LibBalsaMessage * message)
 
     gtk_widget_show_all(mw->window);
     mw_set_message(mw, message);
+    gtk_widget_add_accelerator(file_menu[MENU_FILE_CLOSE_POS].widget,
+			       "activate",
+			       GNOME_APP(mw->window)->accel_group,
+			       GDK_Escape, (GdkModifierType)0, 
+                               (GtkAccelFlags) 0);
+    GTK_WIDGET_UNSET_FLAGS(toolbar, GTK_CAN_FOCUS);
 }
 
 static void
@@ -489,55 +628,32 @@ mw_clear_message(MessageWindow * mw)
 {
     if (mw->idle_handler_id) {
 	g_source_remove(mw->idle_handler_id);
-	if (mw->message)
-	    g_object_unref(mw->message);
 	mw->idle_handler_id = 0;
-    } else if (mw->message) {
+    } 
+    if (mw->message) {
 	g_object_set_data(G_OBJECT(mw->message), BALSA_MESSAGE_WINDOW_KEY,
 			  NULL);
-	g_object_weak_unref(G_OBJECT(mw->message),
-			    (GWeakNotify) mw_message_weak_notify, mw);
+	g_object_unref(mw->message);
+	mw->message = NULL;
     }
-    mw->message = NULL;
 }
 
 /* Handler for the "destroy" signal for mw->window. */
 static void
 destroy_message_window(GtkWidget * widget, MessageWindow * mw)
 {
-    if (mw->bindex)
-	g_signal_handlers_disconnect_matched(G_OBJECT(mw->bindex),
-					     G_SIGNAL_MATCH_DATA, 0, 0,
-					     NULL, NULL, mw);
+    if (mw->bindex) { /* BalsaIndex still exists */
+        g_object_weak_unref(G_OBJECT(mw->bindex), bindex_closed_cb, mw);
+        g_signal_handlers_disconnect_matched(G_OBJECT(mw->bindex),
+                                             G_SIGNAL_MATCH_DATA, 0, 0,
+                                             NULL, NULL, mw);
+    }
 
     mw_clear_message(mw);
 
     g_free(mw);
 }
 
-/* GWeakNotify for mw->message. */
-static gboolean
-mw_destroy_window(MessageWindow * mw)
-{
-    gdk_threads_enter();
-    g_object_unref(mw->window);
-    gtk_widget_destroy(mw->window);
-    gdk_threads_leave();
-
-    return FALSE;
-}
-
-static void
-mw_message_weak_notify(MessageWindow * mw, GObject *obj)
-{
-    mw->message = NULL;
-    g_object_ref(mw->window);
-    /* Destroy the window in an idle handler, because the BalsaMessage
-     * must get its own weak notification about the message before we
-     * destroy the window. */
-    g_idle_add((GSourceFunc) mw_destroy_window, mw);
-}
-
 static void
 replyto_message_cb(GtkWidget * widget, gpointer data)
 {
@@ -611,6 +727,7 @@ next_part_cb(GtkWidget * widget, gpointer data)
     MessageWindow *mw = (MessageWindow *) data;
 
     balsa_message_next_part(BALSA_MESSAGE(mw->bmessage));
+    mw_set_part_buttons_sensitive(mw);
 }
 
 static void
@@ -619,6 +736,7 @@ previous_part_cb(GtkWidget * widget, gpointer data)
     MessageWindow *mw = (MessageWindow *) data;
 
     balsa_message_previous_part(BALSA_MESSAGE(mw->bmessage));
+    mw_set_part_buttons_sensitive(mw);
 }
 
 static void
@@ -661,7 +779,7 @@ show_selected_cb(GtkWidget * widget, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) data;
 
-	mw->headers_shown=HEADERS_SELECTED;
+    mw->headers_shown=HEADERS_SELECTED;
 
     reset_show_all_headers(mw);
     balsa_message_set_displayed_headers(BALSA_MESSAGE(mw->bmessage),
@@ -700,17 +818,36 @@ static void
 mw_set_buttons_sensitive(MessageWindow * mw)
 {
     GtkWidget *toolbar =
-	balsa_toolbar_get_from_gnome_app(GNOME_APP(mw->window));
+        balsa_toolbar_get_from_gnome_app(GNOME_APP(mw->window));
     LibBalsaMailbox *mailbox = mw->message->mailbox;
-    gint other_unread =
-	mailbox->unread_messages - LIBBALSA_MESSAGE_IS_UNREAD(mw->message);
+    BalsaIndex *index = mw->bindex;
+    guint current_msgno = mw->message->msgno;
+    gboolean enable;
 
-    gtk_widget_set_sensitive(mw->next_unread, other_unread > 0);
+    if (!mailbox) {
+        gtk_widget_destroy(mw->window);
+        return;
+    }
+
+    enable = index && balsa_index_next_msgno(index, current_msgno) > 0;
+    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT, enable);
+    gtk_widget_set_sensitive(mw->next, enable);
+
+    enable = index && balsa_index_previous_msgno(index, current_msgno) > 0;
+    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_PREVIOUS,
+                                       enable);
+    gtk_widget_set_sensitive(mw->previous, enable);
+
+    enable = index && index->mailbox_node->mailbox->unread_messages > 0;
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT_UNREAD,
-				       other_unread > 0);
+                                       enable);
+    gtk_widget_set_sensitive(mw->next_unread, enable);
+
+    enable = index
+        && libbalsa_mailbox_total_messages(index->mailbox_node->mailbox) > 0;
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT_FLAGGED,
-				       mailbox
-				       && mailbox->total_messages > 0);
+                                       enable);
+    gtk_widget_set_sensitive(mw->next_flagged, enable);
 }
 
 static void
@@ -730,37 +867,63 @@ select_all_cb(GtkWidget * widget, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) (data);
 
-    libbalsa_window_select_all(GTK_WINDOW(mw->window));
+    balsa_window_select_all(GTK_WINDOW(mw->window));
 }
 
 static void
-mw_set_selected(MessageWindow * mw)
+mw_set_selected(MessageWindow * mw, void (*select_func)(BalsaIndex *))
 {
-    GList *list;
+    LibBalsaMessage *message;
+    MessageWindow *tmp;
 
+    balsa_index_select(mw->bindex, mw->message);
+    select_func(mw->bindex);
+
+    message = mw->bindex->current_message;
+    g_object_ref(message);
+    tmp = g_object_get_data(G_OBJECT(message), BALSA_MESSAGE_WINDOW_KEY);
+    if (tmp) {
+        /*
+         * The message is already displayed in a window, so just use
+         * that one.
+         */
+        g_object_unref(message);
+        gdk_window_raise(tmp->window->window);
+        return;
+    }
+
+    /* Temporarily tell the BalsaMessage not to close when its message
+     * is finalized, so we can safely unref it in mw_clear_message.
+     * We'll restore the usual close-with-message behavior after setting
+     * the new message. */
+    balsa_message_set_close(BALSA_MESSAGE(mw->bmessage), FALSE);
     mw_clear_message(mw);
+    mw_set_message(mw, message);
+    g_object_unref(message);
+}
 
-    list = balsa_index_selected_list(mw->bindex);
-    if (g_list_length(list) == 1)
-	mw_set_message(mw, LIBBALSA_MESSAGE(list->data));
-    g_list_free(list);
+static void
+next_message_cb(GtkWidget * widget, gpointer data)
+{
+    mw_set_selected((MessageWindow *) data, balsa_index_select_next);
+}
+
+static void
+previous_message_cb(GtkWidget * widget, gpointer data)
+{
+    mw_set_selected((MessageWindow *) data, balsa_index_select_previous);
 }
 
 static void
 next_unread_cb(GtkWidget * widget, gpointer data)
 {
-    MessageWindow *mw = (MessageWindow *) (data);
-
-    balsa_index_select_next_unread(mw->bindex);
-    mw_set_selected(mw);
+    mw_set_selected((MessageWindow *) data, balsa_index_select_next_unread);
 }
 
-static void next_flagged_cb(GtkWidget * widget, gpointer data)
+static void
+next_flagged_cb(GtkWidget * widget, gpointer data)
 {
-    MessageWindow *mw = (MessageWindow *) (data);
-
-    balsa_index_select_next_flagged(mw->bindex);
-    mw_set_selected(mw);
+    mw_set_selected((MessageWindow *) data, balsa_index_select_next_flagged);
 }
 
 
@@ -774,7 +937,6 @@ static void print_cb(GtkWidget * widget, gpointer data)
 static void trash_cb(GtkWidget * widget, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) (data);
-
     message_window_move_message(mw, balsa_app.trash);
 }
 
