@@ -21,6 +21,8 @@
 
 #include "config.h"
 
+#define _XOPEN_SOURCE 500
+
 #include <gnome.h>
 #include "balsa-app.h"
 
@@ -33,7 +35,6 @@
 #include <libgnomeprint/gnome-font.h>
 
 #include <libgnomeprintui/gnome-font-dialog.h>
-#ifdef HAVE_LGPRINT22
 #  include <libgnomeprint/gnome-print-job.h>
 #  include <libgnomeprintui/gnome-print-job-preview.h>
 #  define BALSA_GNOME_PRINT_UI GnomePrintJob
@@ -45,19 +46,6 @@
 #  define BALSA_GNOME_PRINT_UI_CLOSE gnome_print_job_close
 #  define BALSA_GNOME_PRINT_UI_PREVIEW_NEW gnome_print_job_preview_new
 #  define BALSA_GNOME_PRINT_UI_PRINT gnome_print_job_print
-#else
-#  include <libgnomeprint/gnome-print-master.h>
-#  include <libgnomeprintui/gnome-print-master-preview.h>
-#  define BALSA_GNOME_PRINT_UI GnomePrintMaster
-#  define BALSA_GNOME_PRINT_UI_GET_CONFIG gnome_print_master_get_config
-#  define BALSA_GNOME_PRINT_UI_GET_PAGE_SIZE_FROM_CONFIG gnome_print_master_get_page_size_from_config
-#  define BALSA_GNOME_PRINT_UI_GET_CONTEXT gnome_print_master_get_context
-#  define BALSA_GNOME_PRINT_DIALOG_NEW gnome_print_dialog_new_from_master
-#  define BALSA_GNOME_PRINT_UI_NEW gnome_print_master_new()
-#  define BALSA_GNOME_PRINT_UI_CLOSE gnome_print_master_close
-#  define BALSA_GNOME_PRINT_UI_PREVIEW_NEW gnome_print_master_preview_new
-#  define BALSA_GNOME_PRINT_UI_PRINT gnome_print_master_print
-#endif
 
 #include <libgnomeprint/gnome-print-paper.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
@@ -73,6 +61,7 @@
 #include "quote-color.h"
 
 #include <string.h>
+#include "i18n.h"
 
 #define BALSA_PRINT_TYPE_HEADER     1
 #define BALSA_PRINT_TYPE_SEPARATOR  2
@@ -80,7 +69,7 @@
 #define BALSA_PRINT_TYPE_IMAGE      4
 #define BALSA_PRINT_TYPE_DEFAULT    5
 #ifdef HAVE_GPGME
-#define BALSA_PRINT_TYPE_GPG_SIGN   6
+#define BALSA_PRINT_TYPE_CRYPT_SIGN   6
 #endif
 #ifdef HAVE_GTKHTML
 # define BALSA_PRINT_TYPE_HTML      7
@@ -135,7 +124,8 @@ struct _CommonInfo {
     GObject *parent_object;
 };
 
-typedef void (*prepare_func_t)(PrintInfo * pi, LibBalsaMessageBody * body);
+typedef void (*prepare_func_t)(PrintInfo * pi, LibBalsaMessageBody * body,
+			       gpointer data);
 
 typedef struct _mime_action_t {
     gchar *mime_type;
@@ -318,7 +308,7 @@ print_header_string(GList **header_list, const gchar *field_id,
 
 static void
 print_header_list(GList **header_list, const gchar *field_id,
-		  const gchar *label, GList *values)
+		  const gchar *label, InternetAddressList * values)
 {
     gchar **hdr_pair;
 
@@ -330,7 +320,7 @@ print_header_list(GList **header_list, const gchar *field_id,
 
     hdr_pair = g_new0(gchar *, 3);
     hdr_pair[0] = g_strdup(label);
-    hdr_pair[1] = libbalsa_make_string_from_list(values);
+    hdr_pair[1] = internet_address_list_to_string(values, FALSE);
     *header_list = g_list_append(*header_list, hdr_pair);
 }
 
@@ -351,7 +341,7 @@ prepare_header_real(PrintInfo * pi, LibBalsaMessageBody * sig_body,
 
     subject = g_strdup(the_subject);
     libbalsa_utf8_sanitize(&subject, balsa_app.convert_unknown_8bit,
-			   balsa_app.convert_unknown_8bit_codeset, NULL);
+			   NULL);
     if (subject) {
 	print_header_string (&pdata->headers, "subject", _("Subject:"),
 			     subject);
@@ -363,7 +353,7 @@ prepare_header_real(PrintInfo * pi, LibBalsaMessageBody * sig_body,
     g_free(date);
 
     if (headers->from) {
-	gchar *from = libbalsa_address_to_gchar(headers->from, 0);
+	gchar *from = internet_address_list_to_string(headers->from, FALSE);
 	print_header_string (&pdata->headers, "from", _("From:"), from);
 	g_free(from);
     }
@@ -374,7 +364,8 @@ prepare_header_real(PrintInfo * pi, LibBalsaMessageBody * sig_body,
     print_header_string (&pdata->headers, "fcc", _("Fcc:"), headers->fcc_url);
 
     if (headers->dispnotify_to) {
-	gchar *mdn_to = libbalsa_address_to_gchar(headers->dispnotify_to, 0);
+	gchar *mdn_to =
+	    internet_address_list_to_string(headers->dispnotify_to, FALSE);
 	print_header_string (&pdata->headers, "disposition-notification-to", 
 			     _("Disposition-Notification-To:"), mdn_to);
 	g_free(mdn_to);
@@ -418,17 +409,25 @@ prepare_header_real(PrintInfo * pi, LibBalsaMessageBody * sig_body,
     }
 
 #ifdef HAVE_GPGME
-    /* add the signature status info if available */
     if (balsa_app.shown_headers != HEADERS_NONE && sig_body &&
-	libbalsa_message_body_protection(sig_body) == 
-	(LIBBALSA_PROTECT_SIGN | LIBBALSA_PROTECT_RFC3156) &&
-	sig_body->parts->next->sig_info) {
-	pdata->sig_status =
-	    g_strdup(libbalsa_gpgme_sig_stat_to_gchar(sig_body->parts->next->sig_info->status));
-	lines += print_wrap_string(&pdata->sig_status, pi->header_font,
-				   pi->printable_width, pi->tab_width);
+        sig_body->parts && sig_body->parts->next &&
+        sig_body->parts->next->sig_info) {
+        gint prot = libbalsa_message_body_protection(sig_body);
+
+        if ((prot & LIBBALSA_PROTECT_SIGN) &&
+            (prot & (LIBBALSA_PROTECT_RFC3156 | LIBBALSA_PROTECT_SMIMEV3))) {
+            GMimeGpgmeSigstat *siginfo = sig_body->parts->next->sig_info;
+
+            pdata->sig_status =
+                g_strconcat
+                (libbalsa_gpgme_sig_protocol_name(siginfo->protocol),
+                 libbalsa_gpgme_sig_stat_to_gchar(siginfo->status),
+                 NULL);
+            lines += print_wrap_string(&pdata->sig_status, pi->header_font,
+                                       pi->printable_width, pi->tab_width);
+        }
     } else {
-	pdata->sig_status = NULL;
+        pdata->sig_status = NULL;
     }
 #endif
 
@@ -465,7 +464,7 @@ prepare_message_header(PrintInfo * pi, LibBalsaMessageBody * body)
 
     /* create the footer */
     libbalsa_utf8_sanitize(&subject, balsa_app.convert_unknown_8bit,
-			   balsa_app.convert_unknown_8bit_codeset, NULL);
+			   NULL);
     if (subject)
 	footer_string = g_string_new(subject);
     g_free(subject);
@@ -480,7 +479,9 @@ prepare_message_header(PrintInfo * pi, LibBalsaMessageBody * body)
     g_free(date);
 
     if (pi->message->headers->from) {
-	gchar *from = libbalsa_address_to_gchar(pi->message->headers->from, 0);
+	gchar *from =
+	    internet_address_list_to_string(pi->message->headers->from,
+		                            FALSE);
 	if (footer_string) {
 	    footer_string = g_string_prepend(footer_string, " - ");
 	    footer_string = g_string_prepend(footer_string, from);
@@ -498,7 +499,7 @@ prepare_message_header(PrintInfo * pi, LibBalsaMessageBody * body)
 }
 
 static void
-prepare_embedded_header(PrintInfo * pi, LibBalsaMessageBody * body)
+prepare_embedded_header(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 {
     HeaderInfo *pdata;
 
@@ -580,7 +581,7 @@ typedef struct _SeparatorInfo {
 } SeparatorInfo;
 
 static void
-prepare_separator(PrintInfo * pi, LibBalsaMessageBody * body)
+prepare_separator(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 {
     SeparatorInfo *pdata;
     gdouble font_size = gnome_font_get_size(pi->header_font);
@@ -628,60 +629,51 @@ typedef struct _HtmlInfo {
     GtkWidget *html;
 } HtmlInfo;
 
-static void prepare_default(PrintInfo * pi, LibBalsaMessageBody * body);
+static void prepare_default(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data);
 
 static void
-prepare_html(PrintInfo * pi, LibBalsaMessageBody * body)
+prepare_html(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 {
     GtkWidget *dialog;
     gint response;
     HtmlInfo *pdata;
-    FILE *fp;
     size_t len;
     gchar *html_text;
+    gchar *conttype;
+    LibBalsaHTMLType html_type;
 
-    if (!libbalsa_html_can_print()) {
-	prepare_default(pi, body);
+    conttype = libbalsa_message_body_get_mime_type(body);
+    html_type = libbalsa_html_type(conttype);
+    g_free(conttype);
+
+    if (!libbalsa_html_can_print() || !html_type) {
+	prepare_default(pi, body, data);
 	return;
     }
 
     dialog =
 	gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
 			       GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-			       _("Processing an HTML message part, "
+			       _("Preparing an HTML part, "
 				 "which must start on a new page.\n"
 				 "Print this part?"));
     response = gtk_dialog_run(GTK_DIALOG(dialog));
     gtk_widget_destroy(dialog);
     if (response != GTK_RESPONSE_YES) {
-	prepare_default(pi, body);
+	prepare_default(pi, body, data);
 	return;
     }
 
-    if (!libbalsa_message_body_save_temporary(body, NULL)) {
-	balsa_information
-	    (LIBBALSA_INFORMATION_ERROR,
-	     _("Error writing to temporary file %s.\n"
-	       "Check the directory permissions."), body->temp_filename);
-	return;
-    }
-
-    if ((fp = fopen(body->temp_filename, "r")) == NULL) {
-	balsa_information(LIBBALSA_INFORMATION_ERROR,
-			  _("Cannot open temporary file %s."),
-			  body->temp_filename);
-	return;
-    }
-
-    len = libbalsa_readfile(fp, &html_text);
-    fclose(fp);
+    len = libbalsa_message_body_get_content(body, &html_text);
     if (!html_text)
 	return;
 
+    len = libbalsa_html_filter(html_type, &html_text, len);
+
     pdata = g_new(HtmlInfo, 1);
     pdata->id_tag = BALSA_PRINT_TYPE_HTML;
     pdata->html =
-	libbalsa_html_new(html_text, len, NULL, pi->message, NULL);
+	libbalsa_html_new(html_text, len, pi->message, NULL);
     g_free(html_text);
 
     if (libbalsa_html_can_zoom(pdata->html)) {
@@ -855,7 +847,7 @@ print_wrap_body(gchar * str, GnomeFont * font, gint width, gint tab_width)
 }
 
 static void
-prepare_plaintext(PrintInfo * pi, LibBalsaMessageBody * body)
+prepare_plaintext(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 {
     PlainTextInfo *pdata;
     gdouble font_size;
@@ -868,17 +860,8 @@ prepare_plaintext(PrintInfo * pi, LibBalsaMessageBody * body)
     /* copy the text body to a buffer */
     if (body->buffer)
 	textbuf = g_strdup(body->buffer);
-    else {
-	FILE *part;
-
-	textbuf = NULL;
-	libbalsa_message_body_save_temporary(body, NULL);
-	part = fopen(body->temp_filename, "r");
-	if (part) {
-	    libbalsa_readfile(part, &textbuf);
-	    fclose(part);
-	    }
-    }
+    else
+	libbalsa_message_body_get_content(body, &textbuf);
 
     /* fake an empty buffer if textbuf is NULL */
     if (!textbuf)
@@ -886,7 +869,7 @@ prepare_plaintext(PrintInfo * pi, LibBalsaMessageBody * body)
 
     /* be sure the we have correct utf-8 stuff here... */
     libbalsa_utf8_sanitize(&textbuf, balsa_app.convert_unknown_8bit,
-			   balsa_app.convert_unknown_8bit_codeset, NULL);
+			   NULL);
     
     /* wrap lines (if necessary) */
     pdata->textlines = 
@@ -964,28 +947,22 @@ typedef struct _DefaultInfo {
 } DefaultInfo;
 
 static void
-prepare_default(PrintInfo * pi, LibBalsaMessageBody * body)
+prepare_default(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 {
     DefaultInfo *pdata;
-    gchar *icon_name, *conttype;
+    gchar *conttype;
     gint hdr = 0, lines;
-    GError* err = NULL;
 
     pdata = g_malloc(sizeof(DefaultInfo));
     pdata->id_tag = BALSA_PRINT_TYPE_DEFAULT;
 
-    if (body->mutt_body)
-	conttype = libbalsa_message_body_get_content_type(body);
-    else
-	conttype = libbalsa_lookup_mime_type(body->filename);
+    conttype = libbalsa_message_body_get_mime_type(body);
 
     /* get a pixbuf according to the mime type */
-    icon_name = libbalsa_icon_finder(conttype, NULL, NULL);
-    pdata->pixbuf = gdk_pixbuf_new_from_file(icon_name, &err);
-    if(err) { g_warning("error loading pixbuf."); g_error_free(err); }
+    pdata->pixbuf = libbalsa_icon_finder(conttype, NULL, NULL,
+					 GTK_ICON_SIZE_DND);
     pdata->image_width = gdk_pixbuf_get_width (pdata->pixbuf);
     pdata->image_height = gdk_pixbuf_get_height (pdata->pixbuf);
-    g_free(icon_name);
 
     /* gather some info about this part */
     pdata->labels = g_new0(gchar *, 5); /* four fields, one terminator */
@@ -1105,23 +1082,22 @@ typedef struct _ImageInfo {
 } ImageInfo;
 
 static void
-prepare_image(PrintInfo * pi, LibBalsaMessageBody * body)
+prepare_image(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 {
     ImageInfo * pdata;
     GError *err = NULL;
     pdata = g_malloc(sizeof(ImageInfo));
     pdata->id_tag = BALSA_PRINT_TYPE_IMAGE;
 
-    libbalsa_message_body_save_temporary(body, NULL);
-    pdata->pixbuf = gdk_pixbuf_new_from_file(body->temp_filename, &err);
+    pdata->pixbuf = libbalsa_message_body_get_pixbuf(body, &err);
     if(err) {
-        g_warning("Error loading image from file.");
+        g_warning("Error loading image from file: %s", err->message);
         g_error_free(err);
     }
     /* fall back to default if the pixbuf could no be loaded */
     if (!pdata->pixbuf) {
 	g_free(pdata);
-	prepare_default(pi, body);
+	prepare_default(pi, body, data);
 	return;
     }
 
@@ -1175,7 +1151,7 @@ print_image(PrintInfo * pi, gpointer * data)
  * ~~~ print a gpg signature info (like plain text, but with header font) ~~~
  */
 static void
-prepare_gpg_signature(PrintInfo * pi, LibBalsaMessageBody * body)
+prepare_crypto_signature(PrintInfo * pi, LibBalsaMessageBody * body, gpointer data)
 {
     PlainTextInfo *pdata;
     gdouble font_size;
@@ -1184,16 +1160,21 @@ prepare_gpg_signature(PrintInfo * pi, LibBalsaMessageBody * body)
 
     /* check if there is a sig_info and prepare as unknown if not */
     if (!body->sig_info) {
-	prepare_default(pi, body);
+	prepare_default(pi, body, data);
 	return;
     }
 
     pdata = g_malloc(sizeof(PlainTextInfo));
-    pdata->id_tag = BALSA_PRINT_TYPE_GPG_SIGN;
+    pdata->id_tag = BALSA_PRINT_TYPE_CRYPT_SIGN;
 
     /* create a buffer with the signature info */
     textbuf =
 	libbalsa_signature_info_to_gchar(body->sig_info, balsa_app.date_string);
+    if (data) {
+	gchar * newbuf = g_strconcat("\n", (gchar *)data, "\n", textbuf, NULL);
+	g_free(textbuf);
+	textbuf = newbuf;
+    }
 
     /* wrap lines (if necessary) */
     pdata->textlines = 
@@ -1222,12 +1203,12 @@ prepare_gpg_signature(PrintInfo * pi, LibBalsaMessageBody * body)
 }
 
 static void
-print_gpg_signature(PrintInfo * pi, gpointer * data)
+print_crypto_signature(PrintInfo * pi, gpointer * data)
 {
     PlainTextInfo *pdata = (PlainTextInfo *)data;
     GList *l;
 
-    g_return_if_fail(pdata->id_tag == BALSA_PRINT_TYPE_GPG_SIGN);
+    g_return_if_fail(pdata->id_tag == BALSA_PRINT_TYPE_CRYPT_SIGN);
 
     gnome_print_setfont(pi->pc, pi->header_font);
     l = pdata->textlines;
@@ -1262,12 +1243,18 @@ scan_body(PrintInfo * pi, LibBalsaMessageBody * body)
 	{"text/html", prepare_default},   /* don't print html source */
 #else /* HAVE_GTKHTML */
 	{"text/html", prepare_html},
+	{"text/enriched", prepare_html},
+	{"text/richtext", prepare_html},
 #endif /* HAVE_GTKHTML */
 	{"text", prepare_plaintext},
 	{"image", prepare_image},
 	{"message/rfc822", prepare_embedded_header},
 #ifdef HAVE_GPGME
-	{"application/pgp-signature", prepare_gpg_signature},
+        {"application/pgp-signature", prepare_crypto_signature},
+#ifdef HAVE_SMIME
+        {"application/pkcs7-signature", prepare_crypto_signature},
+        {"application/x-pkcs7-signature", prepare_crypto_signature},
+#endif
 #endif
 	{NULL, prepare_default}           /* anything else... */
     };
@@ -1276,29 +1263,38 @@ scan_body(PrintInfo * pi, LibBalsaMessageBody * body)
     while (body) {
 	gchar *conttype;
 
-	if (body->buffer)
-	    conttype = g_strdup("text");
-	else
-	    if (!body->mutt_body)
-		conttype = g_strdup("default");
-	    else
-		conttype = libbalsa_message_body_get_content_type(body);
+	conttype = libbalsa_message_body_get_mime_type(body);
 	
 	for (action = mime_actions; 
 	     action->mime_type && 
-		 g_ascii_strncasecmp(action->mime_type, conttype, 
-			             strlen(action->mime_type));
+		 g_ascii_strncasecmp(action->mime_type, conttype,
+				     strlen(action->mime_type));
 	     action++);
-	g_free(conttype);
 
 	if (action->prepare_func) {
-	    prepare_separator(pi, body);
-	    action->prepare_func(pi, body);
+	    prepare_separator(pi, body, NULL);
+	    action->prepare_func(pi, body, NULL);
 	}
 
 	if (body->parts)
 	    scan_body(pi, body->parts);
 
+#ifdef HAVE_GPGME
+	if (body->sig_info &&
+	    g_ascii_strcasecmp(conttype, "application/pgp-signature") &&
+	    g_ascii_strcasecmp(conttype, "application/pkcs7-signature") &&
+	    g_ascii_strcasecmp(conttype, "application/x-pkcs7-signature")) {
+	    gchar * header =
+		g_strdup_printf(_("This is an inline %s signed %s message part:"),
+				body->sig_info->protocol == GPGME_PROTOCOL_OpenPGP ?
+				_("OpenPGP") : _("S/MIME"),
+				conttype);
+	    prepare_crypto_signature(pi, body, header);
+	    g_free(header);
+	}
+#endif
+	g_free(conttype);
+
 	body = body->next;
     }
 }
@@ -1388,7 +1384,9 @@ print_info_new(CommonInfo * ci)
     
     /* now get the message contents... */
     if (!pi->message->mailbox 
-        || libbalsa_message_body_ref(pi->message, TRUE)) {
+        || libbalsa_message_body_ref(pi->message, TRUE,
+                                     balsa_app.shown_headers == HEADERS_ALL||
+                                     balsa_app.show_all_headers)) {
 	prepare_message_header(pi, pi->message->body_list);
         scan_body(pi, pi->message->body_list);
         libbalsa_message_body_unref(pi->message);
@@ -1445,8 +1443,8 @@ print_message(PrintInfo * pi)
 	    print_image(pi, print_task->data);
 	    break;
 #ifdef HAVE_GPGME
-	case BALSA_PRINT_TYPE_GPG_SIGN:
-	    print_gpg_signature(pi, print_task->data);
+	case BALSA_PRINT_TYPE_CRYPT_SIGN:
+	    print_crypto_signature(pi, print_task->data);
 #endif
 	default:
 	    break;
@@ -1487,8 +1485,8 @@ print_message(PrintInfo * pi)
 		print_image(pi, print_task->data);
 		break;
 #ifdef HAVE_GPGME
-	    case BALSA_PRINT_TYPE_GPG_SIGN:
-		print_gpg_signature(pi, print_task->data);
+	    case BALSA_PRINT_TYPE_CRYPT_SIGN:
+		print_crypto_signature(pi, print_task->data);
 		break;
 #endif
 	    default:
@@ -1607,6 +1605,7 @@ font_frame(gchar * title, FontInfo * fi)
 /*
  * creates the print dialog, and adds a page for fonts
  */
+static void common_info_setup(CommonInfo * ci);
 static GtkWidget *
 print_dialog(CommonInfo * ci)
 {
@@ -1621,6 +1620,8 @@ print_dialog(CommonInfo * ci)
 
     dialog = BALSA_GNOME_PRINT_DIALOG_NEW(ci->master, _("Print message"),
 				    GNOME_PRINT_DIALOG_COPIES);
+    ci->dialog = dialog;
+    common_info_setup(ci);
     gtk_window_set_wmclass(GTK_WINDOW(dialog), "print", "Balsa");
     dlgVbox = GTK_DIALOG(dialog)->vbox;
     childList = gtk_container_get_children(GTK_CONTAINER(dlgVbox));
@@ -1667,11 +1668,12 @@ font_info_setup(FontInfo * fi, gchar ** font_name, CommonInfo * ci)
     fi->common_info = ci;
     fi->font = find_font(*font_name);
     if (!fi->font)
-	balsa_information(LIBBALSA_INFORMATION_WARNING,
-			  _("Balsa could not find font \"%s\".\n"
-			    "Use the \"Fonts\" page on the "
-                            "\"Print message\" dialog to change it."),
-                          *fi->font_name);
+	balsa_information_parented
+            (GTK_WINDOW(ci->dialog), LIBBALSA_INFORMATION_WARNING,
+             _("Balsa could not find font \"%s\".\n"
+               "Use the \"Fonts\" page on the "
+               "\"Print message\" dialog to change it."),
+             *fi->font_name);
 }
 
 static void
@@ -1821,7 +1823,6 @@ message_print(LibBalsaMessage * msg, GtkWindow * parent)
                            (GDestroyNotify) common_info_destroy);
     ci->parent_object = parent_object;
 
-    common_info_setup(ci);
     ci->message = msg;
     if (!msg->mailbox) {
         /* temporary message from the compose window */
