@@ -65,19 +65,14 @@ make_gradient(GdkColor colors[], gint first, gint last)
 gint
 is_a_quote(const gchar * str, regex_t * rex)
 {
-    gint cnt = 0, off;
-    regmatch_t rm[1];
+    gint cnt;
 
     g_return_val_if_fail(rex != NULL, 0);
 
     if (str == NULL)
 	return 0;
 
-    off = 0;
-    while (regexec(rex, str + off, 1, rm, 0) == 0) {
-	off += rm[0].rm_eo;
-	cnt++;
-    }
+    libbalsa_match_regex(str, rex, &cnt, NULL);
 
     return cnt;
 }
@@ -91,14 +86,18 @@ is_a_quote(const gchar * str, regex_t * rex)
  */
 void
 allocate_quote_colors(GtkWidget * widget, GdkColor color[],
-		      gint first, gint last)
+                      gint first, gint last)
 {
     gint i;
 
     for (i = first; i <= last; i++) {
-	if (!gdk_colormap_alloc_color
-	    (balsa_app.colormap, &color[i], FALSE,
-	     TRUE)) gdk_color_black(balsa_app.colormap, &color[i]);
+        if (!gdk_colormap_alloc_color(balsa_app.colormap,
+                                      &color[i], FALSE, TRUE)) {
+            /* gdk_color_black(balsa_app.colormap, &color[i]); */
+            color[i].red = color[i].green = color[i].blue = 0;
+            gdk_colormap_alloc_color(balsa_app.colormap, &color[i], FALSE,
+                                     TRUE);
+        }
     }
 }
 
