@@ -37,7 +37,11 @@ extern "C" {
     } SendType;
 
 
-#define VIEW_MENU_LENGTH 10
+#if defined(ENABLE_TOUCH_UI)
+#define VIEW_MENU_LENGTH 4
+#else
+#define VIEW_MENU_LENGTH 5
+#endif
     typedef struct _BalsaSendmsg BalsaSendmsg;
     typedef struct _BalsaSendmsgAddress BalsaSendmsgAddress;
 
@@ -50,11 +54,12 @@ extern "C" {
 
     struct _BalsaSendmsg {
 	GtkWidget *window;
-	GtkWidget *to[3], *from[3], *subject[2], *cc[3], *bcc[3], *fcc[3],
-	    *reply_to[3];
-        BalsaSendmsgAddress to_info, from_info, cc_info, bcc_info,
-            reply_to_info;
-	GtkWidget *comments[2], *keywords[2];
+	GtkWidget *to[3], *from[3], *subject[2], *cc[3], *bcc[3], *fcc[3];
+        BalsaSendmsgAddress to_info, cc_info, bcc_info;
+#if !defined(ENABLE_TOUCH_UI)        
+        GtkWidget *reply_to[3];
+        BalsaSendmsgAddress reply_to_info;
+#endif
 	GtkWidget *attachments[4];
 	GtkWidget *text;
 	GtkWidget *spell_checker;
@@ -79,6 +84,7 @@ extern "C" {
         gulong changed_sig_id;
         gulong delete_range_sig_id;
         guint wrap_timeout_id;
+        gulong identities_changed_id;
 	gboolean modified;
 	gboolean flow;          /* send format=flowed */ 
 	gboolean req_dispnotify; /* send a MDN */ 
@@ -104,7 +110,7 @@ extern "C" {
     void sendmsg_window_set_field(BalsaSendmsg *bsmsg, const gchar* key,
                                   const gchar* val);
 
-    gboolean add_attachment(GnomeIconList * iconlist, char *filename, 
+    gboolean add_attachment(BalsaSendmsg * bsmsg, char *filename, 
                             gboolean is_a_tmp_file, 
                             const gchar *forced_mime_type);
 
