@@ -23,6 +23,7 @@
 
 #include <gnome.h>
 #include <string.h>
+#include "i18n.h"
 
 #include "balsa-app.h"
 #include "store-address.h"
@@ -58,9 +59,11 @@ static void store_address_book_menu_cb(GtkWidget * widget,
                                        StoreAddressInfo * info);
 static void store_address_add_address(StoreAddressInfo * info,
                                       const gchar * label,
-                                      LibBalsaAddress * address);
-static void store_address_add_glist(StoreAddressInfo * info,
-                                    const gchar * label, GList * list);
+                                      const InternetAddress * address,
+                                      const InternetAddress * group);
+static void store_address_add_list(StoreAddressInfo * info,
+                                   const gchar * label,
+				   const InternetAddressList * list);
 
 /* 
  * public interface: balsa_store_address
@@ -112,7 +115,7 @@ balsa_store_address(GList * messages)
 
     g_signal_connect(G_OBJECT(info->dialog), "response",
                      G_CALLBACK(store_address_response), info);
-    gtk_widget_show_all(GTK_WIDGET(info->dialog));
+    gtk_widget_show(GTK_WIDGET(info->dialog));
 }
 
 /* Weak notify that a message was deleted; remove it from our list. */
@@ -183,16 +186,20 @@ store_address_dialog(StoreAddressInfo * info)
                                     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                     NULL);
     GtkWidget *vbox = GTK_DIALOG(dialog)->vbox;
-    GtkWidget *frame;
+    GtkWidget *frame, *label;
 
     frame = store_address_book_frame(info);
+    if(g_list_length(balsa_app.address_book_list)>1)
+        gtk_widget_show_all(frame);
     gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
     frame = store_address_note_frame(info);
+    gtk_widget_show_all(frame);
     gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), 
-                       gtk_label_new(_("Save this address "
-                                       "and close the dialog?")),
+                       label = gtk_label_new(_("Save this address "
+                                               "and close the dialog?")),
                        TRUE, TRUE, 0);
+    gtk_widget_show(label);
     return dialog;
 }
 
@@ -245,11 +252,13 @@ store_address_book_frame(StoreAddressInfo * info)
 {
     GList *ab_list;
     GtkWidget *frame = gtk_frame_new(_("Choose Address Book"));
-    GtkWidget *ab_option, *menu_item, *ab_menu;
+    GtkWidget *combo_box;
     LibBalsaAddressBook *address_book;
     guint default_ab_offset = 0;
 
-    ab_menu = gtk_menu_new();
+    combo_box = gtk_combo_box_new_text();
+    g_signal_connect(combo_box, "changed",
+                     G_CALLBACK(store_address_book_menu_cb), info);
     if (balsa_app.address_book_list) {
 	info->current_address_book = balsa_app.default_address_book;
 
@@ -261,24 +270,18 @@ store_address_book_frame(StoreAddressInfo * info)
 	    if (info->current_address_book == NULL)
 		info->current_address_book = address_book;
 
-	    menu_item = gtk_menu_item_new_with_label(address_book->name);
-	    gtk_menu_shell_append(GTK_MENU_SHELL(ab_menu), menu_item);
-
-	    g_signal_connect(G_OBJECT(menu_item), "activate",
-			     G_CALLBACK(store_address_book_menu_cb), info);
-            g_object_set_data(G_OBJECT(menu_item),"address-book",
-                              GUINT_TO_POINTER(default_ab_offset));
-
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),
+                                      address_book->name);
 	    if (address_book == balsa_app.default_address_book)
-		gtk_menu_set_active(GTK_MENU(ab_menu), default_ab_offset);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),
+                                         default_ab_offset);
 
 	    default_ab_offset++;
 
 	}
     }
-    ab_option = gtk_option_menu_new();
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(ab_option), ab_menu);
-    gtk_container_add(GTK_CONTAINER(frame), ab_option);
+    gtk_container_add(GTK_CONTAINER(frame), combo_box);
+
     return frame;
 }
 
@@ -296,14 +299,13 @@ store_address_note_frame(StoreAddressInfo *info)
     gtk_container_set_border_width(GTK_CONTAINER(info->notebook), 5);
     gtk_container_add(GTK_CONTAINER(frame), info->notebook);
 
-    for (list = info->message_list; list; list = g_list_next(list)) {
+    for (list = info->message_list; list; list = list->next) {
         message = LIBBALSA_MESSAGE(list->data);
 	if (message->headers) {
-	    if (message->headers->from)
-		store_address_add_address(info, _("From:"), message->headers->from);
-	    store_address_add_glist(info, _("To:"), message->headers->to_list);
-	    store_address_add_glist(info, _("Cc:"), message->headers->cc_list);
-	    store_address_add_glist(info, _("Bcc:"), message->headers->bcc_list);
+	    store_address_add_list(info, _("From:"), message->headers->from);
+	    store_address_add_list(info, _("To:"), message->headers->to_list);
+	    store_address_add_list(info, _("Cc:"), message->headers->cc_list);
+	    store_address_add_list(info, _("Bcc:"), message->headers->bcc_list);
 	}
     }
 
@@ -316,9 +318,7 @@ static void
 store_address_book_menu_cb(GtkWidget * widget, 
                            StoreAddressInfo * info)
 {
-    guint i =
-        GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(widget),
-                                           "address-book"));
+    guint i = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
     GList *nth = g_list_nth(balsa_app.address_book_list, i);
     if(nth)
         info->current_address_book = LIBBALSA_ADDRESS_BOOK(nth->data);
@@ -328,20 +328,28 @@ store_address_book_menu_cb(GtkWidget * widget,
  * make a new page in the notebook */
 static void
 store_address_add_address(StoreAddressInfo * info,
-                          const gchar * lab, LibBalsaAddress * address)
+                          const gchar * lab, const InternetAddress * ia,
+			  const InternetAddress * group)
 {
     gchar *text;
+    LibBalsaAddress *address;
     gchar *label_text;
     GtkWidget **entries, *ew;
 
-    if (address == NULL)
+    if (ia == NULL)
         return;
 
     entries = g_new(GtkWidget *, NUM_FIELDS);
     info->entries_list = g_list_append(info->entries_list, entries);
-    ew = libbalsa_address_get_edit_widget(address, entries);
 
-    text = libbalsa_address_to_gchar(address, 0);
+    text = internet_address_to_string(ia, FALSE);
+    address = libbalsa_address_new();
+    address->full_name =
+        g_strdup(ia->name ? ia->name : group ? group->name : NULL);
+    address->address_list = g_list_prepend(NULL, g_strdup(ia->value.addr));
+    ew = libbalsa_address_get_edit_widget(address, entries, NULL, NULL);
+    g_object_unref(address);
+
     label_text = g_strconcat(lab, text, NULL);
     g_free(text);
     if (g_utf8_strlen(label_text, -1) > 10)
@@ -352,15 +360,29 @@ store_address_add_address(StoreAddressInfo * info,
     g_free(label_text);
 }
 
-/* store_address_add_glist:
- * take a GList of addresses and pass them one at a time to
+/* store_address_add_list:
+ * take a list of addresses and pass them one at a time to
  * store_address_add_address */
 static void
-store_address_add_glist(StoreAddressInfo * info,
-                        const gchar * label, GList * list)
+store_address_add_list(StoreAddressInfo * info,
+                       const gchar * label,
+                       const InternetAddressList * list)
 {
-    while (list) {
-        store_address_add_address(info, label, list->data);
-        list = g_list_next(list);
+    for (; list; list = list->next) {
+        InternetAddress *ia = list->address;
+
+        if (ia->type == INTERNET_ADDRESS_NAME)
+            store_address_add_address(info, label, ia, NULL);
+        else if (ia->type == INTERNET_ADDRESS_GROUP) {
+            InternetAddressList *member;
+
+            for (member = ia->value.members; member; member = member->next) {
+                InternetAddress *member_address = member->address;
+
+                if (member_address->type == INTERNET_ADDRESS_NAME)
+                    store_address_add_address(info, label, member_address,
+                                              ia);
+            }
+        }
     }
 }
