@@ -23,26 +23,13 @@
 
 #include <string.h>
 #include <gnome.h>
+#include "i18n.h"
 
-#include "libbalsa.h"
-
-#include "address-book.h"
 #include "balsa-app.h"
 #include "balsa-icons.h"
-#include "balsa-index.h"
-#include "balsa-mblist.h"
-#include "balsa-message.h"
-#include "folder-conf.h"
-#include "mailbox-conf.h"
 #include "main-window.h"
 #include "message-window.h"
-#include "pref-manager.h"
-#include "print.h"
 #include "sendmsg-window.h"
-#include "store-address.h"
-#include "save-restore.h"
-
-#include "libinit_balsa/init_balsa.h"
 
 #include "toolbar-prefs.h"
 #include "toolbar-factory.h"
@@ -75,8 +62,7 @@ struct BalsaToolbarIcon_ {
 };
 
 /* callbacks */
-static void bt_button_toggled(GtkToggleButton * button,
-                              BalsaToolbarIcon * bti);
+static void bt_button_toggled(GtkWidget * button, BalsaToolbarIcon * bti);
 static void bt_destroy(GtkWidget * toolbar, BalsaToolbarData * btd);
 
 /* forward references */
@@ -91,7 +77,7 @@ button_data toolbar_buttons[]={
     {"", N_("Separator"), "", 0},
     {BALSA_PIXMAP_RECEIVE, N_("Check"),
      N_("Check for new email"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_NEW, N_("Compose"),
+    {BALSA_PIXMAP_COMPOSE, N_("Compose"),
      N_("Compose message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_CONTINUE, N_("Continue"),
      N_("Continue message"), TOOLBAR_BUTTON_TYPE_BUTTON},
@@ -111,11 +97,15 @@ button_data toolbar_buttons[]={
      N_("Open next unread message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_NEXT_FLAGGED, N_("Next\nflagged"),
      N_("Open next flagged message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_TRASH, N_("Trash /\nDelete"),
+    {BALSA_PIXMAP_PREVIOUS_PART, N_("Previous\npart"),
+     N_("View previous part of message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_NEXT_PART, N_("Next\npart"),
+     N_("View next part of message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {GTK_STOCK_DELETE, N_("Trash /\nDelete"),
      N_("Move the current message to trash"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_POSTPONE, N_("Postpone"),
      N_("Postpone current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_PRINT, N_("Print"),
+    {GTK_STOCK_PRINT, N_("Print"),
      N_("Print current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_SEND, N_("Send"),
      N_("Send this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
@@ -123,23 +113,23 @@ button_data toolbar_buttons[]={
      N_("Send and Receive messages"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_ATTACHMENT, N_("Attach"),
      N_("Add attachments to this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_SAVE, N_("Save"),
+    {GTK_STOCK_SAVE, N_("Save"),
      N_("Save the current item"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_IDENTITY, N_("Identity"),
      N_("Set identity to use for this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {GTK_STOCK_SPELL_CHECK, N_("Spelling"),
      N_("Run a spell check"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GTK_STOCK_CLOSE, N_("Cancel"), 
-     N_("Cancel this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {GTK_STOCK_CLOSE, N_("Close"), 
+     N_("Close the compose window"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_MARKED_NEW, N_("Toggle\nnew"),
      N_("Toggle new message flag"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_MARKED_ALL, N_("Mark all"),
+    {BALSA_PIXMAP_MARK_ALL, N_("Mark all"),
      N_("Mark all messages in current mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_SHOW_HEADERS, N_("All\nheaders"),
      N_("Show all headers"), TOOLBAR_BUTTON_TYPE_TOGGLE},
     {BALSA_PIXMAP_TRASH_EMPTY, N_("Empty Trash"),
      N_("Delete messages from the Trash mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_CLOSE_MBOX, N_("Close"),
+    {GTK_STOCK_CANCEL, N_("Close"),
      N_("Close current mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_SHOW_PREVIEW, N_("Msg Preview"),
      N_("Show preview pane"), TOOLBAR_BUTTON_TYPE_TOGGLE},
@@ -159,23 +149,27 @@ const int toolbar_button_count =
     sizeof(toolbar_buttons) / sizeof(button_data);
 
 /* Public methods. */
+const gchar *
+balsa_toolbar_sanitize_id(const gchar *id)
+{
+    gint button = get_toolbar_button_index(id);
+
+    if (button >= 0)
+	return toolbar_buttons[button].pixmap_id;
+    else
+	return NULL;
+}
 
 /* this should go to GTK because it modifies its internal structures. */
 void
 balsa_toolbar_remove_all(GtkWidget * widget)
 {
-    GtkToolbar *toolbar = GTK_TOOLBAR(widget);
-    GList *children;
+    GList *child, *children;
     
-    for (children = toolbar->children; children; children = children->next) {
-	GtkToolbarChild *child = children->data;
-	
-	if (child->type != GTK_TOOLBAR_CHILD_SPACE)
-	    gtk_widget_unparent(child->widget);
-	g_free(child);
-    }
-    g_list_free(toolbar->children);
-    toolbar->children = NULL;
+    children = gtk_container_get_children(GTK_CONTAINER(widget));
+    for (child = children; child; child = child->next)
+        gtk_widget_destroy(child->data);
+    g_list_free(children);
 }
 
 /* update_all_toolbars:
@@ -230,9 +224,11 @@ void
 balsa_toolbar_model_insert_icon(BalsaToolbarModel * model, gchar * icon,
                                 gint position)
 {
-    if (get_toolbar_button_index(icon) >= 0)
+    const gchar* real_button = balsa_toolbar_sanitize_id(icon);
+
+    if (real_button)
         *model->current =
-            g_slist_insert(*model->current, g_strdup(icon), position);
+            g_slist_insert(*model->current, g_strdup(real_button), position);
     else
         g_warning(_("Unknown toolbar icon \"%s\""), icon);
 }
@@ -299,10 +295,11 @@ balsa_toolbar_refresh(GtkWidget * toolbar)
         BalsaToolbarIcon *bti;
         gint button;
         gchar *text, *tmp;
-        gint type;
+	GtkToolItem *tool_item;
 
         if (!*icon) {
-            gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
+            gtk_toolbar_insert(GTK_TOOLBAR(toolbar),
+                               gtk_separator_tool_item_new(), -1);
             continue;
         }
 
@@ -325,36 +322,39 @@ balsa_toolbar_refresh(GtkWidget * toolbar)
         }
 
         switch (toolbar_buttons[button].type) {
-        case TOOLBAR_BUTTON_TYPE_RADIO:
-            type = GTK_TOOLBAR_CHILD_RADIOBUTTON;
-            break;
         case TOOLBAR_BUTTON_TYPE_TOGGLE:
-            type = GTK_TOOLBAR_CHILD_TOGGLEBUTTON;
+	    tool_item = gtk_toggle_tool_button_new_from_stock(icon);
+            gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON
+                                              (tool_item), bti->active);
+	    g_signal_connect(G_OBJECT(tool_item), "toggled",
+                             G_CALLBACK(bt_button_toggled), bti);
+	    if (bti->callback)
+		g_signal_connect(G_OBJECT(tool_item), "toggled",
+                                 G_CALLBACK(bti->callback), bti->user_data);
             break;
         case TOOLBAR_BUTTON_TYPE_BUTTON:
         default:
-            type = GTK_TOOLBAR_CHILD_BUTTON;
+	    tool_item = gtk_tool_button_new_from_stock(icon);
+	    if (bti->callback)
+		g_signal_connect(G_OBJECT(tool_item), "clicked",
+                                 G_CALLBACK(bti->callback), bti->user_data);
             break;
         }
-        bti->widget =
-            gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), type, NULL,
-                                       text,
-                                       _(toolbar_buttons[button].help_text),
-                                       _(toolbar_buttons[button].help_text),
-                                       gtk_image_new_from_stock(icon,
-                                           GTK_ICON_SIZE_LARGE_TOOLBAR),
-                                       GTK_SIGNAL_FUNC(bti->callback),
-                                       bti->user_data);
+	gtk_tool_button_set_label(GTK_TOOL_BUTTON(tool_item), text);
+        gtk_tool_item_set_tooltip(tool_item,
+                                  GTK_TOOLBAR(toolbar)->tooltips,
+                                  _(toolbar_buttons[button].help_text),
+                                  _(toolbar_buttons[button].help_text));
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tool_item, -1);
+	bti->widget = GTK_WIDGET(tool_item);
+	g_object_add_weak_pointer(G_OBJECT(bti->widget),
+				  (gpointer) &bti->widget);
+        GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(bti->widget), GTK_CAN_FOCUS);
         g_free(text);
         gtk_widget_set_sensitive(bti->widget, bti->sensitive);
-        if (type == GTK_TOOLBAR_CHILD_TOGGLEBUTTON) {
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bti->widget),
-                                         bti->active);
-            g_signal_connect(G_OBJECT(bti->widget), "toggled",
-                             G_CALLBACK(bt_button_toggled), bti);
-        }
     }
 
+    gtk_widget_show_all(toolbar);
     parent = gtk_widget_get_parent(toolbar);
     if (parent)
         gtk_widget_queue_resize(parent);
@@ -429,17 +429,18 @@ balsa_toolbar_set_button_active(GtkWidget * toolbar, const gchar * icon,
 
     bti->active = active;
     if (bti->widget)
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bti->widget),
-                                     active);
+        gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON
+                                          (bti->widget), active);
 }
 
 /* Signal handlers. */
 
 /* Handler for the "toggled" signal. */
 static void
-bt_button_toggled(GtkToggleButton * button, BalsaToolbarIcon * bti)
+bt_button_toggled(GtkWidget * button, BalsaToolbarIcon * bti)
 {
-    bti->active = gtk_toggle_button_get_active(button);
+    bti->active =
+        gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(button));
 }
 
 /* Handler for the "destroy" signal. */
@@ -456,6 +457,16 @@ bt_destroy(GtkWidget * toolbar, BalsaToolbarData * btd)
 /* Create and initialize a BalsaToolbarData structure with the given
  * BalsaToolbarModel.
  */
+static void
+bt_free_icon(gpointer data)
+{
+    BalsaToolbarIcon *bti = data;
+    if (bti->widget)
+	g_object_remove_weak_pointer(G_OBJECT(bti->widget),
+				     (gpointer) &bti->widget);
+    g_free(bti);
+}
+
 static BalsaToolbarData *
 bt_data_new(BalsaToolbarModel * model)
 {
@@ -463,7 +474,7 @@ bt_data_new(BalsaToolbarModel * model)
 
     btd->model = model;
     btd->table =
-        g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+        g_hash_table_new_full(g_str_hash, g_str_equal, g_free, bt_free_icon);
 
     return btd;
 }
